#include "licensehelper.h"
#include "licensemodel.h"
// #include "../../agent/logencryption.h"

#include <QDebug>
#include <QFile>
#include <QTemporaryFile>
#include <QtDBus>
#include <QDateTime>

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h> /* open */
#include <unistd.h> /* exit */
#include <sys/ioctl.h> /* ioctl */

#include <QJsonDocument>
#include "uihelper.h"
#include <iostream>
#include "easylogging++.h"

namespace  {
constexpr int heartbeatType = 6;
}

LicenseHelper::LicenseHelper(QObject *parent)
    : QObject(parent)
    , m_heartbeatNotify(false)
{
    licenseAgent = new QDBusInterface("com.nfs.license", "/com/nfs/license/Agent", "com.nfs.license.Agent",
                                      QDBusConnection::systemBus());
                                      // QDBusConnection::sessionBus());

    licenseInfo = new QDBusInterface("com.nfs.license", "/com/nfs/license/Info", "com.nfs.license.Info",
                                     QDBusConnection::systemBus());
                                     // QDBusConnection::sessionBus());

    dock = new QDBusInterface("com.nfs.dde.daemon.Dock", "/com/nfs/dde/daemon/Dock", "com.nfs.dde.daemon.Dock",
                              QDBusConnection::sessionBus());

    display = new QDBusInterface("com.nfs.daemon.Display", "/com/nfs/daemon/Display",
                                 "com.nfs.daemon.Display", QDBusConnection::sessionBus());

    QDBusConnection::sessionBus().connect("com.nfs.dde.daemon.Dock",
                                         "/com/nfs/dde/daemon/Dock",
                                         "org.freedesktop.DBus.Properties",
                                         QLatin1String("PropertiesChanged"),
                                         this,
                                         SLOT(dockGeometryChanged()));


    connect(licenseInfo, SIGNAL(showActivator(const QString&, const QString&, const QString&, bool)), this, SIGNAL(showActivator(const QString&, const QString&, const QString&, bool)));

    connect(licenseInfo, SIGNAL(LicenseStateChange(quint32)), this, SLOT(onActiveStateChanged(quint32)));
    connect(licenseInfo, SIGNAL(sendActivateErrorInfo(QString)), this, SLOT(onSendActivateErrorInfo(QString)));

//    connect(licenseInfo, SIGNAL(sendQrcodeUrl(QString)), this, SIGNAL(sendQrcodeUrl(QString)));
//    connect(licenseInfo, SIGNAL(sendActiveMsg(quint32, QString)), this, SIGNAL(activeMsgChange(quint32, QString)));
//    connect(licenseInfo, SIGNAL(heartbeatNotify()), this, SLOT(onHeartbeatNotify()));
//    connect(licenseInfo, SIGNAL(errorMessage(quint32)), this, SIGNAL(errorMsg(quint32)));
//    connect(licenseInfo, SIGNAL(serverReturn(quint32)), this, SIGNAL(serverReturn(quint32)));
//    connect(licenseInfo, SIGNAL(sendServerMsg(quint32, quint32, QVariantMap)), this,
//            SLOT(onSendServerMessage(quint32, quint32, QVariantMap)));
//    connect(licenseInfo, SIGNAL(diagnoseFinishedReturn()), this, SIGNAL(diagnoseFinishedReturn()));
    connect(licenseInfo, SIGNAL(show()), this, SIGNAL(show()));
    connect(licenseInfo, SIGNAL(notify()), this, SIGNAL(notify()));

    // m_previousAuthState = licenseState();
    m_previousAuthorizationState = authorizationState();
}

quint32 LicenseHelper::licenseState()
{
    QVariantMap serverConfMap;
    QList<QVariant> argumentList;
    argumentList << "qy";
    QDBusReply<QByteArray> reply = licenseInfo->callWithArgumentList(QDBus::AutoDetect, "ProductInfo", argumentList);
    if (reply.isValid()) {
        LOG(INFO) << "requestServerConfUrl" << reply.value();
        QJsonDocument docObject = QJsonDocument::fromJson(reply.value());
        serverConfMap = docObject.toVariant().toMap();
    } else {
        LOG(INFO) << "licenseState() D-Bus Error";
    }

    return 1;
//    quint32 activeState = licenseInfo->property("LicenseState").toUInt();
//    qDebug() << "ActiveState:" << activeState;
//    return activeState;
}

QString LicenseHelper::organization()
{
    return licenseInfo->property("Organization").toString();
}

quint32 LicenseHelper::getAuthObjectHidden()
{
    quint32 showAuthHidden = licenseInfo->property("GetAuthObjectHidden").toUInt();
    LOG(INFO) << "GetAuthObjectHidden:" << showAuthHidden;
    return showAuthHidden;
}

QString LicenseHelper::product()
{
    return licenseInfo->property("Product").toString();
}

QString LicenseHelper::copyright()
{
    return licenseInfo->property("Copyright").toString();
}

QString LicenseHelper::machineCode()
{
    return licenseInfo->property("MachineCode").toString();
}

QDateTime LicenseHelper::validityBegin()
{
    QString timeString = licenseInfo->property("ValidityBegin").toString();
    return QDateTime::fromString(timeString, Qt::ISODate);
}

QString LicenseHelper::validityEnd()
{
    QString timeString = licenseInfo->property("ValidityEnd").toString();
    // qInfo() << "validityEnd:" << LogEncryption::encryptionMessage(timeString);
    return timeString;
}

QString LicenseHelper::validityTrial()
{
    QString validityTrialString = licenseInfo->property("ValidityTrial").toString();
    // qInfo() << "validityTrial:" << LogEncryption::encryptionMessage(validityTrialString);
    return validityTrialString;
}

quint32 LicenseHelper::authorizationState()
{
    quint32 authState = licenseInfo->property("AuthorizationState").toUInt();
    LOG(INFO) << "AuthorizationState:" << authState;
    return authState;
}

quint32 LicenseHelper::activeResetState()
{
    quint32 authState = licenseInfo->property("ActiveResetState").toUInt();
    LOG(INFO) << "ActiveResetState:" << authState;
    return authState;
}

quint32 LicenseHelper::activeMethod()
{
    return licenseInfo->property("ActiveMethod").toUInt();
}

quint32 LicenseHelper::activeMsgType()
{
    return licenseInfo->property("ActiveMsgType").toUInt();
}

QString LicenseHelper::activeCode()
{
    return licenseInfo->property("ActiveCode").toString();
}

QString LicenseHelper::authorizeMode()
{
    return licenseInfo->property("AuthorizeMode").toString();
}

int LicenseHelper::trialRemind()
{
    return licenseAgent->property("TrialRemind").toInt();
}

int LicenseHelper::systemType()
{
    return licenseInfo->property("SystemType").toInt();
}

void LicenseHelper::onSendActivateErrorInfo(QString error_info) {
    LOG(INFO) << "SIGNAL---ActiveStateChanged";

    emit activateError(error_info);
}

void LicenseHelper::onActiveStateChanged(quint32 operationMessage)
{
//    if (!licenseStateIsChange() && m_heartbeatNotify && operationMessage == heartbeatType) {
//        m_heartbeatNotify = false;
//        return;
//    }
//
    LOG(INFO) << "SIGNAL---ActiveStateChanged";
    LOG(INFO) << "SIGNAL---ActiveStateChanged(onActiveStateChanged)";
    Q_EMIT activeStateChanged();
}

void LicenseHelper::onHeartbeatNotify()
{
    m_heartbeatNotify = true;
    emit heartbeatNotify();
}

void LicenseHelper::onSendServerMessage(quint32 showScene, quint32 code, QVariantMap errData)
{
    emit sendServerMessage(showScene, static_cast<ServerMessageType>(code), errData);
}

void LicenseHelper::dockGeometryChanged()
{
    qint32 displayWidth = display->property("ScreenWidth").toInt();
    qint32 displayHeight = display->property("ScreenHeight").toInt();
    qint32 dockPosition = dock->property("Position").toInt();
    qint32 dockHeight = dock->property("WindowSizeEfficient").toInt();
    qint32 dockHideState = dock->property("HideState").toInt();

    //dockHideState隐藏时状态值为2，显示时状态值为1
    if ((dockPosition == DockPosition::Top || dockPosition == DockPosition::Bottom) && (dockHideState == 1))
        displayHeight = displayHeight - dockHeight;
    else if ((dockPosition == DockPosition::Left || dockPosition == DockPosition::Right) && (dockHideState == 1))
        displayWidth = displayWidth - dockHeight;
    else
        dockHeight = 0;

    LOG(INFO) << "dockPosition, dockHeight, displayHeight, displayWidth"
            << dockPosition << dockHeight << displayWidth << displayHeight;
    Q_EMIT screenCurrentDpi(dockPosition, dockHeight, displayWidth, displayHeight);
}

bool LicenseHelper::licenseStateIsChange()
{
    // quint32 authState = licenseState();
    quint32 authorization = authorizationState();
//    if ((authState != Activated && authorization != Authorized)
//            && (m_previousAuthState == Activated && m_previousAuthorizationState == Authorized)) {
//        m_previousAuthState = authState;
//        m_previousAuthorizationState = authorization;
//        return true;
//    }

    // m_previousAuthState = authState;
    m_previousAuthorizationState = authorization;
    return false;
}

//-----------------------------------------------
void LicenseHelper::importOnlineSerialA(const QString &online_url, const QString& serial_a, const QString& label) {
    QList<QVariant> argumentList;
    argumentList << online_url;
    argumentList << serial_a;
    argumentList << label;

     LOG(INFO) << online_url.toStdString() << serial_a.toStdString();
    QDBusMessage reply = licenseAgent->callWithArgumentList(QDBus::NoBlock, "ImportOnlineSerialA", argumentList);

    if (reply.type() == QDBusMessage::ErrorMessage) {
        LOG(INFO) << "D-Bus Error:" << reply.errorMessage();
    }
}

void LicenseHelper::importInlineURL(const QString &inline_url, const QString &type) {
    m_heartbeatNotify = false;
    QList<QVariant> argumentList;
    argumentList << inline_url;
    argumentList << type;//CommonMethods::getProductLabel();
    QDBusMessage reply = licenseAgent->callWithArgumentList(QDBus::NoBlock, "ImportInlineURL", argumentList);

    LOG(INFO) << "inline_url: " << inline_url.toStdString();
    LOG(INFO) << "label: " << CommonMethods::getProductLabel().toStdString();
    if (reply.type() == QDBusMessage::ErrorMessage) {
        LOG(INFO) << "D-Bus Error:" << reply.errorMessage();
    }
}

void LicenseHelper::importSerialB(const QString &serial_b, const QString &product_name) {
    QList<QVariant> argumentList;
    argumentList << serial_b;
    argumentList << product_name;
    QDBusMessage reply = licenseAgent->callWithArgumentList(QDBus::NoBlock, "ImportSerialB", argumentList);

    LOG(INFO) << "serial_b:" << serial_b.toStdString();

    if (reply.type() == QDBusMessage::ErrorMessage) {
        LOG(INFO) << "D-Bus Error:" << reply.errorMessage();
        std::cout << "D-Bus Error:" << reply.errorMessage().toStdString();
    }
}

void LicenseHelper::importSerialB(const QString &serial_b, const QString &product_name, const QString &serial_a)
{
    QList<QVariant> argumentList;
    argumentList << serial_b;
    argumentList << product_name;
    argumentList << serial_a;
    QDBusMessage reply = licenseAgent->callWithArgumentList(QDBus::NoBlock, "ImportSerialB", argumentList);

    LOG(INFO) << "serial_b:" << serial_b.toStdString();

    if (reply.type() == QDBusMessage::ErrorMessage) {
        LOG(INFO) << "D-Bus Error:" << reply.errorMessage();
        std::cout << "D-Bus Error:" << reply.errorMessage().toStdString();
    }
}

void LicenseHelper::importStartProductD(const QString &url, const QString &label)
{
    QList<QVariant> argumentList;
    argumentList << url;
    argumentList << label;
    QDBusMessage reply = licenseAgent->callWithArgumentList(QDBus::NoBlock, "ImportStartProductD", argumentList);

    LOG(INFO) << "url:" << url << "\t" << "label:" << label;

    if (reply.type() == QDBusMessage::ErrorMessage) {
        LOG(INFO) << "D-Bus Error:" << reply.errorMessage();
        std::cout << "D-Bus Error:" << reply.errorMessage().toStdString();
    }
}

QVariantMap LicenseHelper::requestLicenseInfo(const QString &type) {
    QVariantMap serverConfMap;
    QList<QVariant> argumentList;
    argumentList << type;
    QDBusReply<QByteArray> reply = licenseInfo->callWithArgumentList(QDBus::AutoDetect, "ProductInfo", argumentList);
    if (reply.isValid()) {
        LOG(INFO) << "requestServerConfUrl" << reply.value();
        QJsonDocument docObject = QJsonDocument::fromJson(reply.value());
        serverConfMap = docObject.toVariant().toMap();
    } else {
        LOG(INFO) << "requestLicenseInfo D-Bus Error";
    }
    LOG(INFO) << "requestLicenseInfo: get ProductInfo";
    return serverConfMap;
}

bool LicenseHelper::get_network_status(const QString &url)
{
    bool status = false;
    QList<QVariant> argumentList;
    argumentList << url;
    QDBusReply<bool> reply = licenseInfo->callWithArgumentList(QDBus::AutoDetect, "check_net_work", argumentList);
    if(reply.isValid())
    {
        LOG(INFO) << "get_network_status" << reply.value();
        status = reply.value();
    }
    else
    {
        LOG(INFO) << "get_network_status D-Bus Error";
    }

    return status;
}

//-----------------------------------------------

void LicenseHelper::importSerialKey(const QString &key, const quint32 &networkStatus, const qint32 &isActiveFile)
{
    m_heartbeatNotify = false;
    QList<QVariant> argumentList;
    argumentList << key;
    argumentList << networkStatus;
    argumentList << isActiveFile;
    QDBusMessage reply = licenseAgent->callWithArgumentList(QDBus::NoBlock, "ImportSerialKey", argumentList);

    if (reply.type() == QDBusMessage::ErrorMessage) {
        LOG(INFO) << "D-Bus Error:" << reply.errorMessage();
    }
}

void LicenseHelper::importOfflineSerialKey(const QString &offlinekey, const QString &activationKey)
{
    m_heartbeatNotify = false;
    QList<QVariant> argumentList;
    argumentList << offlinekey;
    argumentList << activationKey;
    QDBusMessage reply = licenseAgent->callWithArgumentList(QDBus::AutoDetect, "ImportOffLineSerialKey", argumentList);

    if (reply.type() == QDBusMessage::ErrorMessage) {
        LOG(INFO) << "D-Bus Error:" << reply.errorMessage();
    }
}

void LicenseHelper::importVolumeSerialKey(const QString &licensePath, const quint32 &networkStatus)
{
    m_heartbeatNotify = false;
    QList<QVariant> argumentList;
    argumentList << licensePath << networkStatus;
    QDBusMessage reply = licenseAgent->callWithArgumentList(QDBus::NoBlock, "ImportVolumeSerialKey", argumentList);

    if (reply.type() == QDBusMessage::ErrorMessage) {
        LOG(INFO) << "D-Bus Error:" << reply.errorMessage();
    }
}

void LicenseHelper::importCustomServerUrl(const QString &customServerUrl)
{
    m_heartbeatNotify = false;
    QList<QVariant> argumentList;
    argumentList << customServerUrl;
    QDBusMessage reply = licenseAgent->callWithArgumentList(QDBus::Block, "ImportCustomServerUrl", argumentList);

    if (reply.type() == QDBusMessage::ErrorMessage) {
        emit errorMsg(ErrorMsgType::SetServerUrlFail);
        LOG(INFO) << "D-Bus Error:" << reply.errorMessage();
    }
}

void LicenseHelper::importDefaultServerUrl()
{
    m_heartbeatNotify = false;

    QDBusMessage reply = licenseAgent->call(QDBus::Block, "ImportDefaultServerUrl");

    if (reply.type() == QDBusMessage::ErrorMessage) {
        emit errorMsg(ErrorMsgType::SetServerUrlFail);
        LOG(INFO) << "D-Bus Error:" << reply.errorMessage();
    }
}

void LicenseHelper::replaceSerialKey(const QString &repalcedSerialKey, const quint32 &networkStatus)
{
    m_heartbeatNotify = false;
    QList<QVariant> argumentList;
    argumentList << repalcedSerialKey;
    argumentList << networkStatus;
    QDBusMessage reply = licenseAgent->callWithArgumentList(QDBus::NoBlock, "RequestReplaceSerialKey", argumentList);

    if (reply.type() == QDBusMessage::ErrorMessage)
        LOG(INFO) << "D-Bus Error:" << reply.errorMessage();
}

void LicenseHelper::tryFullVersion(const quint32 &networkStatus)
{
    m_heartbeatNotify = false;
    QList<QVariant> argumentList;
    argumentList << networkStatus;
    QDBusMessage reply = licenseAgent->callWithArgumentList(QDBus::AutoDetect, "requestTrialActive", argumentList);
    if (reply.type() == QDBusMessage::ErrorMessage) {
        LOG(INFO) << "D-Bus Error:" << reply.errorMessage();
    }
}

QVariantMap LicenseHelper::requestServerConfUrl()
{
    QVariantMap serverConfMap;
    QDBusReply<QByteArray> reply = licenseAgent->call(QDBus::AutoDetect, "requestServerConfUrl");
    if (reply.isValid()) {
        LOG(INFO) << "requestServerConfUrl" << reply.value();
        QJsonDocument docObject = QJsonDocument::fromJson(reply.value());
        serverConfMap = docObject.toVariant().toMap();
    } else {
        LOG(INFO) << "requestServerConfUrl D-Bus Error";
    }

    return serverConfMap;
}

void LicenseHelper::deactive()
{
    QList<QVariant> argumentList;
    QDBusMessage reply = licenseAgent->callWithArgumentList(QDBus::AutoDetect, "Deactive", argumentList);
    if (reply.type() == QDBusMessage::ErrorMessage) {
        LOG(INFO) << "D-Bus Error:" << reply.errorMessage();
    }
}

void LicenseHelper::quit()
{
    QList<QVariant> argumentList;
    QDBusMessage reply = licenseAgent->callWithArgumentList(QDBus::AutoDetect, "Quit", argumentList);
    if (reply.type() == QDBusMessage::ErrorMessage) {
        LOG(INFO) << "D-Bus Error:" << reply.errorMessage();
    }
}

bool LicenseHelper::exportInformation(const QString &infoFilename)
{
    QFile infoFile(infoFilename);
    if (!infoFile.open(QIODevice::Truncate | QIODevice::ReadWrite)) {
        LOG(INFO) << infoFilename << "file open fail !!!";
        return false;
    }

    QVariantMap info;
    info["machine_code"] = machineCode();
    QJsonDocument josnDoc = QJsonDocument::fromVariant(info);
    infoFile.write(josnDoc.toJson());
    infoFile.close();
    return true;
}

void LicenseHelper::requestDiagnostics(const quint32 networkStatus)
{
    m_heartbeatNotify = false;
    QList<QVariant> argumentList;
    argumentList << networkStatus;
    QDBusMessage reply = licenseAgent->callWithArgumentList(QDBus::AutoDetect, "DiagnosticsAcitve", argumentList);

    if (reply.type() == QDBusMessage::ErrorMessage) {
        LOG(INFO) << "D-Bus Error:" << reply.errorMessage();
    }
}
