#ifndef LICENSEHELPER_H
#define LICENSEHELPER_H

#include "licensemodel.h"

#include <QObject>
#include <QVariantMap>

class QDateTime;
class QDBusInterface;
class LicenseHelper : public QObject
{
    Q_OBJECT

public:
    explicit LicenseHelper(QObject *parent = Q_NULLPTR);
    // --------------------------------
    void importOnlineSerialA(const QString &online_url, const QString& serial_a, const QString& label);
    void importInlineURL(const QString &inline_url, const QString &type);
    void importSerialB(const QString& serial_b, const QString& product_name);
    void importSerialB(const QString &serial_b, const QString &product_name, const QString &serial_a);
    void importStartProductD(const QString &url, const QString &label);

    QVariantMap requestLicenseInfo(const QString &type);
    bool get_network_status(const QString &url);
    // ---------------------------------

    void importSerialKey(const QString &key, const quint32 &networkStatus, const qint32 &isActiveFile = 0);
    void importOfflineSerialKey(const QString &offlinekey, const QString &activationKey);
    void importVolumeSerialKey(const QString &licensePath, const quint32 &networkStatus);
    void importCustomServerUrl(const QString &customServerUrl);
    void importDefaultServerUrl();
    void replaceSerialKey(const QString &repalcedSerialKey, const quint32 &networkStatus);
    //试用期激活
    void tryFullVersion(const quint32 &networkStatus);
    //请求ServerUrl
    QVariantMap requestServerConfUrl();
    void requestDiagnostics(const quint32 networkStatus);

    void deactive();
    void quit();
    bool exportInformation(const QString &infoFile);
    quint32 licenseState();
    QString organization();
    quint32 getAuthObjectHidden();
    QString product();
    QString copyright();
    QString machineCode();
    QDateTime validityBegin();
    QString validityEnd();
    QString validityTrial();
    int trialRemind();
    int systemType();
    quint32 authorizationState();
    quint32 activeResetState();
    quint32 activeMethod();
    quint32 activeMsgType();
    QString activeCode();
    QString authorizeMode();

public slots:
    //
    void onSendActivateErrorInfo(QString error_info);
    //
    void onActiveStateChanged(quint32 operationMessage);
    void onHeartbeatNotify();
    void onSendServerMessage(quint32 showScene, quint32 code, QVariantMap errData);
    void dockGeometryChanged();

signals:
    //
    void activateError(const QString &error_info);
    void showActivator(const QString &label, const QString &product_name, const QString &version, bool showProbationPeriod);
    //

    void activeStateChanged();
    void sendQrcodeUrl(const QString &qrcodeUrl);
    void activeMsgChange(const quint32 &message, const QString &startTime);
    void heartbeatNotify();
    void errorMsg(const quint32 &message);
    void serverReturn(const quint32 &serverStatus);
    void sendServerMessage(quint32 showScene, ServerMessageType code, QVariantMap errData);
    void diagnoseFinishedReturn();
    void show();
    void notify();
    void screenCurrentDpi(qint32 dockPosition, qint32 dockHeight, qint32 displayWidth, qint32 displayHeight);

private:
    bool licenseStateIsChange();

private:
    QDBusInterface *licenseAgent = Q_NULLPTR;
    QDBusInterface *licenseInfo = Q_NULLPTR;
    QDBusInterface *display = Q_NULLPTR;
    QDBusInterface *dock = Q_NULLPTR;
    quint32 m_previousAuthState = Unactivated;
    quint32 m_previousAuthorizationState = Notauthorized;
    bool m_heartbeatNotify;
};

#endif // LICENSECHECKER_H
