#ifndef LICENSEMODEL_H
#define LICENSEMODEL_H

#include <QObject>

#define KMS_CONFIG "/usr/share/kms/kmsserver.conf"
namespace {
const qint32 stayTime = 3000;
const quint32 notifyId = 0;
const QString notificationsService = "org.freedesktop.Notifications";
const QString notificationsPath = "/org/freedesktop/Notifications";
const QString notificationsInterface = "org.freedesktop.Notifications";
const QString notificationsMethod = "Notify";
//默认的更换次数为３次
constexpr quint32 replaceTimes = 3;

constexpr qint32 buttonWidth = 372;
constexpr qint32 buttonHeight = 36;
}

const QString confDefaultName = "confDefault";
const QString confDefaultIndexName = "confDefaultIndex";
const QString confCustomHeadindexName = "confCustomHeadindex";
const QString confCustomUrlName = "confCustomUrl";
const QString defaultUrlName = "defaultUrl";
const QString confOptions = "options";
const QString confProtocolName = "protocol";
const QString confCustomServerName = "address";
const QString openKeyType = "OPEN";

enum SystemType {
    Professional,
    Server,
    Personal,
    DesktopDevice,
    ServerDevice
};

enum Operation {
    NoOperation,
    SerialLicense,
    TrialLicense,
    OfflineLicense,
    FileLicense,
    SetUrl,
    VolumeActiveFile,
    CheckDevice,
    ReplaceSerialKey
};

enum ActiveState {
    Activated = 1,
    Unactivated
};

enum CheckNetworkState {
    NetworkNormal,
    NetworkConnectionTimeout,
    NoNetwork
};

enum ActiveResetState {
    NullResetState = 0,
    IsResetState,
    NotResetState
};

enum NetworkState {
    //物理网络正常，服务器正常 0
    NetOnLine,
    //物理网络无 1
    NetOffLine,
    //物理网络正常，服务器无 2
    NetPhysical,
    //初始值
    InitNetworkState = 30
};

enum AuthorizationState {
    Notauthorized = 0,
    Authorized,
    Expired,
    //试用期已授权
    TrialAuthorized,
    //试用期已过期
    TrialExpired
};

enum ActiveMethod {
    ActiveCode = 0,
    KMSProductionLine,
    OEM,
    VolumeActive,
    H5OfflineActivate,
    KmsApplication,
    ContractImport,
    TryFullVersionActivate,
    ExperienceActivate,
    OffLineTrialActive
};

enum ActiveMessageType {
    Successed,
    //激活码、激活文件、离线激活码无效
    ActiveCodeUnvalid,
    //激活码错误
    ActiveCodeError,
    //激活码已使用
    ActiveCodeUsed,
    //网络连接已断开(试用期激活)
    ActiveNetOffLine,
    //服务器连接失败
    ActiveUnconnect,
    //验证签名失败
    ActiveUnknow,
    //批量激活文件无效
    VolumeActiveFileUnvalid,
    //批量激活文件已超过使用次数
    VolumeActiveFileOverUsed,
    //激活码未到激活时间
    FutureActivationCode,
    //激活码已过期
    ActiveCodeExpired = 10,
    //批量激活码已过期(11已被标识为离线激活的非首次扫码)
    VolumeActiveCodeExpired = 12,
    //批量离线激活记录的激活时间有误
    VolumeActiveTimeUnvalid,
    //激活时间不在激活码期限内或当前时间不在激活文件有效期内
    VolumeActiveUnvalid,
    //系统文件被篡改
    SystemFilesUnvalid,

    //agent中内部使用的错误类型
    //激活时间不在激活码期限内
    ActiveTimeUnvalid = 100,
    //当前时间不在激活文件有效期内
    VolumeActiveFileTimeUnvalid,
    //批量激活状态改变
    VolumeActiveStateChange,
    //激活码，激活文件版本错误
    ActiveVersionError,
    //诊断完成
    DiagnoseFinished,
    //激活失败，请检查服务器设置
    ActiveServerUnmatch,
    //激活服务器连接异常，暂无法更换序列号
    ReplaceUnconnect,
    //HW_KLU中缺失cid不允许进行激活操作
    HWKLUSystemLostCID
};

enum ServerMessageType {
    //服务器返回错误类型
    SignatureVerificationFailed = 30000,    //验签失败
    InvalidECCVersion,      //无效的ecc版本
    InvalidUuid,            //无效的机器码
    InvalidServerSum,       //服务端文件checksum校验出错
    InvalidClientSum,       //客户端文件checksum校验出错
    InvalidHardwareInfo,    //无效的硬件信息
    InternalServerError = 30500,    //系统错误
    CodeDoesNotExist = 31001,       //激活码查询错误, 激活码不存在
    CodeCorrespondingTimeError,     //激活时间早于激活码有效期开始时间
    CodeExpired,            //激活码过期
    InvalidActivationFile,  //激活文件无效
    CodeHasBeenUsed,        //激活码已使用
    VolumeCodeInsufficient = 31007, //离线激活码数量不足
    VolumeCodeExpired,      //离线激活码过期
    MachineIdHasBeenActivatedTrial, //machineID 已被激活(试用期)
    InvalidCode,            //激活码参数格式错误
    BatchActivationFileExpired,     //激活文件已过期
    MachineIdHasBeenActivated,      //machineID 已被激活
    ExpiredActivationFile,          //激活文件已过期
    UsedActivationFile,             //激活文件已使用
    InexistenceOrder = 31018,       //批量离线激活订单不存在
    GetSerialNumberFailed,          //获取序列号失败
    InsertActivateCodeError,        //激活失败
    UpdateOfflineCodeError,          //更新离线激活码数量失败
    AuthorizationTypeError = 31030,    //授权类型错误
    CodeReplaceNumError,               //序列号更换次数已达上限
    AuthorizationHasExpired,           //授权已过期
    SystemUnActive,                    //系统未正式激活
    KMSNotImport                       //kms量产数据未导入后台服务器
};

enum ErrorMsgType {
    Unknown = 0,
    //网络连接超时
    ConnectionTimeout,
    //设置服务器失败
    SetServerUrlFail
};

enum VerifyMethod {
    Url,
    AvtiveCode,
    MaxInput
};

struct System {
    SystemType systemType;
    int trialRemind;
    QString localeEdition;
    QString machineCode;
};

enum CheckShowAuthObject {
    AuthObjectHidden = 1, //隐藏授权对象
    AuthObjectShowed = 2 //显示授权对象
};

struct License {
    QString product_name;
    int state;
    QString mac;
    QString serial_b;
    QString valid_date;
    QString product_label;
    QString product_seq;
    QString disk;
    QString sn;
    //QString active_ident;

//    QString licenseObject;
//    quint32 authObjectHidden;
//    QString product;
//    QString machineCode;
//    QString edition;
//    //剩余天数（试用期）
//    QString validityTrial;
//    //可用日期
//    QString validityEnd;
//    quint32 actived;
//    QString activeOverAction;
//    QString activeSign;
//    QString activeObject;
//    //已授权 未授权 已过期 试用期 试用期过期
//    quint32 authorizationState;
//    quint32 activeMethod;
//    quint32 activeMsgType;
//    QString activeCode;
//    QString authorizeMode;
//    quint32 activeResetState;
};

struct LicenseModel {
    System system;
    License license;
};

enum UrlType {
    Https,
    Http,
    Kms
};

enum ErrorMessageDisplay {
    DoNotShow,
    InApplication,
    Bubble
};

enum DockPosition {
    Top,
    Right,
    Bottom,
    Left
};

Q_DECLARE_METATYPE(LicenseModel)

#endif // LICENSEMODEL_H
