#include "licensemodel.h"
#include "networkconnectivitycheck.h"
#include "easylogging++.h"

#include <QScopedPointer>
#include <QTimer>
#include <QNetworkInterface>

NetworkConnectivityCheck::NetworkConnectivityCheck(QObject *parent)
    : QObject(parent)
{
    connect(&m_networkManager, &QNetworkAccessManager::finished, this,
            &NetworkConnectivityCheck::quitEventLoop);
}

NetworkConnectivityCheck::~NetworkConnectivityCheck()
{
}

int NetworkConnectivityCheck::getNetworkStatus(QString url)
{
    m_isEnter = true;
    QNetworkReply *reply = m_networkManager.get(QNetworkRequest(QUrl(url)));

    QTimer checkNetwork;
    checkNetwork.setSingleShot(true);
    bool isResult = false;
    int connectStatus = -1;
    checkNetwork.start(500);
    connect(&checkNetwork, &QTimer::timeout, [&]() {
        if (!isResult) {
            checkNetwork.stop();
            LOG(INFO) << "check network timeout 500ms.";
            quitEventLoop();
            connectStatus = CheckNetworkState::NetworkConnectionTimeout;
        }
    });

    m_eventLoop.exec();

    if (CheckNetworkState::NetworkConnectionTimeout != connectStatus) {
        checkNetwork.stop();
        reply->close();

        if (reply->error() == QNetworkReply::NoError
            && reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt() == 200) {
            LOG(INFO) << "Connected to url:" << url;

            if (connectStatus == -1)
                connectStatus = CheckNetworkState::NetworkNormal;
        } else {
            LOG(INFO) << "Failed to connect url:" << url;

            if (connectStatus == -1)
                connectStatus = CheckNetworkState::NoNetwork;
        }
        isResult = true;
    }

    if (reply) {
        reply->deleteLater();
        reply = Q_NULLPTR;
    }

    m_isEnter = false;
    return connectStatus;
}

bool NetworkConnectivityCheck::getNetWorkInterfaceStatus()
{
    QList<QNetworkInterface> networkList = QNetworkInterface::allInterfaces();
    bool isConnect = false;

    for (auto interface : networkList) {
        if (interface.isValid()) {
            //返回与此网络接口关联的标志。
            QNetworkInterface::InterfaceFlags flags = interface.flags();
            LOG(INFO) << "flags: " << flags;
            LOG(INFO) << "name: " << interface.name();

            if (flags.testFlag(QNetworkInterface::IsUp) && flags.testFlag(QNetworkInterface::IsRunning)
                && !flags.testFlag(QNetworkInterface::IsLoopBack)) {
                isConnect = true;
                LOG(INFO) << "get local network status is:" << isConnect;
            }
        }
    }

    return isConnect;
}

bool NetworkConnectivityCheck::getIsEnter()
{
    return m_isEnter;
}

void NetworkConnectivityCheck::quitEventLoop()
{
    if (m_eventLoop.isRunning())
        m_eventLoop.quit();
}
