#include "activatorcontrol.h"
#include "activatorwindow.h"
#include "licensehelper.h"
#include "licensetrayicon.h"
#include "widgets/status_widget.h"
// #include "../../agent/logencryption.h"

// #include <DWidgetUtil>
// #include <DHiDPIHelper>

#include <QTimer>
#include <QBuffer>
#include <QFile>
#include <QCoreApplication>
#include <QDBusInterface>
#include <QDBusPendingCall>
#include <QSystemTrayIcon>
#include <QDebug>

#include <QMessageBox>
#include <QPushButton>
#include "uihelper.h"
#include <QDateTime>
#include "easylogging++.h"

namespace  {
    const auto environmentUseSystemTray = "DEEPIN_LICENSE_USE_SYSTEM_TRAY";
    const QString brand = "授权认证工具";
}

bool isUseSystemTray()
{
    qputenv(environmentUseSystemTray, "1");
    return qgetenv(environmentUseSystemTray) == "1" || qgetenv("XDG_CURRENT_DESKTOP") != "Deepin";
}

class ActivatorControlPrivate
{
public:
    ActivatorControlPrivate(ActivatorControl *parent)
        : q_ptr(parent)
    {
    }

    ActivatorWindow *activatorWindow = Q_NULLPTR;
    LicenseHelper *licenseHelper = Q_NULLPTR;
    LicenseTrayIcon *m_licenseTrayIcon = Q_NULLPTR;
    bool quitAfterNotify = true;
    LicenseModel state() const;

    ActivatorControl *q_ptr;
    Q_DECLARE_PUBLIC(ActivatorControl)
};

ActivatorControl::ActivatorControl(QObject *parent)
    : QObject(parent)
    , dd_ptr(new ActivatorControlPrivate(this))
{
    Q_D(ActivatorControl);

    qRegisterMetaType<LicenseModel>();

    connect(this, &ActivatorControl::showActiveWindow, this, &ActivatorControl::initConnection);

    d->licenseHelper = new LicenseHelper(this);
//    if (isUseSystemTray())
//         initTrayIcon();

    connect(d->licenseHelper, &LicenseHelper::showActivator, this, &ActivatorControl::onShowActivator);
    connect(d->licenseHelper, &LicenseHelper::activateError, this, &ActivatorControl::showMessage);
    connect(d->licenseHelper, &LicenseHelper::activeStateChanged, this, [ = ]() {
        emit activeStateChanged(d->state());
    });


//    connect(d->licenseHelper, &LicenseHelper::heartbeatNotify, this, [ = ]() {
//        emit heartbeatNotify(d->state());
//    });
//    connect(d->licenseHelper, &LicenseHelper::sendQrcodeUrl, this, &ActivatorControl::setQrcodeUrl);
//    connect(d->licenseHelper, &LicenseHelper::errorMsg, [ = ](const quint32 & message) {
//        if (SetServerUrlFail == message) {
//            QString tips = tr("Failed to set the activation server");
//            emit d->activatorWindow->serverMessageStopWaiting();
//            notifyErrorMessage(tips);
//        }
//    });
//    connect(d->licenseHelper, &LicenseHelper::activeMsgChange, [ = ](const quint32 &message, const QString &startTime) {
//        if (SystemFilesUnvalid == message) {
//            QString tips = tr("Important system files have been tampered");
//            emit d->activatorWindow->serverMessageStopWaiting();
//            notifyErrorMessage(tips);
//        } else {
//            Q_EMIT d->activatorWindow->receiveActiveFailMessage(message, startTime);
//        }
//    });
//    connect(d->licenseHelper, &LicenseHelper::activeMsgChange, this, &ActivatorControl::volumeActiveStateChange);
//    connect(d->licenseHelper, &LicenseHelper::sendServerMessage, this, &ActivatorControl::sendServerMessage);
    connect(d->licenseHelper, &LicenseHelper::show, this, &ActivatorControl::Show);
    connect(d->licenseHelper, &LicenseHelper::notify, this, &ActivatorControl::Notify);
    connect(d->licenseHelper, &LicenseHelper::screenCurrentDpi, [ = ]
            (qint32 dockPosition, qint32 dockHeight, qint32 displayWidth, qint32 displayHeight) {
            d->activatorWindow->setWindowHeightAccordingDpi(dockPosition, dockHeight,
                                                            displayHeight, displayWidth);
    });

    d->licenseHelper->dockGeometryChanged();
}

ActivatorControl::~ActivatorControl()
{
    Q_D(ActivatorControl);
    if (d->activatorWindow) {
        delete d->activatorWindow;
        d->activatorWindow = Q_NULLPTR;
    }
}

const LicenseModel ActivatorControl::activeState() const
{
    Q_D(const ActivatorControl);
    return d->state();
}

void ActivatorControl::notifyValidityDay(const QString &validDay, const QString &remainingDays,
                                         const quint32 &authorizationType)
{
    QStringList actions = QStringList() << "_open" << tr("View");
    QVariantMap hints;
    hints["x-deepin-action-_open"] =
        "dbus-send,--print-reply,--dest=com.deepin.license.activator,/com/deepin/license/activator,com.deepin.license.activator.Show";

    QList<QVariant> argumentList;
    argumentList << "uos-activator";
    argumentList << notifyId;
    if (TrialAuthorized == authorizationType) {
        argumentList << "uos-activator-trial";
        argumentList << tr("Your trial is going to end");
    } else {
        argumentList << "uos-activator-true";
        argumentList << tr("System license will expire soon");
    }

    argumentList << QString(tr("Expiration date: %1, %2 days left")).arg(validDay).arg(remainingDays);
    argumentList << actions;
    argumentList << hints;
    argumentList << static_cast<int>(stayTime);

    static QDBusInterface notifyApp(notificationsService, notificationsPath, notificationsInterface);
    notifyApp.asyncCallWithArgumentList(notificationsMethod, argumentList);
}

QString ActivatorControl::getErrorMessage(ServerMessageType code, QVariantMap errData)
{
    QString errorMessage;

    LOG(INFO) << "error_Code" << code;
    // qInfo() << "error_Data" << LogEncryption::encryptionMessage(QVariant(errData).toString());

    switch (code) {
    case InvalidServerSum:
    case InvalidClientSum:
        errorMessage = tr("Important system files have been tampered");
        break;
    case InvalidHardwareInfo:
        errorMessage = tr("Invalid hardware info (code: %1)").arg(code);
        break;
    case InternalServerError:
        errorMessage = tr("Server connection failed (code: %1)").arg(code);
        break;
    case CodeDoesNotExist:
        errorMessage = tr("Invalid serial number");
        break;
    case CodeCorrespondingTimeError:
        errorMessage = tr("The effective date is %1, and it cannot be activated now").arg(errData["date"].toString());
        break;
    case CodeExpired:
        errorMessage = tr("Expired serial number");
        break;
    case InvalidActivationFile:
        errorMessage = tr("Invalid activation file");
        break;
    case CodeHasBeenUsed:
        errorMessage = tr("Used serial number");
        break;
    case VolumeCodeInsufficient:
        errorMessage = tr("The volume activation file has exceeded number of uses");
        break;
    case VolumeCodeExpired:
        errorMessage = tr("Expired volume activation file");
        break;
    case MachineIdHasBeenActivatedTrial:
        errorMessage = tr("You have tried the full version and cannot try it again");
        break;
    case InvalidCode:
        errorMessage = tr("Invalid serial number");
        break;
    case BatchActivationFileExpired:
        errorMessage = tr("Expired volume activation file");
        break;
    case MachineIdHasBeenActivated:
        errorMessage = tr("Your system has been activated, and cannot be activated again");
        break;
    case ExpiredActivationFile:
        errorMessage = tr("Expired activation file");
        break;
    case UsedActivationFile:
        errorMessage = tr("Used activation file");
        break;
    case AuthorizationTypeError:
        errorMessage = tr("Your system does not support changing serial numbers");
        break;
    case CodeReplaceNumError:
        errorMessage = QString(tr("You have reached the upper limit (%1 times) for changing serial "
                                  "numbers")).arg(errData.value("change_num", replaceTimes).toUInt());
        break;
    case AuthorizationHasExpired:
        errorMessage = tr("The authorization is expired, please activate by serial number directly");
        break;
    case SystemUnActive:
        errorMessage = tr("You cannot change the serial number, as your system has not been activated yet");
        break;
    case KMSNotImport:
        errorMessage = tr("Unable to change serial numbers, as the authorization server connection error");
        break;
    default:
        errorMessage = tr("Server connection failed (code: %1)").arg(code);
        break;
    }

    return errorMessage;
}

QString ActivatorControl::GetMachineID()
{
    Q_D(ActivatorControl);
    return d->licenseHelper->machineCode();
}

void ActivatorControl::exportInformation(const QString &file)
{
    Q_D(ActivatorControl);
    emit informationExported(d->licenseHelper->exportInformation(file));
}

// ----------------------------------------------------
void ActivatorControl::importInlineURL(const QString &url, const QString &type) {
    Q_D(ActivatorControl);

    d->licenseHelper->importInlineURL(url, type);
}

void ActivatorControl::importOnlineSerialA(const QString &online_url, const QString& serial_a, const QString& label) {
    Q_D(ActivatorControl);

    d->licenseHelper->importOnlineSerialA(online_url, serial_a, label);
}

void ActivatorControl::importSerialB(const QString& serial_b, const QString& product_name) {
    Q_D(ActivatorControl);

    d->licenseHelper->importSerialB(serial_b, product_name);
}

void ActivatorControl::offline_importSerialB(const QString &serial_b, const QString &product_name, const QString& serial_a)
{
    Q_D(ActivatorControl);

    d->licenseHelper->importSerialB(serial_b, product_name, serial_a);
}

void ActivatorControl::importStartProductD(const QString &url, const QString &label)
{
    Q_D(ActivatorControl);

    d->licenseHelper->importStartProductD(url, label);
}

//void ActivatorControl::initMainWindow()
//{
//    Q_D(ActivatorControl);
//    status_widget::GetInstance()->init(d->state());
//}

// -----------------------------------------------------

void ActivatorControl::importSerialKey(const QString &key, const quint32 &networkStatus, const qint32 &isActiveFile)
{
    Q_D(ActivatorControl);
    // qInfo() << "-------key------" << LogEncryption::encryptionMessage(key);
    d->licenseHelper->importSerialKey(key, networkStatus, isActiveFile);
}

void ActivatorControl::importOfflineSerialKey(const QString &key, const QString &activationKey)
{
    Q_D(ActivatorControl);
    // qInfo() << "-------offline key------" << LogEncryption::encryptionMessage(key);
    d->licenseHelper->importOfflineSerialKey(key, activationKey);
}

void ActivatorControl::importVolumeSerialKey(const QString &licensePath, const quint32 &networkStatus)
{
    Q_D(ActivatorControl);
    // qInfo() << "-------Volume key------" << LogEncryption::encryptionMessage(licensePath);
    d->licenseHelper->importVolumeSerialKey(licensePath, networkStatus);
}

void ActivatorControl::deactive()
{
    Q_D(ActivatorControl);
    d->licenseHelper->deactive();
    emit activeStateChanged(d->state());
}

void ActivatorControl::notifyActivator()
{
    QStringList actions = QStringList() << "_open" << tr("Activate");
    QVariantMap hints;
    hints["x-deepin-action-_open"] =
        "dbus-send,--print-reply,--dest=com.deepin.license.activator,/com/deepin/license/activator,com.deepin.license.activator.Show";

    QList<QVariant> argumentList;
    argumentList << "uos-activator";
    argumentList << notifyId;
    argumentList << "uos-activator-false";
    argumentList << "";
    argumentList << tr("Your system is not activated. Please activate as soon as possible for normal use.");
    argumentList << actions;
    argumentList << hints;
    argumentList << static_cast<int>(stayTime);

    static QDBusInterface notifyApp(notificationsService, notificationsPath, notificationsInterface);
    notifyApp.asyncCallWithArgumentList(notificationsMethod, argumentList);
}

void ActivatorControl::notifyErrorMessage(const QString &messageTips)
{
    QStringList actions;
    QVariantMap hints;
    hints["x-deepin-action-_open"] = "";

    QList<QVariant> argumentList;
    argumentList << "uos-activator";
    argumentList << notifyId;
    argumentList << "uos-activator-false";
    argumentList << "";
    argumentList << messageTips;
    argumentList << actions;
    argumentList << hints;
    argumentList << static_cast<int>(stayTime);

    static QDBusInterface notifyApp(notificationsService, notificationsPath, notificationsInterface);
    notifyApp.asyncCallWithArgumentList(notificationsMethod, argumentList);
}

void ActivatorControl::volumeActiveStateChange(const quint32 &message)
{
    if (message == VolumeActiveStateChange) {
        QStringList actions;
        QVariantMap hints;
        hints["x-deepin-action-_open"] = "";

        QList<QVariant> argumentList;
        argumentList << "uos-activator";
        argumentList << notifyId;
        argumentList << "uos-activator-false";
        argumentList << "";
        argumentList << tr("Volume activation exceeds the limit.");
        argumentList << actions;
        argumentList << hints;
        argumentList << static_cast<int>(stayTime);

        static QDBusInterface notifyApp(notificationsService, notificationsPath, notificationsInterface);
        notifyApp.asyncCallWithArgumentList(notificationsMethod, argumentList);
    }
}

void ActivatorControl::initConnection()
{
    Q_D(ActivatorControl);
#if 0
    if (!d->activatorWindow) {
        d->activatorWindow = new ActivatorWindow;
        if (!isUseSystemTray())
            d->activatorWindow->setAttribute(Qt::WA_DeleteOnClose);

        qApp->installEventFilter(d->activatorWindow);
        //
        connect(d->activatorWindow, &ActivatorWindow::importInlineURL, this, &ActivatorControl::importInlineURL);
        connect(d->activatorWindow, &ActivatorWindow::importOnlineSerialA, this, &ActivatorControl::importOnlineSerialA);
        connect(d->activatorWindow, &ActivatorWindow::importSerialB, this, &ActivatorControl::importSerialB);
        //
        connect(d->activatorWindow, &ActivatorWindow::importSerialKey, this, &ActivatorControl::importSerialKey);
        connect(d->activatorWindow, &ActivatorWindow::replaceSerialKey, this, &ActivatorControl::replaceSerialKey);
        /*****************导入离线序列号**********************/
        connect(d->activatorWindow, &ActivatorWindow::importOfflineSerialKey, this,
                &ActivatorControl::importOfflineSerialKey);
        /*****************导入批量序列号**********************/
        connect(d->activatorWindow, &ActivatorWindow::importVolumeSerialKey, this,
                &ActivatorControl::importVolumeSerialKey);
        /*****************导入离线激活码**********************/
        connect(d->activatorWindow, &ActivatorWindow::deactive, this, &ActivatorControl::deactive);
        connect(d->activatorWindow, &ActivatorWindow::requestExportInformation, this,
                &ActivatorControl::exportInformation);

        connect(d->activatorWindow, &ActivatorWindow::importCustomServerUrl, this,
                &ActivatorControl::importCustomServerUrl);
        connect(d->activatorWindow, &ActivatorWindow::importDefaultServerUrl, this,
                &ActivatorControl::importDefaultServerUrl);
        //试用期
        connect(d->activatorWindow, &ActivatorWindow::tryFullVersion, this, &ActivatorControl::tryFullVersion);

        connect(d->activatorWindow, &ActivatorWindow::requestServerConfUrl, this,
                &ActivatorControl::requestServerConfUrl);
        //激活失敗
        connect(this, &ActivatorControl::activeFailMessage, d->activatorWindow, &ActivatorWindow::receiveActiveFailMessage);

        connect(this, &ActivatorControl::activeStateChanged, d->activatorWindow,
                &ActivatorWindow::onActiveStateChanged);
        connect(this, &ActivatorControl::heartbeatNotify, d->activatorWindow, &ActivatorWindow::onHeartbeatNotify);

        connect(this, &ActivatorControl::setQrcodeUrl, d->activatorWindow, &ActivatorWindow::setQrcodeUrl);

        connect(this, &ActivatorControl::informationExported, d->activatorWindow,
                &ActivatorWindow::onInformationExported);

        connect(this, &ActivatorControl::sendServerConfUrl, d->activatorWindow, &ActivatorWindow::receiveServerConfUrl);

        connect(this, &ActivatorControl::landSuccessfully, d->activatorWindow, &ActivatorWindow::landSuccess);

        connect(this, &ActivatorControl::sendServerMessage,
                [=](quint32 showScene, ServerMessageType code, QVariantMap errData) {
                    QString messageTips = getErrorMessage(code, errData);
                    if (ErrorMessageDisplay::Bubble == showScene) {
                        emit d->activatorWindow->serverMessageStopWaiting();
                        notifyErrorMessage(messageTips);
                    } else if (ErrorMessageDisplay::InApplication == showScene) {
                        emit d->activatorWindow->sendServerMessage(messageTips);
                    } else {
                        messageTips = tr("Server connection failed (code: %1)").arg(code);
                        emit d->activatorWindow->sendServerMessage(messageTips);
                    }
                });

        // chushihua
        d->activatorWindow->initializationInitialInterface(d->state());
    }

    d->activatorWindow->initializationInitialInterface(d->state());
    d->activatorWindow->hide();
    if (!d->activatorWindow->isVisible()) {
        d->activatorWindow->show();
        d->activatorWindow->showMainWindow();
        // Dtk::Widget::moveToCenter(d->activatorWindow);
    }

    d->activatorWindow->activateWindow();
    d->activatorWindow->raise();
#endif
    connect(status_widget::GetInstance(), &status_widget::importInlineURL, this, &ActivatorControl::importInlineURL);
    connect(status_widget::GetInstance(), &status_widget::importOnlineSerialA, this, &ActivatorControl::importOnlineSerialA);
    connect(status_widget::GetInstance(), &status_widget::importSerialB, this, &ActivatorControl::importSerialB);
    connect(status_widget::GetInstance(), &status_widget::offline_importSerialB, this, &ActivatorControl::offline_importSerialB);
    connect(status_widget::GetInstance(), &status_widget::importCustomServerUrl, this, &ActivatorControl::importCustomServerUrl);
    connect(status_widget::GetInstance(), &status_widget::importDefaultServerUrl, this,&ActivatorControl::importDefaultServerUrl);
    connect(status_widget::GetInstance(), &status_widget::requestServerConfUrl, this, &ActivatorControl::requestServerConfUrl);
    connect(this, &ActivatorControl::sendServerConfUrl, status_widget::GetInstance(), &status_widget::receiveServerConfUrl);
    //connect(status_widget::GetInstance(), &status_widget::initMainWindow, this, &ActivatorControl::initMainWindow);
    connect(this, &ActivatorControl::activeStateChanged, status_widget::GetInstance(), &status_widget::show_success_widget);
    connect(this, &ActivatorControl::sendNetWorkStatus, status_widget::GetInstance(), &status_widget::get_network_state);
    connect(status_widget::GetInstance(), &status_widget::request_network_state, this, &ActivatorControl::get_network_status);
    connect(status_widget::GetInstance(), &status_widget::importStartProductD, this, &ActivatorControl::importStartProductD);

    status_widget::GetInstance()->show();
}

void ActivatorControl::initTrayIcon()
{
    Q_D(ActivatorControl);
    d->m_licenseTrayIcon = new LicenseTrayIcon(this);
    d->m_licenseTrayIcon->onTrayIconFlush(d->licenseHelper->authorizationState());
    connect(d->m_licenseTrayIcon, &LicenseTrayIcon::clickTrayIconShowWindow, this, &ActivatorControl::showActiveWindow);
    connect(this, &ActivatorControl::showActiveWindow, d->m_licenseTrayIcon, &LicenseTrayIcon::onShowTrayIcon);
    connect(d->m_licenseTrayIcon, &LicenseTrayIcon::endGui, []() {
        qApp->quit();
    });
    connect(d->licenseHelper, &LicenseHelper::activeStateChanged, [ = ]() {
        d->m_licenseTrayIcon->onIconFlash(d->licenseHelper->authorizationState());
    });
    connect(d->licenseHelper, &LicenseHelper::heartbeatNotify, [ = ]() {
        d->m_licenseTrayIcon->onIconFlash(d->licenseHelper->authorizationState());
    });
}

void ActivatorControl::showMessage(const QString& msg) {
    Q_D(ActivatorControl);

    //d->activatorWindow->onActivateErrorInfo(msg);
    status_widget::GetInstance()->show_error_widget(msg);
    // this->warningBox(msg, d->activatorWindow);
}

void ActivatorControl::onShowActivator(const QString &label, const QString &product_name, const QString &version, bool showProbationPeriod) {
    Q_D(ActivatorControl);

    CommonMethods::setProductLabel(label);
    CommonMethods::setProductName(product_name);
    //主界面显示名称、标识、版本
    status_widget::GetInstance()->set_version(version);
    status_widget::GetInstance()->init(d->state(), showProbationPeriod);
    this->initConnection();
}


// message
void ActivatorControl::tipBox(const QString& msg, QWidget *parent) {
    QMessageBox box(parent);
    box.setText(msg);
    box.setWindowTitle(brand);
    box.setIcon(QMessageBox::Information);
    box.setStandardButtons(0);
    box.exec();
}

void ActivatorControl::messageBox(const QString& msg, QWidget *parent) {
    QMessageBox box(parent);
    box.setText(msg);
    box.setWindowTitle(brand);
    box.setIcon(QMessageBox::Information);
    box.addButton(tr("OK"), QMessageBox::YesRole);
    box.exec();

}
void ActivatorControl::warningBox(const QString& msg, QWidget *parent) {
    QMessageBox box(parent);
    box.setText(msg);
    box.setWindowTitle(brand);
    box.setIcon(QMessageBox::Warning);
    box.addButton(tr("OK"), QMessageBox::YesRole);
    box.exec();
}

bool ActivatorControl::yesOrNoBox(const QString& msg, QWidget *parent, bool default_val) {
    QMessageBox box(parent);
    box.setText(msg);
    box.setWindowTitle(brand);
    box.setIcon(QMessageBox::Question);
    QPushButton *yes_btn = box.addButton(tr("Yes"), QMessageBox::YesRole);
    QPushButton *no_btn = box.addButton(tr("No"), QMessageBox::NoRole);
    box.setDefaultButton(default_val ? yes_btn: no_btn);
    box.exec();

    return box.clickedButton() == yes_btn;
}

bool ActivatorControl::detailedYesOrNoBox(const QString& msg, const QString& detailed_text, QWidget *parent, bool default_val) {
    QMessageBox msgBox(QMessageBox::Question,
                       brand,
                       msg,
                       QMessageBox::Yes | QMessageBox::No,
                       parent);
    msgBox.setDetailedText(detailed_text);
    msgBox.setButtonText(QMessageBox::Yes, tr("Yes"));
    msgBox.setButtonText(QMessageBox::No, tr("No"));
    // Turns out the layout box in the QMessageBox is a grid
    // You can force the resize using a spacer this way:
    QSpacerItem* horizontalSpacer = new QSpacerItem(400, 0, QSizePolicy::Minimum, QSizePolicy::Expanding);
    QGridLayout* layout = (QGridLayout*)msgBox.layout();
    layout->addItem(horizontalSpacer, layout->rowCount(), 0, 1, layout->columnCount());
    msgBox.setDefaultButton(default_val ? QMessageBox::Yes : QMessageBox::No);
    return msgBox.exec() == QMessageBox::Yes;
}

QMessageBox::StandardButton ActivatorControl::yesNoCancelBox(const QString& msg,
                                           QWidget *parent,
                                           QMessageBox::StandardButton default_btn) {
    QMessageBox box(parent);
    box.setText(msg);
    box.setWindowTitle(brand);
    box.setIcon(QMessageBox::Question);
    QPushButton *yes_btn = box.addButton(tr("Yes"), QMessageBox::YesRole);
    QPushButton *no_btn = box.addButton(tr("No"), QMessageBox::NoRole);
    box.addButton(tr("Cancel"), QMessageBox::RejectRole);
    box.setDefaultButton(default_btn);
    box.exec();

    QAbstractButton *btn = box.clickedButton();
    if (btn == yes_btn) {
        return QMessageBox::Yes;
    } else if (btn == no_btn) {
        return QMessageBox::No;
    }

    return QMessageBox::Cancel;
}

bool ActivatorControl::yesOrCancelBox(const QString& msg, QWidget *parent, bool default_yes) {
    QMessageBox box(parent);
    box.setText(msg);
    box.setWindowTitle(brand);
    box.setIcon(QMessageBox::Question);
    QPushButton *yes_btn = box.addButton(tr("Yes"), QMessageBox::YesRole);
    QPushButton *cancel_btn = box.addButton(tr("Cancel"), QMessageBox::RejectRole);
    box.setDefaultButton(default_yes ? yes_btn: cancel_btn);
    box.exec();

    return box.clickedButton() == yes_btn;
}
// message

void ActivatorControl::notifyCallbackAction()
{
    emit showActiveWindow();
}

void ActivatorControl::setQuitAfterNotify(bool quit)
{
    Q_D(ActivatorControl);
    d->quitAfterNotify = quit;
}

void ActivatorControl::importCustomServerUrl(const QString &url)
{
    Q_D(ActivatorControl);
    d->licenseHelper->importCustomServerUrl(url);
}

void ActivatorControl::importDefaultServerUrl()
{
    Q_D(ActivatorControl);
    d->licenseHelper->importDefaultServerUrl();
}

void ActivatorControl::tryFullVersion(const quint32 &networkStatus)
{
    Q_D(ActivatorControl);
    d->licenseHelper->tryFullVersion(networkStatus);
}

void ActivatorControl::requestServerConfUrl()
{
    Q_D(ActivatorControl);
    QVariantMap serverCon = d->licenseHelper->requestServerConfUrl();
    emit sendServerConfUrl(serverCon);
}

void ActivatorControl::replaceSerialKey(const QString &replacedSerialKey, const quint32 &networkStatus)
{
    Q_D(ActivatorControl);
    // qInfo() << "-------replaceSerialkey:------" << LogEncryption::encryptionMessage(replacedSerialKey);
    d->licenseHelper->replaceSerialKey(replacedSerialKey, networkStatus);
}

void ActivatorControl::Show()
{
    emit showActiveWindow();
}

quint32 ActivatorControl::GetIndicatorData()
{
    Q_D(ActivatorControl);
    auto state = d->state();
    // return state.license.authorizationState;
}

void ActivatorControl::Notify()
{
    Q_D(ActivatorControl);
    auto state = d->state();
//    //试用期小于10天
//    if (state.license.authorizationState == TrialAuthorized) {
//        if (state.license.validityTrial.toInt() <= 10)
//            notifyValidityDay(state.license.validityEnd, state.license.validityTrial);
//    } else if (state.license.authorizationState == Authorized) {
//        //已授权小于10天
//        if (state.license.validityEnd.isEmpty() || state.license.validityEnd == VALID_FOR_LIFE
//            || state.license.activeMethod == KmsApplication || state.license.activeMethod == KMSProductionLine) {
//            return;
//        } else if (state.license.validityTrial.toInt() <= 10) {
//            notifyValidityDay(state.license.validityEnd, state.license.validityTrial, Authorized);
//        }
//    } else {
//        notifyActivator();
//    }
}

LicenseModel ActivatorControlPrivate::state() const
{
    LicenseModel model;
    // licenseHelper->licenseState();
    QVariantMap licenseInfo = licenseHelper->requestLicenseInfo(CommonMethods::getProductLabel());
    model.license.serial_b = licenseInfo.value("license_b", "").toString();
    model.license.product_name = CommonMethods::getProductName();//licenseInfo.value("license_b_info", "").toString();
    QDateTime expiration= QDateTime::fromString(licenseInfo.value("expiration", "").toString(), "yyyyMMdd");
    model.license.valid_date = expiration.toString("yyyy-MM-dd");
    model.license.product_label= licenseInfo.value("product_label", "").toString();
    model.license.mac = licenseInfo.value("mac", "").toString();
    model.license.state = licenseInfo.value("state", 0).toInt();
    model.license.disk = licenseInfo.value("disk").toString();
    model.license.sn = licenseInfo.value("sn").toString();
    LOG(INFO) << "state:requestLicenseInfo";
//    licenseHelper->licenseState();


//    model.license.actived = licenseHelper->licenseState();
//    model.license.machineCode = licenseHelper->machineCode();
//    model.license.licenseObject = licenseHelper->organization();
//    model.license.authObjectHidden = licenseHelper->getAuthObjectHidden();
//    model.license.product = licenseHelper->product();
//    model.license.authorizationState = licenseHelper->authorizationState();
//    model.license.activeResetState = licenseHelper->activeResetState();
//    model.license.validityEnd = licenseHelper->validityEnd();
//    model.license.activeMethod = licenseHelper->activeMethod();
//    model.license.authorizeMode = licenseHelper->authorizeMode();
//    model.license.activeMsgType = licenseHelper->activeMsgType();
//    model.license.activeCode = licenseHelper->activeCode();
//    // qInfo() << "state-activeCode:" << LogEncryption::encryptionMessage(licenseHelper->activeCode());
//    model.license.validityTrial = licenseHelper->validityTrial();
//    model.system.trialRemind = licenseHelper->trialRemind();
//    model.system.systemType = SystemType(licenseHelper->systemType());
    return model;
}

void ActivatorControl::get_network_status(const QString &url)
{
    Q_D(ActivatorControl);
    bool bRet = d->licenseHelper->get_network_status(url);
    emit sendNetWorkStatus(bRet);
}
