#pragma once

#include "licensemodel.h"

#include <QObject>
#include <QScopedPointer>
#include <QVariantList>
#include <QMessageBox>
#include <QWidget>
#include <QString>

#define ACTIVATOR_SERVICE_NAME "com.nfs.license.activator"
#define ACTIVATOR_SERVICE_PATH "/com/nfs/license/activator"
#define ACTIVATOR_SERVICE_IFC "com.nfs.license.activator"

#define VALID_FOR_LIFE "9999999999"

class ActivatorControlPrivate;
class ActivatorControl : public QObject
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface", ACTIVATOR_SERVICE_IFC)

public:
    explicit ActivatorControl(QObject *parent = Q_NULLPTR);
    ~ActivatorControl() Q_DECL_OVERRIDE;

    const LicenseModel activeState() const;
    void notifyValidityDay(const QString &validDay, const QString &remainingDays,
                           const quint32 &authorizationType = TrialAuthorized);
    QString getErrorMessage(ServerMessageType code, QVariantMap errData);

    Q_SCRIPTABLE QString GetMachineID();

public slots:
    Q_SCRIPTABLE void Show();
    Q_SCRIPTABLE void Notify();
    Q_SCRIPTABLE quint32 GetIndicatorData();

    //
    void importInlineURL(const QString &url, const QString &type);
    void importOnlineSerialA(const QString &online_url, const QString& serial_a, const QString& label);
    void importSerialB(const QString& serial_b, const QString& product_name);
    void offline_importSerialB(const QString &serial_b, const QString &product_name, const QString& serial_a);
    void get_network_status(const QString &url);
    void importStartProductD(const QString &url, const QString &label);
    //

    void exportInformation(const QString &file);
    void importSerialKey(const QString &key, const quint32 &networkStatus, const qint32 &isActiveFile);

    //void initMainWindow();
    //导入离线激活码
    void importOfflineSerialKey(const QString &key, const QString &activationKey);
    void importVolumeSerialKey(const QString &licensePath, const quint32 &networkStatus);
    void deactive();
    void notifyActivator();
    void notifyCallbackAction();
    void notifyErrorMessage(const QString &messageTips);
    void setQuitAfterNotify(bool quit);
    void importCustomServerUrl(const QString &url);
    void importDefaultServerUrl();
    void tryFullVersion(const quint32 &networkStatus);
    void requestServerConfUrl();
    void replaceSerialKey(const QString &changedSerialKey, const quint32 &networkStatus);
    //弹出气泡提示用户激活状态发生改变
    void volumeActiveStateChange(const quint32 &message);
    void initConnection();
    void initTrayIcon();

    // message
    void showMessage(const QString &msg);
    void onShowActivator(const QString &label, const QString &product_name, const QString &version, bool showProbationPeriod);
    //

    void tipBox(const QString& msg, QWidget *parent=0);
    void messageBox(const QString& msg, QWidget *parent=0);
    void warningBox(const QString& msg, QWidget *parent=0);
    bool yesOrNoBox(const QString& msg, QWidget *parent=0, bool default_val=true);
    bool detailedYesOrNoBox(const QString& msg, const QString& detailed_text, QWidget *parent, bool default_val=true);
    QMessageBox::StandardButton yesNoCancelBox(const QString& msg,
                                               QWidget *parent,
                                               QMessageBox::StandardButton default_btn);
    bool yesOrCancelBox(const QString& msg, QWidget *parent, bool default_ok);

Q_SIGNALS:
    void showActiveWindow();
    void informationExported(bool success);
    void activeFailMessage(const quint32 &message, const QString &time);
    void activeStateChanged(const LicenseModel &licenseModel);
    void heartbeatNotify(const LicenseModel &licenseModel);
    void setQrcodeUrl(const QString &url);
    void sendServerConfUrl(const QVariantMap &qvariantMap);
    void landSuccessfully();
    void errorMessage(const quint32 &message);
    void sendServerMessage(quint32 showScene, ServerMessageType code, QVariantMap errData);
    void sendNetWorkStatus(bool bState);

private:
    QScopedPointer<ActivatorControlPrivate> dd_ptr;
    Q_DECLARE_PRIVATE_D(qGetPtrHelper(dd_ptr), ActivatorControl)
};
