#pragma once

#include "licensemodel.h"
#include "widgets/activatesettingswidget.h"
#include "widgets/unactivatedwidget.h"
#include "widgets/titlebar-widget.h"
// #include "widgets/tryfullversionwidget.h"
// #include "widgets/inputserialwidget.h"
// #include "widgets/inputofflineserialwidget.h"
// #include "widgets/importfilewidget.h"
// #include "widgets/activatedetails.h"

// #include <DMainWindow>
#include <QMainWindow>

#include <QWidget>
#include <QStackedWidget>
#include <QScopedPointer>

// DWIDGET_USE_NAMESPACE

class QObject;
class QEvent;

class ActivatorWindowPrivate;
class ActivatorWindow : public QMainWindow
{
    Q_OBJECT
public:
    explicit ActivatorWindow(QWidget *parent = Q_NULLPTR);
    ~ActivatorWindow() Q_DECL_OVERRIDE;
    void initializationInitialInterface(const LicenseModel &model);

    // show center
    void showMainWindow();

public Q_SLOTS:
    // -------------------------------------------
    void onOfflineActivate();
    void onReturnBtnClicked();
    void onNextBtnClicked();
    void onPreviousBtnClicked();
    void onOffActivateBtnClicked();
    void onFailedReturnBtnClicked();

    // waiwang
    void onOnlineActivate(const QString &url, const QString &key_a);

    // 内网
    void onInlineActivate(const QString& url);
    void onOfflineActivateWithInline();

    void onPreviousBtnClickedInline();
    void onOfflineBtnClickedInline(const QString& serial_b);

    void onConfirmBtnClicked();

    // error_info
    void onActivateErrorInfo(const QString &message);

    //-----------------------------------------------------
    void onActiveStateChanged(const LicenseModel &model);
    void onHeartbeatNotify(const LicenseModel &model);
    void onInformationExported(bool success);
    void onTitleBarMenuClicked(QAction *action);
    void onOfflineWidgetBackButtonClicked();
    void setWindowHeightAccordingDpi(qint32 dockPosition, qint32 dockHeight, qint32 displayHeight, qint32 displayWidth);
    void receiveActiveFailMessage(const quint32 &message, const QString &startTime);
    void sendServerMessage(const QString &messageTips);
    void serverMessageStopWaiting();
    void onInputSerialClicked();
    void onReplaceButtonClicked();
    void onImportLicenseClicked();
    void onTryFullVersionClicked();
    void onActivateDetails();
    void onDetailsBackButtonClicked();
    void onTryFullBackButtonClicked();
    void onTryFullActivateNowBtnClicked();
    void onInputSerialBackButtonClicked();
    void onInputSerialImportSerialKey(const QString &key);
    void onInputSerialReplaceSerialKey(const QString &key);
    void setQrcodeUrl(const QString &urlStr);
    void onImportFileBackButtonClicked();

Q_SIGNALS:
    void importInlineURL(const QString &url);
    void importOnlineSerialA(const QString &online_url, const QString& serial_a);
    void importSerialB(const QString& serial_b);

    void requestExportInformation(const QString &filePath);
    void importSerialKey(const QString &key, const quint32 &networkStatus, const qint32 &isActiveFile = 0);
    void replaceSerialKey(const QString &replacedSerialKey, const quint32 &networkStatus);
    void importOfflineSerialKey(const QString &key, const QString &activationKey);
    void importVolumeSerialKey(const QString &licensePath, const quint32 &networkStatus);

    void deactive();
    void importCustomServerUrl(const int &header, const QString &url, const quint32 &networkStatus);
    void importDefaultServerUrl(const int &index, const quint32 &networkStatus);
    //试用期激活
    void tryFullVersion(const quint32 &networkStatus);
    void hideActiveErrorMsg();
    void requestServerConfUrl();
    void receiveServerConfUrl(const QVariantMap &serverConfUrl);
    void exportInformation(bool success);
    void updateView(const LicenseModel &model);
    void setSerialKey(const QString &key);
    void landSuccess();

protected:
    void closeEvent(QCloseEvent *event) Q_DECL_OVERRIDE;
    void showEvent(QShowEvent *event) Q_DECL_OVERRIDE;
    bool eventFilter(QObject *obj, QEvent *ev) Q_DECL_OVERRIDE;

private:
    void clearFontSizeSignal();

private:
    quint32 m_operation;
    LicenseModel m_licenseModel;
    QStackedWidget *m_stackMainWindows;
    TitleBarWidget* m_titlebar_widget;
    QScopedPointer<ActivatorWindowPrivate> dd_ptr;
    ActivateSettingsWidget *m_settingsWidget;
    bool m_abortRequest;
    Q_DECLARE_PRIVATE_D(qGetPtrHelper(dd_ptr), ActivatorWindow)
};
