#include <iostream>
#include <QApplication>
#include <QDBusConnection>
#include <QDBusInterface>
#include <QDebug>
#include <QTranslator>

#include <QFile>
#include <QFileInfo>

#include "activatorcontrol.h"
#include "uihelper.h"
#include "easylogging++.h"

INITIALIZE_EASYLOGGINGPP

using namespace std;

bool loadQss(const QString& path)
{
    QFile file(path);
    if (!QFileInfo(file).exists()) {
        return false;
    }
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
        return false;
    }

    QString style_;
    QTextStream input(&file);
    style_ += "\n";
    style_ += input.readAll();
    qApp->setStyleSheet(style_);

    return true;
}

void initLog() {
    el::Configurations conf;
    conf.setToDefault();
    conf.set(el::Level::Info, el::ConfigurationType::Filename, "/var/log/gui.log");
    conf.set(el::Level::Info, el::ConfigurationType::Format, "%datetime %level: %func %msg");
    // el::Loggers::setLoggingLevel(el::Level::Error);
    el::Loggers::reconfigureLogger("default", conf);
    LOG(INFO) << "main";
}

int main(int argc, char *argv[])
{
    initLog();

    if (argc == 1) {
        CommonMethods::setProductLabel("os");
        CommonMethods::setProductName("方德操作系统");
    } else {
        const QString product_label = argv[1];
        const QString product_name = argv[2];

        CommonMethods::setProductLabel(product_label);
        CommonMethods::setProductName(product_name);
    }


    QApplication app(argc, argv);

    QTranslator translator;
    translator.load(QString(":/icons/deepin/builtin/texts/qt_zh_CN.qm"));
    app.installTranslator(&translator);

    app.setOrganizationName("nfs");
    app.setApplicationName("nfs-license-activator");
    app.setAttribute(Qt::AA_UseHighDpiPixmaps);
    app.setApplicationDisplayName(QObject::tr("System Authorization"));
#ifdef CVERSION
    app.setApplicationVersion(CVERSION);
    LOG(INFO) << "License Version:" << app.applicationVersion();
#endif
    app.setQuitOnLastWindowClosed(false);

    loadQss(":/qt.css");

    if (!QDBusConnection::sessionBus().registerService(ACTIVATOR_SERVICE_NAME)) {
        LOG(INFO) << "registerService Failed, Another activator maybe be running!";
        //exit(0x0000);
        return -1;
    }

    ActivatorControl control;

    if (!QDBusConnection::sessionBus().registerObject(ACTIVATOR_SERVICE_PATH,
                                                      &control, QDBusConnection::ExportScriptableSlots)) {
        LOG(INFO) << "registerObject Failed!";
        //exit(0x0000);
        return -2;
    }

    // control.Show();
    return app.exec();
}
