#include "activatesettingswidget.h"
#include "licensemodel.h"
// #include "uihelper.h"
#include "easylogging++.h"

// #include <DTitlebar>
// #include <DHiDPIHelper>
// #include <DFontSizeManager>
// #include <DPalette>
// #include <DLabel>

// #include <QTitlebar>
// #include <QHiDPIHelper>
// #include <QFontSizeManager>
#include <QPalette>
#include <QLabel>

#include <QVBoxLayout>
#include <QDebug>
#include <QWidget>
#include <QDBusInterface>
#include <QDBusPendingCall>
#include <QTimer>
#include <QLocale>
#include <QAbstractItemView>

namespace {
constexpr qint32 maxCustomUrlLength = 100;
constexpr qint32 layoutSpace = 0;
constexpr qint32 errorValueHeight = 6;
constexpr qint32 minWidgetHeight = 240;
}

ActivateSettingsWidget::ActivateSettingsWidget(QWidget *parent)
    : QDialog(parent)
{
    initUI();
    // initConnection();
}

void ActivateSettingsWidget::initUI()
{

//    m_settingsWidgetFontChange =
//            new QDBusInterface(APPEARANCE_SERVICE_NAME, APPEARANCE_SERVICE_PATH, APPEARANCE_INTERFACE_NAME,
//                               QDBusConnection::sessionBus(), this);

//    setContentLayoutContentsMargins(QMargins(0, 0, 0, 0));
//    setIcon(QIcon::fromTheme("dcc_preferenceSystem"));
//    layout()->setMargin(0);
//    layout()->setSpacing(0);
    QWidget *mainWidget = new QWidget(this);
    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->setContentsMargins(0, 0, 0, 0);
    mainLayout->setSpacing(layoutSpace);
    QHBoxLayout *enableServerLayout = new QHBoxLayout;
    enableServerLayout->setContentsMargins(0, 0, 0, 0);
    m_enableServerLabel = new QLabel(tr("Set Up Authorization Server"));
    //设置字体大小
    // DFontSizeManager::instance()->bind(m_enableServerLabel, DFontSizeManager::T6);
    QFont font;
    font.setWeight(QFont::DemiBold);
    m_enableServerLabel->setFont(font);
    QPalette palette = m_enableServerLabel->palette();
    palette.setColor(QPalette::Text, QColor(0, 26, 46));
    m_enableServerLabel->setPalette(palette);
    enableServerLayout->addWidget(m_enableServerLabel, 0, Qt::AlignCenter);
    QVBoxLayout *radioButtonLayout = new QVBoxLayout;
    m_defaultRadioButton = new QRadioButton(tr("Default"), this);
    m_defaultRadioButton->setFocusPolicy(Qt::NoFocus);
    m_defaultRadioButtonText = m_defaultRadioButton->text();
    m_customRadioButton = new QRadioButton(tr("Customize"), this);
    m_customRadioButton->setFocusPolicy(Qt::NoFocus);
    m_customRadioButtonText = m_customRadioButton->text();
    radioButtonLayout->addWidget(m_defaultRadioButton, 0, Qt::AlignVCenter);
    radioButtonLayout->addSpacing(10);
    radioButtonLayout->addWidget(m_customRadioButton, 0, Qt::AlignVCenter);

    m_customURLPreComboBox = new QComboBox(this);
    m_customURLPreComboBox->setFixedWidth(94);
    m_customURLPreComboBox->setFixedHeight(36);
    m_customURLLineEdit = new QLineEdit(this);
    m_customURLLineEdit->setFixedWidth(244);
    m_customURLLineEdit->setFixedHeight(36);
    // m_customURLLineEdit->lineEdit()->setMaxLength(maxCustomUrlLength + 1);
    // m_customURLLineEdit->setAlertMessageAlignment(Qt::AlignRight);
    QHBoxLayout *customUrlLayout = new QHBoxLayout;
    customUrlLayout->addWidget(m_customURLPreComboBox, 0, Qt::AlignVCenter | Qt::AlignRight);
    customUrlLayout->addSpacing(10);
    customUrlLayout->addWidget(m_customURLLineEdit, 0, Qt::AlignVCenter | Qt::AlignRight);

    m_defaultURLComboBox = new QComboBox(this);
    m_defaultURLComboBox->setFixedWidth(348);
    m_defaultURLComboBox->setFixedHeight(36);
    QVBoxLayout *customSettingLayout = new QVBoxLayout;
    customSettingLayout->addWidget(m_defaultURLComboBox, 0, Qt::AlignVCenter | Qt::AlignRight);
    customSettingLayout->addSpacing(14);
    customSettingLayout->addLayout(customUrlLayout);

    QHBoxLayout *serverInfoLayout = new QHBoxLayout;
    serverInfoLayout->setContentsMargins(10, 0, 10, 0);
    serverInfoLayout->addLayout(radioButtonLayout);
    serverInfoLayout->addSpacing(20);
    serverInfoLayout->addLayout(customSettingLayout);

    mainLayout->addWidget(m_enableServerLabel, 0, Qt::AlignCenter);
    mainLayout->addSpacing(10);
    mainLayout->addLayout(serverInfoLayout);
    mainLayout->addSpacing(10);
    mainWidget->setLayout(mainLayout);
    // addContent(mainWidget);
    m_cancelButton = new QPushButton(tr("Cancel"));
    m_cancelButton->setFixedWidth(202);
    // m_confirmButton = new QSuggestButton(tr("Confirm"));
    // m_confirmButton->setFixedWidth(202);
    // insertButton(0, m_cancelButton, false);
    // insertButton(1, m_confirmButton, true);
    systemFontSizeChanged();
    autoTestSettingCode();
}

void ActivateSettingsWidget::initConnection()
{
    connect(m_defaultRadioButton, &QRadioButton::clicked, this, &ActivateSettingsWidget::defaultRadioBtnClicked);
    connect(m_customRadioButton, &QRadioButton::clicked, this, &ActivateSettingsWidget::customRadioBtnClicked);
    // connect(m_confirmButton, &DSuggestButton::clicked, this, &ActivateSettingsWidget::confirmButtonClicked);
    connect(m_customURLLineEdit, &QLineEdit::textChanged, this, &ActivateSettingsWidget::onCustomURLTextChanged);
    connect(m_customURLPreComboBox, static_cast<void (QComboBox::*)(int index)>(&QComboBox::currentIndexChanged), this,
            &ActivateSettingsWidget::onCustomURLIndexChanged);
    connect(m_settingsWidgetFontChange, SIGNAL(Changed(QString, QString)), this,
            SLOT(systemFontSizeChanged(const QString &, const QString &)));
}

void ActivateSettingsWidget::onCustomURLTextChanged(const QString &url)
{
    if (url.size() > maxCustomUrlLength) {
        QString text = url.left(maxCustomUrlLength);
        m_customURLLineEdit->setText(text);
//        m_customURLLineEdit->setAlert(true);
//        QTimer::singleShot(1000, [=]() { m_customURLLineEdit->setAlert(false); });
//        m_customURLLineEdit->showAlertMessage(
//                QString(tr("It should be no more than %1 characters")).arg(QString::number(maxCustomUrlLength)),
//                m_confirmButton, 1000);
    }
    checkCustomUrlAndKMSUrl(url);
}

void ActivateSettingsWidget::onCustomURLIndexChanged(int index)
{
    if (!previousProtocol.isEmpty())
        m_temporaryData.insert(previousProtocol, m_customURLLineEdit->text());

    if (!m_temporaryData.isEmpty()) {
        switchSavedServerUrl(m_customURLPreComboBox->currentText());
        previousProtocol = m_customURLPreComboBox->currentText();
    }

    checkCustomUrlAndKMSUrl(m_customURLLineEdit->text());
    // if ("kms://" == m_customURLPreComboBox->currentText())
        // m_customURLLineEdit->lineEdit()->setPlaceholderText(tr("eg: 127.1.1.1:10010:authcode"));
    // else
        // m_customURLLineEdit->lineEdit()->setPlaceholderText(tr(""));

    systemFontSizeChanged();
}

void ActivateSettingsWidget::setUIEnable(bool enable)
{
    m_defaultRadioButton->setChecked(enable);
    m_defaultURLComboBox->setEnabled(enable);
    m_customURLLineEdit->setEnabled(!enable);
    m_customURLPreComboBox->setEnabled(!enable);
    m_customRadioButton->setChecked(!enable);
}

void ActivateSettingsWidget::defaultRadioBtnClicked(bool checked)
{
    setUIEnable(checked);
    // m_confirmButton->setEnabled(true);
}

void ActivateSettingsWidget::customRadioBtnClicked(bool checked)
{
    setUIEnable(!checked);
    checkCustomUrlAndKMSUrl(m_customURLLineEdit->text());
}

void ActivateSettingsWidget::confirmButtonClicked()
{
    if (m_defaultRadioButton->isChecked()) {
        emit importDefaultServerUrl(m_defaultURLComboBox->currentIndex());
        serverUrlNotify(true, m_defaultURLComboBox->currentText());
    } else {
        emit importCustomServerUrl(m_customURLPreComboBox->currentIndex(), m_customURLLineEdit->text());
        serverUrlNotify(false, m_customURLPreComboBox->currentText() + m_customURLLineEdit->text());
    }
}

void ActivateSettingsWidget::checkCustomUrlAndKMSUrl(const QString &url)
{
    //自定义服务器正则匹配规则：没有路由，字符有（-_.:）四种
    QRegExp rxSpecial(QString("^([\\w-]+\\.)+[\\w:-]+$"));
    // kms正则匹配规则：kmsUrl:端口号:口令
    //口令Base64编码后特殊符号( +、/、=)，=代表补了0的个数
    QRegExp kmsRxSpecial(QString("^([\\w-]+\\.)+([\\w-]+):[\\d]{1,5}:[A-Za-z0-9\\+=/]+$"));
    QRegExp kmsProductLine(QString("^([\\w-]+\\.)+([\\w-]+):[\\d]{1,5}/[^\u4e00-\u9fa5]+$"));
    bool isKMSUrl = ("kms://" == m_customURLPreComboBox->currentText());
//    if (isKMSUrl && (kmsRxSpecial.exactMatch(url) || kmsProductLine.exactMatch(url)))
//        // m_confirmButton->setEnabled(true);
//    else if (!isKMSUrl && rxSpecial.exactMatch(url))
//        // m_confirmButton->setEnabled(true);
//    else
//        // m_confirmButton->setEnabled(false);
}

void ActivateSettingsWidget::initTemporaryData(const QVariantList &urlList)
{
    for (auto iter : urlList) {
        m_temporaryData.insert(iter.toMap().value(confProtocolName).toString(),
                               iter.toMap().value(confCustomServerName).toString());
    }
}

void ActivateSettingsWidget::receiveServerConfUrl(const QVariantMap &serverCon)
{
    LOG(INFO) << "*******receiveServerConfUrl********" << serverCon.size();
    if (serverCon.size() < 4)
        return;

    m_isInitUI = true;
    previousProtocol.clear();
    m_temporaryData.clear();
    m_defaultURLComboBox->clear();
    m_customURLPreComboBox->blockSignals(true);
    m_customURLPreComboBox->clear();
    m_customURLPreComboBox->blockSignals(false);

    bool bDefault = serverCon.value(confDefaultName, true).toBool();
    int nDefaultIndex = serverCon.value(confDefaultIndexName, 0).toInt();
    int nCustomHeadIndex = serverCon.value(confCustomHeadindexName, 0).toInt();
    QString customUrl = serverCon.value(confCustomUrlName, "").toString();
    QVariantList defaultUrlList = serverCon.value(defaultUrlName, "").toList();
    QVariantList customUrlList = serverCon.value(confOptions, "").toList();

    for (auto iter : defaultUrlList)
        m_defaultURLComboBox->addItem(iter.toString());

    for (auto iter : customUrlList) {
        m_customURLPreComboBox->addItem(iter.toMap().value(confProtocolName).toString());
    }

    initTemporaryData(customUrlList);
    setUIEnable(bDefault);

    m_customURLLineEdit->setText(customUrlList.at(nCustomHeadIndex).toMap().value(confCustomServerName).toString());
    m_customURLPreComboBox->setCurrentIndex(nCustomHeadIndex);
    previousProtocol = m_customURLPreComboBox->currentText();
    m_defaultURLComboBox->setCurrentIndex(nDefaultIndex);

    if (m_defaultRadioButton->isChecked())
        // m_confirmButton->setEnabled(true);

    m_isInitUI = false;

    //移动到父窗体的中心位置
    QWidget *parent = dynamic_cast<QWidget *>(this->parent());
    if (parent) {
        move(parent->geometry().center().rx() - this->width() / 2,
             parent->geometry().center().ry() - this->height() / 2);
    }
}

//此函数两个参数是字体大小改变是信号发送的，在此处未使用所以不做声明，当需要参数时再声明
void ActivateSettingsWidget::systemFontSizeChanged(const QString &, const QString &)
{
    qint32 cancelButtonHeight = m_cancelButton->fontMetrics().height() < buttonHeight
            ? buttonHeight
            : m_cancelButton->fontMetrics().height();
//    qint32 confirmButtonHeight = m_confirmButton->fontMetrics().height() < buttonHeight
//            ? buttonHeight
//            : m_confirmButton->fontMetrics().height();
    m_cancelButton->setFixedHeight(cancelButtonHeight);
//    m_confirmButton->setFixedHeight(confirmButtonHeight);
    // 45为customURLPreComboBox后面下拉箭头的宽度
//    m_customURLPreComboBox->setFixedSize(
//            m_customURLPreComboBox->fontMetrics().width(m_customURLPreComboBox->currentText()) + 45,
//            confirmButtonHeight);
//    // 10是m_customURLPreComboBox和m_customURLLineEdit之间的距离
//    m_defaultURLComboBox->setFixedSize(m_customURLPreComboBox->width() + 10 + m_customURLLineEdit->width(),
//                                       confirmButtonHeight);
//    m_customURLLineEdit->setFixedHeight(confirmButtonHeight);
    // 48 30 20 10为各控件之间距离
    qint32 widgetHeight = m_enableServerLabel->fontMetrics().height() + errorValueHeight
            + m_defaultRadioButton->fontMetrics().height() + errorValueHeight
            + m_customRadioButton->fontMetrics().height() + errorValueHeight + m_cancelButton->fontMetrics().height()
            + errorValueHeight + 40 + 30 + 20 + 10;
    // 22为RadioButton小圆点的宽度，35和45是默认服务器和自定义服务器各行各控件之间的距离之和
    qint32 defaultWidth = m_defaultRadioButton->fontMetrics().width(m_defaultRadioButton->text()) + 22
            + errorValueHeight + m_defaultURLComboBox->width() + 35;
    qint32 customWidth = m_customRadioButton->fontMetrics().width(m_customRadioButton->text()) + 22 + errorValueHeight
            + m_customURLPreComboBox->width() + m_customURLLineEdit->width() + 45;
    qint32 widgetWidth = qMax(defaultWidth, customWidth);
    // 10为允许误差值 20为布局距离两边的距离
    this->setFixedSize(widgetWidth + 20, widgetHeight + 10 > minWidgetHeight ? widgetHeight + 10 : minWidgetHeight);
}

void ActivateSettingsWidget::switchSavedServerUrl(const QString &header)
{
    m_customURLLineEdit->setText(m_temporaryData.value(header));
}

//设置服务器气泡通知
void ActivateSettingsWidget::serverUrlNotify(bool isDefault, const QString &url)
{
    QStringList actions;
    QVariantMap hints;
    hints["x-deepin-action-_open"] = "";

    QList<QVariant> argumentList;
    argumentList << "uos-activator";
    argumentList << notifyId;
    argumentList << "uos-activator-true";
    argumentList << "";
    argumentList
        << QString(tr("Modify the %1 server address to %2")).arg(isDefault ? tr("default") : tr("custom")).arg(url);
    argumentList << actions;
    argumentList << hints;
    argumentList << static_cast<int>(stayTime);

    static QDBusInterface notifyApp(notificationsService, notificationsPath, notificationsInterface);
    notifyApp.asyncCallWithArgumentList(notificationsMethod, argumentList);
}

// gui自动化测试代码构造函数初始化一次
void ActivateSettingsWidget::autoTestSettingCode()
{
    setAccessibleName("settingswidget");
    m_enableServerLabel->setAccessibleName("settings_enableServerLabel");
    m_defaultRadioButton->setAccessibleName("settings_defaultRadioBtn");
    m_defaultURLComboBox->setAccessibleName("settings_defaultCombox");
    m_defaultURLComboBox->view()->setAccessibleName("settings_defaultView");

    m_customRadioButton->setAccessibleName("settings_customRadioBtn");
    m_customURLPreComboBox->setAccessibleName("settings_customCombox");
    m_customURLPreComboBox->view()->setAccessibleName("settings_customView");
    //m_customURLLineEdit->lineEdit()->setAccessibleName("settings_customLineEdit");
    m_cancelButton->setAccessibleName("settings_cancelBtn");
    //m_confirmButton->setAccessibleName("settings_okBtn");
}
