#include "active_widget.h"
#include "ui_active_widget.h"
#include "status_widget.h"
#include <QNetworkReply>
#include <QTextStream>

active_widget::active_widget(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::active_widget)
{
    ui->setupUi(this);

    setWindowFlags(Qt::FramelessWindowHint);

    QRegExp regx("[a-zA-Z0-9]+$");
    validator_a = new QRegExpValidator(regx, ui->lineEdit_seq_a);
    ui->lineEdit_seq_a->setValidator(validator_a);

    validator_b = new QRegExpValidator(regx, ui->lineEdit_seq_b);
    ui->lineEdit_seq_b->setValidator(validator_b);

    validator_c = new QRegExpValidator(regx, ui->lineEdit_seq_c);
    ui->lineEdit_seq_c->setValidator(validator_c);

    ui->label_3->hide();
}

active_widget::~active_widget()
{
    if(validator_a != nullptr)
    {
        delete validator_a;
        validator_a = nullptr;
    }

    if(validator_b != nullptr)
    {
        delete validator_b;
        validator_b = nullptr;
    }

    if(validator_c != nullptr)
    {
        delete validator_c;
        validator_c = nullptr;
    }

    delete ui;
}

void active_widget::on_pushButton_back_clicked()
{
    status_widget::GetInstance()->Is_input_click = false;
    status_widget::GetInstance()->take_active_widget();
    status_widget::GetInstance()->set_status_widget();
    ui->label_3->hide();
    clear_lineedit();
}

//设置QLineEdit控件可编辑
void active_widget::set_lineedit_able()
{
    ui->lineEdit_seq_a->setEnabled(true);
    ui->lineEdit_seq_b->setEnabled(true);
    ui->lineEdit_seq_c->setEnabled(true);

    ui->lineEdit_seq_a->setStyleSheet("QLineEdit{border: 1px solid #B3B3B3;};");
    ui->lineEdit_seq_b->setStyleSheet("QLineEdit{border: 1px solid #B3B3B3;};");
    ui->lineEdit_seq_c->setStyleSheet("QLineEdit{border: 1px solid #B3B3B3;};");
    ui->pushButton->setStyleSheet("QPushButton{border: 1px solid #B3B3B3;};");
    ui->pushButton_2->setStyleSheet("QPushButton{border: 1px solid #B3B3B3;};");
}

//设置QLineEdit控件不可编辑
void active_widget::set_lineedit_unable()
{
    ui->lineEdit_seq_a->setEnabled(false);
    ui->lineEdit_seq_b->setEnabled(false);
    ui->lineEdit_seq_c->setEnabled(false);

    ui->lineEdit_seq_a->setStyleSheet("QLineEdit{border: 1px solid #E4E4E4;};");
    ui->lineEdit_seq_b->setStyleSheet("QLineEdit{border: 1px solid #E4E4E4;};");
    ui->lineEdit_seq_c->setStyleSheet("QLineEdit{border: 1px solid #E4E4E4;};");
    ui->pushButton->setStyleSheet("QPushButton{border: 1px solid #E4E4E4;};");
    ui->pushButton_2->setStyleSheet("QPushButton{border: 1px solid #E4E4E4;};");
}

void active_widget::clear_lineedit()
{
    ui->lineEdit_seq_a->setText("");
    ui->lineEdit_seq_b->setText("");
    ui->lineEdit_seq_c->setText("");
}

void active_widget::on_pushButton_active_clicked()
{
    default_url = status_widget::GetInstance()->get_default_url().section(QRegExp("[/]"), 0, 2);
    serial_a = ui->lineEdit_seq_a->text() + "-" + ui->lineEdit_seq_b->text() + "-" + ui->lineEdit_seq_c->text();

    if(serial_a.length() < 14)
    {
        ui->label_3->show();
        return;
    }
    else
    {
        ui->label_3->hide();
    }

    status_widget::GetInstance()->load_ptr->show();

    QEventLoop loop;
    QTimer::singleShot(100, &loop, SLOT(quit()));
    loop.exec();

    ui->pushButton_active->setStyleSheet("QPushButton{background: #A6D1FF; color: #FFFFFF; font-size: 14px; border-radius: 6px;}");
    ui->pushButton_back->setStyleSheet("QPushButton{color: #B8B8B8; border: 1px solid #E4E4E4; font-size: 14px; border-radius: 6px;}");
    ui->pushButton_active->setEnabled(false);
    ui->pushButton_back->setEnabled(false);

    emit request_network_state(default_url);

    clear_lineedit();
}

void active_widget::recover_style_sheet()
{
    ui->pushButton_active->setStyleSheet("QPushButton{background: #007AFF; color: #FFFFFF; font-size: 14px; border-radius: 6px; border: none;}");
    ui->pushButton_back->setStyleSheet("QPushButton{color: #333333; border: 1px solid #B3B3B3; font-size: 14px; border-radius: 6px;}");
    ui->pushButton_active->setEnabled(true);
    ui->pushButton_back->setEnabled(true);
}

void active_widget::recv_Network_State(bool bState)
{
    if(bState)
    {
        bIsclick = true;
        emit importOnlineSerialA(default_url, serial_a, status_widget::GetInstance()->get_label());
    }
    else
    {
        recover_style_sheet();
        status_widget::GetInstance()->load_ptr->hide();
        QString url = status_widget::GetInstance()->get_customurl() + "&label=" + status_widget::GetInstance()->get_label();
        status_widget::GetInstance()->offline_active_widget_ptr->init_offline_widget(url + "&serial=" + serial_a, status_widget::GetInstance()->disk);
        //跳转至离线激活页面
        status_widget::GetInstance()->take_active_widget();
        status_widget::GetInstance()->show_offline_active_widget();
    }
}

void active_widget::on_lineEdit_seq_a_textChanged(const QString &arg1)
{
    if(arg1.length() == 4)
    {
        ui->lineEdit_seq_b->setFocus();
    }
}

void active_widget::on_lineEdit_seq_b_textChanged(const QString &arg1)
{
    if(arg1.length() == 4)
    {
        ui->lineEdit_seq_c->setFocus();
    }
}
