#include "offline-activated-widget.h"
#include <QProcess>
#include <QDebug>
#include <QNetworkInterface>

#include "../../agent/logencryption.h"
#include "easylogging++.h"
// #include "../../agent/systeminfo.h"

namespace {

} // namespace

OfflineActivatedWidget::OfflineActivatedWidget(QWidget *parent)
    : QWidget(parent)
{
    this->getMacAddress();

    this->initUI();
    this->initConnection();


}

OfflineActivatedWidget::~OfflineActivatedWidget(){

}

void OfflineActivatedWidget::initUI() {
    setupUi(this);

    m_macAddress->setText(getMacAddress());
}

void OfflineActivatedWidget::initConnection() {
    connect(mPreviousBtn, SIGNAL(clicked()),
            this, SLOT(onPreviousBtnClicked()));

    connect(mOffActivatedBtn, SIGNAL(clicked()),
            this, SLOT(onOffActivateBtnClicked()));
}

void OfflineActivatedWidget::initialize() {
    mProductNumLe->setText("");
}

void OfflineActivatedWidget::setProductNum(const QString& product_num) {
    mProductContent->setText(product_num);
}

QString OfflineActivatedWidget::getMacAddress(bool deleteColon) {
    //获得所有的虚拟网卡
    QStringList allVirtualMac;
    QProcess *process = new QProcess(this);

    connect(process,
            static_cast<void (QProcess::*)(int exitCode, QProcess::ExitStatus exitStatus)>(&QProcess::finished), this,
            [&](int exitCode, QProcess::ExitStatus exitStatus) {
                if (exitCode == 0 && exitStatus == QProcess::ExitStatus::NormalExit) {
                    allVirtualMac << QString::fromLocal8Bit(process->readAllStandardOutput())
                                         .split(QRegExp("[\r\n]"), QString::SkipEmptyParts);
                    LOG(INFO) << "all virtual mac " << LogEncryption::encryptionMessage(allVirtualMac);
                }
                process->deleteLater();
            });

    process->start("ls", QStringList() << "/sys/devices/virtual/net/");
    //等待process执行结束
    process->waitForFinished();

    QList<QNetworkInterface> nets = QNetworkInterface::allInterfaces(); // 获取所有网络接口列表
    QStringList physicsMacDevices;
    QStringList virtualMacDevices;
    QString tempMacAddr;

    for (int i = 0; i < nets.count(); i++) {
        // 非回环地址:127.0.0.1
        if (!nets[i].flags().testFlag(QNetworkInterface::IsLoopBack)) {
            tempMacAddr = nets[i].hardwareAddress();
            if (deleteColon)
                tempMacAddr.remove(":");
            LOG(INFO) << "macCount--strMacAddr" << i << LogEncryption::encryptionMessage(tempMacAddr);
            if (allVirtualMac.contains(nets[i].humanReadableName())) {
                virtualMacDevices.append(tempMacAddr);
                LOG(INFO) << "virtual address" << LogEncryption::encryptionMessage(tempMacAddr);
            } else {
                physicsMacDevices.append(tempMacAddr);
                LOG(INFO) << "Physics address" << LogEncryption::encryptionMessage(tempMacAddr);
            }
        }

        if (physicsMacDevices.size() == 2)
            break;
    }

    QString macAddr;
    if (!physicsMacDevices.isEmpty()) {
        macAddr = physicsMacDevices[0];//QString(physicsMacDevices.join(","));
        LOG(INFO) << "return physics macAddr" << LogEncryption::encryptionMessage(macAddr);
        return macAddr;
    }

    if (!virtualMacDevices.isEmpty()) {
        if (virtualMacDevices.size() > 1)
            macAddr = virtualMacDevices.at(0); //+ "," + virtualMacDevices.at(1);
        else
            macAddr = virtualMacDevices.at(0);
    }

    LOG(INFO) << "return macAddr" << LogEncryption::encryptionMessage(macAddr);
    return macAddr;
}


QString OfflineActivatedWidget::getSerialB() {
    return mProductNumLe->text();
}

void OfflineActivatedWidget::onPreviousBtnClicked() {
    emit previousBtnClicked();
}

void OfflineActivatedWidget::onOffActivateBtnClicked() {
    emit offActivateBtnClicked();
}
