#include "offline-inline-widget.h"

#include "../../agent/logencryption.h"
#include <QProcess>
#include <QNetworkInterface>
#include "easylogging++.h"

namespace {

} // namespace

OfflineInlineWidget::OfflineInlineWidget(QWidget *parent)
    : QWidget(parent)
{
    this->initUI();
    this->initConnection();
}

OfflineInlineWidget::~OfflineInlineWidget(){

}

void OfflineInlineWidget::initUI() {
    setupUi(this);

    mMacAddress->setText(getMacAddress().toLower());
}

void OfflineInlineWidget::initConnection() {
    connect(mPreviousBtn, SIGNAL(clicked()),
            this, SLOT(onPreviousBtnClicked()));

    connect(mOffActivatedBtn, SIGNAL(clicked()),
            this, SLOT(onOffActivateBtnClicked()));
}

void OfflineInlineWidget::initialize() {
    mProductNumLe->setText("");
}

QString OfflineInlineWidget::getMacAddress(bool deleteColon) {
    //获得所有的虚拟网卡
    QStringList allVirtualMac;
    QProcess *process = new QProcess(this);

    connect(process,
            static_cast<void (QProcess::*)(int exitCode, QProcess::ExitStatus exitStatus)>(&QProcess::finished), this,
            [&](int exitCode, QProcess::ExitStatus exitStatus) {
                if (exitCode == 0 && exitStatus == QProcess::ExitStatus::NormalExit) {
                    allVirtualMac << QString::fromLocal8Bit(process->readAllStandardOutput())
                                         .split(QRegExp("[\r\n]"), QString::SkipEmptyParts);
                    LOG(INFO) << "all virtual mac " << LogEncryption::encryptionMessage(allVirtualMac);
                }
                process->deleteLater();
            });

    process->start("ls", QStringList() << "/sys/devices/virtual/net/");
    //等待process执行结束
    process->waitForFinished();

    QList<QNetworkInterface> nets = QNetworkInterface::allInterfaces(); // 获取所有网络接口列表
    QStringList physicsMacDevices;
    QStringList virtualMacDevices;
    QString tempMacAddr;

    for (int i = 0; i < nets.count(); i++) {
        // 非回环地址:127.0.0.1
        if (!nets[i].flags().testFlag(QNetworkInterface::IsLoopBack)) {
            tempMacAddr = nets[i].hardwareAddress();
            if (deleteColon)
                tempMacAddr.remove(":");
            LOG(INFO) << "macCount--strMacAddr" << i << LogEncryption::encryptionMessage(tempMacAddr);
            if (allVirtualMac.contains(nets[i].humanReadableName())) {
                virtualMacDevices.append(tempMacAddr);
                LOG(INFO) << "virtual address" << LogEncryption::encryptionMessage(tempMacAddr);
            } else {
                physicsMacDevices.append(tempMacAddr);
                LOG(INFO) << "Physics address" << LogEncryption::encryptionMessage(tempMacAddr);
            }
        }

        if (physicsMacDevices.size() == 2)
            break;
    }

    QString macAddr;
    if (!physicsMacDevices.isEmpty()) {
        macAddr = physicsMacDevices[0]; // QString(physicsMacDevices.join(","));
        LOG(INFO) << "return physics macAddr" << LogEncryption::encryptionMessage(macAddr);
        LOG(INFO) << "return physics macAddr" << macAddr.toStdString();
        return macAddr;
    }

    if (!virtualMacDevices.isEmpty()) {
        if (virtualMacDevices.size() > 1)
            macAddr = virtualMacDevices.at(0); // + "," + virtualMacDevices.at(1);
        else
            macAddr = virtualMacDevices.at(0);
    }

    LOG(INFO) << "return macAddr" << LogEncryption::encryptionMessage(macAddr);
    LOG(INFO) << "return macAddr" << macAddr.toStdString();
    return macAddr;
}


void OfflineInlineWidget::onPreviousBtnClicked() {
    emit previousBtnClickedInline();
}

void OfflineInlineWidget::onOffActivateBtnClicked() {
    QString serial_b = mProductNumLe->text();
    emit offActivateBtnClickedInline(serial_b);
}
