#include <QDebug>
#include <QPainter>

#include "offline-qrcode-widget.h"
#include "easylogging++.h"

extern "C" {
#include "qrencode.h"
}

OfflineQrcodeWidget::OfflineQrcodeWidget(QWidget *parent)
    : QWidget(parent)
{
    this->initUI();
    this->initConnection();
}

OfflineQrcodeWidget::~OfflineQrcodeWidget()
{
}

void OfflineQrcodeWidget::initUI() {
    setupUi(this);
    //
    mQRCodeLab->setScaledContents(true);
    mQRCodeLab->setFixedSize(120, 120);
}

void OfflineQrcodeWidget::initConnection() {
    connect(mReturnBtn, SIGNAL(clicked()),
            this, SLOT(onReturnBtnClicked()));

    connect(mNextBtn, SIGNAL(clicked()),
            this, SLOT(onNextBtnClicked()));
}

void OfflineQrcodeWidget::generateQRCode(const QString& url) {
    //生成二维码,二维码数据,二维码图像是25x25 QR_ECLEVEL_Q 容错等级
    QRcode *qrcode = QRcode_encodeString(url.toStdString().c_str(), 2, QR_ECLEVEL_L, QR_MODE_8, 1);
    if (!qrcode) {
        LOG(INFO) << "qrcode point is null";
        return;
    }

    if (0 >= qrcode->width) {
        LOG(INFO) << "qrcode width Less than 1";
        QRcode_free(qrcode);
        return;
    }

    qint32 qrcodeWidth = qrcode->width;
    LOG(INFO) << "qrcodeWidth" << qrcodeWidth;
    double scaleX = static_cast<double>(width()) / static_cast<double>(qrcodeWidth);
    double scaleY = static_cast<double>(height()) / static_cast<double>(qrcodeWidth);
    QPixmap pixmap = QPixmap(width(), height());
    QPainter painter(&pixmap);
    QColor background(Qt::white);
    painter.setBrush(background);
    painter.setPen(Qt::NoPen);
    painter.drawRect(0, 0, width() + 2, height() + 2);
    QColor foreground(Qt::black);
    painter.setBrush(foreground);
    for (qint32 y = 0; y < qrcodeWidth; y++) {
        for (qint32 x = 0; x < qrcodeWidth; x++) {
            unsigned char data = qrcode->data[y * qrcodeWidth + x];
            if (data & 0x01) {
                QRectF r(x * scaleX, y * scaleY, scaleX, scaleY);
                painter.drawRects(&r, 1);
            }
        }
    }
    mQRCodeLab->setPixmap(pixmap);
    QRcode_free(qrcode);
}


void OfflineQrcodeWidget::onReturnBtnClicked() {
    emit returnBtnClicked();
}
void OfflineQrcodeWidget::onNextBtnClicked() {
    emit nextBtnClicked();
}

