#include "react_active_widget.h"
#include "ui_react_active_widget.h"
#include "status_widget.h"
#include "easylogging++.h"
#include <QPainter>
#include <QDebug>

extern "C" {
#include "qrencode.h"
}

react_active_widget::react_active_widget(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::react_active_widget)
{
    ui->setupUi(this);

    setWindowFlags(Qt::FramelessWindowHint);

    QRegExp regx("[a-zA-Z0-9]+$");
    validator_a = new QRegExpValidator(regx, ui->lineEdit_offline_active_seq);
    ui->lineEdit_offline_active_seq->setValidator(validator_a);

    validator_b = new QRegExpValidator(regx, ui->lineEdit_offline_active_seq_2);
    ui->lineEdit_offline_active_seq_2->setValidator(validator_b);

    validator_c = new QRegExpValidator(regx, ui->lineEdit_offline_active_seq_3);
    ui->lineEdit_offline_active_seq_3->setValidator(validator_c);

    validator_d = new QRegExpValidator(regx, ui->lineEdit_offline_active_seq_4);
    ui->lineEdit_offline_active_seq_4->setValidator(validator_d);

    validator_e = new QRegExpValidator(regx, ui->lineEdit_offline_active_seq_5);
    ui->lineEdit_offline_active_seq_5->setValidator(validator_e);
}

react_active_widget::~react_active_widget()
{
    if(validator_a != nullptr)
    {
        delete validator_a;
        validator_a = nullptr;
    }

    if(validator_b != nullptr)
    {
        delete validator_b;
        validator_b = nullptr;
    }

    if(validator_c != nullptr)
    {
        delete validator_c;
        validator_c = nullptr;
    }

    if(validator_d != nullptr)
    {
        delete validator_d;
        validator_d = nullptr;
    }

    if(validator_e != nullptr)
    {
        delete validator_e;
        validator_e = nullptr;
    }

    delete ui;
}

void react_active_widget::on_pushButton_back_clicked()
{
    status_widget::GetInstance()->take_react_active_widget();
    status_widget::GetInstance()->set_status_widget();
}

void react_active_widget::on_pushButton_offline_active_clicked()
{\
    Is_click = true;
    QString serial_b = ui->lineEdit_offline_active_seq->text() + ui->lineEdit_offline_active_seq_2->text()
            + ui->lineEdit_offline_active_seq_3->text() + ui->lineEdit_offline_active_seq_4->text() + ui->lineEdit_offline_active_seq_5->text();

    if(serial_b != "")
    {
        emit react_offline_importSerialB(serial_b, status_widget::GetInstance()->get_product_name());
        status_widget::GetInstance()->load_ptr->show();

        ui->pushButton_offline_active->setStyleSheet("QPushButton{background: #A6D1FF; color: #FFFFFF; font-size: 14px; border-radius: 6px;}");
        ui->pushButton_back->setStyleSheet("QPushButton{color: #B8B8B8; border: 1px solid #E4E4E4; font-size: 14px; border-radius: 6px;}");
        ui->pushButton_offline_active->setEnabled(false);
        ui->pushButton_back->setEnabled(false);
    }
}

void react_active_widget::init_react_active_widget(QString &url)
{
    generateQRCode(url);
}

void react_active_widget::generateQRCode(const QString& url)
{
    //生成二维码,二维码数据,二维码图像是25x25 QR_ECLEVEL_Q 容错等级
    QRcode *qrcode = QRcode_encodeString(url.toStdString().c_str(), 2, QR_ECLEVEL_L, QR_MODE_8, 1);
    if (!qrcode) {
        LOG(INFO) << "qrcode point is null";
        return;
    }

    if (0 >= qrcode->width) {
        LOG(INFO) << "qrcode width Less than 1";
        QRcode_free(qrcode);
        return;
    }

    qint32 qrcodeWidth = qrcode->width;
    LOG(INFO) << "qrcodeWidth" << qrcodeWidth;
    double scaleX = static_cast<double>(width()) / static_cast<double>(qrcodeWidth);
    double scaleY = static_cast<double>(height()) / static_cast<double>(qrcodeWidth);
    QPixmap pixmap = QPixmap(width(), height());
    QPainter painter(&pixmap);
    QColor background(Qt::white);
    painter.setBrush(background);
    painter.setPen(Qt::NoPen);
    painter.drawRect(0, 0, width() + 2, height() + 2);
    QColor foreground(Qt::black);
    painter.setBrush(foreground);
    for (qint32 y = 0; y < qrcodeWidth; y++) {
        for (qint32 x = 0; x < qrcodeWidth; x++) {
            unsigned char data = qrcode->data[y * qrcodeWidth + x];
            if (data & 0x01) {
                QRectF r(x * scaleX, y * scaleY, scaleX, scaleY);
                painter.drawRects(&r, 1);
            }
        }
    }
    ui->label_qrcode->setScaledContents(true);
    ui->label_qrcode->setPixmap(pixmap);
    QRcode_free(qrcode);
}

void react_active_widget::on_lineEdit_offline_active_seq_textChanged(const QString &arg1)
{
    if(arg1.length() == 6)
    {
        ui->lineEdit_offline_active_seq_2->setFocus();
    }
}

void react_active_widget::on_lineEdit_offline_active_seq_2_textChanged(const QString &arg1)
{
    if(arg1.length() == 6)
    {
        ui->lineEdit_offline_active_seq_3->setFocus();
    }
}

void react_active_widget::on_lineEdit_offline_active_seq_3_textChanged(const QString &arg1)
{
    if(arg1.length() == 6)
    {
        ui->lineEdit_offline_active_seq_4->setFocus();
    }
}

void react_active_widget::on_lineEdit_offline_active_seq_4_textChanged(const QString &arg1)
{
    if(arg1.length() == 6)
    {
        ui->lineEdit_offline_active_seq_5->setFocus();
    }
}

void react_active_widget::recover_style_sheet()
{
    ui->pushButton_offline_active->setStyleSheet("QPushButton{background: #007AFF; color: #FFFFFF; font-size: 14px; border-radius: 6px; border: none;}");
    ui->pushButton_back->setStyleSheet("QPushButton{color: #333333; border: 1px solid #B3B3B3; font-size: 14px; border-radius: 6px;}");
    ui->pushButton_offline_active->setEnabled(true);
    ui->pushButton_back->setEnabled(true);
}
