#include "setting_widget.h"
#include "ui_setting_widget.h"
#include "status_widget.h"
#include <QBitmap>
#include <QPainter>
#include <QDesktopWidget>
#include <QProcess>
#include <QTextStream>

setting_widget::setting_widget(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::setting_widget)
{
    ui->setupUi(this);

    setWindowFlags(Qt::FramelessWindowHint | Qt::Dialog);
    setWindowModality(Qt::WindowModal);
    QBitmap bmp(this->size());
    bmp.fill();
    QPainter paint(&bmp);
    paint.setPen(Qt::NoPen);
    paint.setBrush(Qt::black);
    paint.drawRoundedRect(bmp.rect(), 12, 12);
    setMask(bmp);

    //QDesktopWidget *desktop = QApplication::desktop();
    //move((desktop->width() - this->width())/2, (desktop->height() - this->height())/2);

    ui->radioButton_default->setStyleSheet("QRadioButton::indicator::unchecked{image: url(:/icons/deepin/builtin/icons/icon_unchecked.png);}"
                                           "QRadioButton::indicator::checked{image: url(:/icons/deepin/builtin/icons/icon_checked.png)};");
    ui->radioButton_agent->setStyleSheet("QRadioButton::indicator::unchecked{image: url(:/icons/deepin/builtin/icons/icon_unchecked.png);}"
                                         "QRadioButton::indicator::checked{image: url(:/icons/deepin/builtin/icons/icon_checked.png)};");

    ui->label_4->setVisible(false);

    setStyleSheet("QLineEdit QMenu::item {color: rgb(0, 0, 0);}");
}

setting_widget::~setting_widget()
{
    delete ui;
}

void setting_widget::set_style_sheet()
{
    ui->label_4->setVisible(true);
    ui->pushButton_ok->setStyleSheet("QPushButton{background: #A6D1FF; border: none; color: #FFFFFF; border-radius: 6px; font-size: 14px;}");
    ui->lineEdit_agent_addr->setStyleSheet("QLineEdit{border: 1px solid #FF453A; border-radius: 6px; font-size: 14px;}");
    ui->pushButton_cancel->setStyleSheet("QPushButton{color: #333333; font-size: 14px;}");
    ui->pushButton_ok->setEnabled(false);
    //ui->pushButton_cancel->setEnabled(false);
}

void setting_widget::recover_style_sheet()
{
    ui->label_4->setVisible(false);
    ui->pushButton_ok->setStyleSheet("QPushButton{background: #007AFF; border: none; color: #FFFFFF; border-radius: 6px; font-size: 14px;}");
    ui->lineEdit_agent_addr->setStyleSheet("QLineEdit{border: 1px solid #B3B3B3; border-radius: 6px; font-size: 14px;}");
    ui->pushButton_ok->setEnabled(true);
    //ui->pushButton_cancel->setEnabled(true);
}

void setting_widget::on_pushButton_cancel_clicked()
{
    this->hide();
    recover_style_sheet();
}

void setting_widget::on_pushButton_ok_clicked()
{
    if(!ui->radioButton_default->isChecked())
    {
        QString url = ui->lineEdit_agent_addr->text();
        Is_modify_kms = true;
        emit request_kms_network_state(url);
    }
    else
    {
        recover_style_sheet();
        Is_modify_kms = false;
        emit importDefaultServerUrl();

        status_widget::GetInstance()->change_button("输入序列号");
        status_widget::GetInstance()->show_react_active();
        this->hide();
    }
}

void setting_widget::on_pushButton_2_clicked()
{
    this->hide();
}

void setting_widget::init(const QString &defaultUrl, const bool bRet, const QString &confCustomUrl)
{
    ui->label_2->setText("默认激活服务器地址:" + defaultUrl);
    if(!confCustomUrl.isEmpty() || !confCustomUrl.isNull())
    {
        ui->lineEdit_agent_addr->setText(confCustomUrl);
    }

    if(bRet)
    {
        ui->radioButton_default->setChecked(true);
    }
    else
    {
        ui->radioButton_agent->setChecked(true);
    }
}

void setting_widget::check_net_work(bool flag)
{
    if(!flag)
    {
        set_style_sheet();
        ui->lineEdit_agent_addr->clear();
    }
    else
    {
        recover_style_sheet();

        emit importCustomServerUrl(ui->lineEdit_agent_addr->text());

        status_widget::GetInstance()->change_button("立即激活");
        status_widget::GetInstance()->hide_react_active();
        this->hide();
    }
}

void setting_widget::on_lineEdit_agent_addr_textChanged(const QString &arg1)
{
    recover_style_sheet();
}

void setting_widget::on_radioButton_default_clicked()
{
    recover_style_sheet();
}
