#include "status_widget.h"
#include "ui_status_widget.h"
#include <QBitmap>
#include <QPainter>
#include <QDesktopWidget>
#include <QProcess>
#include <QJsonParseError>
#include <QJsonObject>
#include <iostream>
#include <cstdlib> // 包含 std::system() 所需的头文件


status_widget* status_widget::instane = nullptr;

status_widget::status_widget(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::status_widget)
{
    ui->setupUi(this);
    setWindowFlags(Qt::FramelessWindowHint);
    setWindowFlags(windowFlags() | Qt::WindowStaysOnTopHint);

    QIcon icon(":/icons/deepin/builtin/icons/logo.png");
    setWindowIcon(icon);
    setWindowTitle("授权认证工具");

    QBitmap bmp(this->size());
    bmp.fill();
    QPainter paint(&bmp);
    paint.setPen(Qt::NoPen);
    paint.setBrush(Qt::black);
    paint.drawRoundedRect(bmp.rect(), 12, 12);
    setMask(bmp);

    QDesktopWidget *desktop = QApplication::desktop();
    move((desktop->width() - this->width())/2, (desktop->height() - this->height())/2);
    setAttribute(Qt::WA_DeleteOnClose, true);

    set_widget_ptr = new setting_widget(this);
    act_widget_ptr = new active_widget;
    offline_active_widget_ptr = new offline_active_widget;
    active_status_ptr = new active_status;
    Info_widget_ptr = new Information_widget(this);
    active_file_widget_ptr = new active_file_widget;
    react_active_widget_ptr = new react_active_widget;

    connect(set_widget_ptr, &setting_widget::importCustomServerUrl, [ = ](const QString & url) {    //设置代理服务器
        emit importCustomServerUrl(url);
    });

    connect(set_widget_ptr, &setting_widget::importDefaultServerUrl, [ = ]() {                      //设置默认服务器
        emit importDefaultServerUrl();
    });

    connect(act_widget_ptr, &active_widget::importOnlineSerialA, [ = ](const QString &online_url, const QString& serial_a, const QString& label) {
        emit importOnlineSerialA(online_url, serial_a, label);
    });                                                                                             //在线激活

    connect(offline_active_widget_ptr, &offline_active_widget::offline_importSerialB, [ = ](const QString& serial_b, const QString& product_name, const QString& serial_a) {
        emit offline_importSerialB(serial_b, product_name, serial_a);
    });                                                                                             //二维码激活

    connect(active_file_widget_ptr, &active_file_widget::importSerialB, [ = ](const QString& serial_b, const QString& product_name) {
        emit importSerialB(serial_b, product_name);                                                 //文件激活
    });

    connect(act_widget_ptr, &active_widget::request_network_state, [ = ](const QString &url) {
        emit request_network_state(url);
    });

    connect(this, &status_widget::sendNetworkState, act_widget_ptr, &active_widget::recv_Network_State);

    connect(react_active_widget_ptr, &react_active_widget::react_offline_importSerialB, [ = ](const QString& serial_b, const QString& product_name) {
        emit importSerialB(serial_b, product_name);                                                     //离线重新激活
    });

    connect(set_widget_ptr, &setting_widget::request_kms_network_state, [ = ](const QString &url) {
        emit request_network_state(url);
    });

    connect(this, &status_widget::sendKmsNetworkState, set_widget_ptr, &setting_widget::check_net_work);

    create_menu();
    create_load();

    read_file();

    if(!bRet)
    {
        change_button("立即激活");
        hide_react_active();
    }
}

status_widget::~status_widget()
{
    delete ui;

    release();
}

//释放控件
void status_widget::release()
{
    if(set_widget_ptr != nullptr)
    {
        delete set_widget_ptr;
        set_widget_ptr = nullptr;
    }

    if(act_widget_ptr != nullptr)
    {
        delete act_widget_ptr;
        act_widget_ptr = nullptr;
    }

    if(offline_active_widget_ptr != nullptr)
    {
        delete offline_active_widget_ptr;
        offline_active_widget_ptr = nullptr;
    }

    if(active_status_ptr != nullptr)
    {
        delete active_status_ptr;
        active_status_ptr = nullptr;
    }

    if(instane != nullptr)
    {
        instane->deleteLater();
        instane = nullptr;
    }

    if(pMenu != nullptr)
    {
        delete pMenu;
        pMenu = nullptr;
    }

    if(pAction_info != nullptr)
    {
        delete pAction_info;
        pAction_info = nullptr;
    }

    if(pAction_set != nullptr)
    {
        delete pAction_set;
        pAction_set = nullptr;
    }

    if(Info_widget_ptr != nullptr)
    {
        delete Info_widget_ptr;
        Info_widget_ptr = nullptr;
    }

    if(active_file_widget_ptr != nullptr)
    {
        delete active_file_widget_ptr;
        active_file_widget_ptr = nullptr;
    }

    if(load_ptr != nullptr)
    {
        delete load_ptr;
        load_ptr = nullptr;
    }

    if(pAction_react != nullptr)
    {
        delete pAction_react;
        pAction_react = nullptr;
    }

    if(react_active_widget_ptr != nullptr)
    {
        delete react_active_widget_ptr;
        react_active_widget_ptr = nullptr;
    }
}

status_widget* status_widget::GetInstance()
{
    if(instane == nullptr)
    {
        instane = new status_widget;
    }

    return instane;
}

//鼠标移动事件
void status_widget::mouseMoveEvent(QMouseEvent* event)
{
    if(m_bDrag)
    {
        QPoint distance = event->globalPos() - mouseStartPoint;
        this->move(windowTopLeftPoint + distance);
    }
}

void status_widget::mousePressEvent(QMouseEvent *event)
{
    if(event->button() == Qt::LeftButton)
    {
        m_bDrag = true;
        mouseStartPoint = event->globalPos();
        windowTopLeftPoint = this->frameGeometry().topLeft();
    }
}

void status_widget::mouseReleaseEvent(QMouseEvent *event)
{
    if(Qt::LeftButton == event->button())
    {
        m_bDrag = false;
    }
}

//取出激活页面
void status_widget::take_active_widget()
{
    act_widget_ptr = static_cast<active_widget*>(ui->scrollArea->takeWidget());
}

//展示状态页
void status_widget::set_status_widget()
{
    ui->scrollArea->setWidget(widget);
}

//取出激活结果页面
void status_widget::take_active_status()
{
    active_status_ptr = static_cast<active_status*>(ui->scrollArea->takeWidget());
}

//取出导入激活文件界面
void status_widget::take_active_file_widget()
{
    active_file_widget_ptr = static_cast<active_file_widget*>(ui->scrollArea->takeWidget());
}

//调用输入序列号按钮事件
void status_widget::click_input_seq()
{
    on_pushButton_input_seq_clicked();
}

//取出离线激活页面
void status_widget::take_offline_active_widget()
{
    offline_active_widget_ptr = static_cast<offline_active_widget*>(ui->scrollArea->takeWidget());
}

//展示离线激活页面
void status_widget::show_offline_active_widget()
{
    ui->scrollArea->setWidget(offline_active_widget_ptr);
}

//取出重新激活页面
void status_widget::take_react_active_widget()
{
    react_active_widget_ptr = static_cast<react_active_widget*>(ui->scrollArea->takeWidget());
}

//显示重新激活页面
void status_widget::show_react_active_widget()
{
    ui->scrollArea->setWidget(react_active_widget_ptr);
}

//隐藏重新激活功能
void status_widget::hide_react_active()
{
    pAction_react->setVisible(false);
}

//显示重新激活功能
void status_widget::show_react_active()
{
    pAction_react->setVisible(true);
}

void status_widget::on_pushButton_close_clicked()
{
    this->close();
}

void status_widget::on_pushButton_input_seq_clicked()
{
    read_file();

    if(bRet)
    {
        Is_input_click = true;
        widget = ui->scrollArea->takeWidget();
        ui->scrollArea->setWidget(act_widget_ptr);
    }
    else
    {
        bIsclick = true;
        emit importInlineURL(confCustomUrl, label);
        load_ptr->show();

        ui->pushButton_input_seq->setStyleSheet("QPushButton{background: #A6D1FF; color: #FFFFFF; font-size: 14px; border-radius: 6px;}");
        ui->pushButton_input_file->setStyleSheet("QPushButton{color: #B8B8B8; border: 1px solid #E4E4E4; font-size: 14px; border-radius: 6px;}");
        ui->pushButton_input_seq->setEnabled(false);
        ui->pushButton_input_file->setEnabled(false);
    }
}

void status_widget::recover_style_sheet()
{
    ui->pushButton_input_seq->setStyleSheet("QPushButton{background: #007AFF; color: #FFFFFF; font-size: 14px; border-radius: 6px; border: none;}");
    ui->pushButton_input_file->setStyleSheet("QPushButton{color: #333333; border: 1px solid #B3B3B3; font-size: 14px; border-radius: 6px;}");
    ui->pushButton_input_seq->setEnabled(true);
    ui->pushButton_input_file->setEnabled(true);
}

void status_widget::init(const LicenseModel &model, bool showProbationPeriod)
{
    ui->label_product_name->setText(model.license.product_name);
    //ui->label_ident->setText(model.license.product_label);
    label = model.license.product_label;

    if(model.license.state == 0)
    {
        ui->label_auth_status->setText("未激活");
        ui->label_auth_status->setStyleSheet("QLabel{color: #FF453A;}");
	const char* command = "echo 'echo '您的系统为未激活状态，请及时激活'' > '/etc/profile.d/license.sh'";
    	int result = std::system(command);

    }
    else if(model.license.state == 1)
    {
        ui->label_auth_status->setText("已授权");
        ui->label_auth_status->setStyleSheet("QLabel{color: #007AFF;}");
	const char* command = "rm -rf '/etc/profile.d/license.sh'";
    	int result = std::system(command);
    }
    else if(model.license.state == 2)
    {
        ui->label_auth_status->setText("授权已过期");
        ui->label_auth_status->setStyleSheet("QLabel{color: #FF453A;}");
	const char* command = "echo 'echo '您的系统授权已过期，请及时激活'' > '/etc/profile.d/license.sh'";
    	int result = std::system(command);

    }
    else if(model.license.state == 3)
    {
        if(!showProbationPeriod)
        {
            ui->label_auth_status->setText("未激活");
            ui->label_auth_status->setStyleSheet("QLabel{color: #FF453A;}");
	    const char* command = "echo 'echo '您的系统为未激活状态，请及时激活'' > '/etc/profile.d/license.sh'";
    	    int result = std::system(command);
        }
        else
        {
            ui->label_auth_status->setText("试用期已授权");
            ui->label_auth_status->setStyleSheet("QLabel{color: #007AFF;}");
	    const char* command = "echo 'echo '您的系统为试用期已授权状态，测试用'' > '/etc/profile.d/license.sh'";
    	    int result = std::system(command);
        }
    }
    else if(model.license.state == 4)
    {
        if(!showProbationPeriod)
        {
            ui->label_auth_status->setText("未激活");
            ui->label_auth_status->setStyleSheet("QLabel{color: #FF453A;}");
	    const char* command = "echo 'echo '您的系统为未激活状态，请及时激活'' > '/etc/profile.d/license.sh'";
    	    int result = std::system(command);
        }
        else
        {
            ui->label_auth_status->setText("试用期已过期");
            ui->label_auth_status->setStyleSheet("QLabel{color: #FF453A;}");
	    const char* command = "echo 'echo '您的系统试用期已过期，请及时激活'' > '/etc/profile.d/license.sh'";
    	    int result = std::system(command);
        }
    }

    ui->label_match_id->setText(model.license.disk);
    disk = model.license.disk;
    mac = model.license.mac;
    sn = model.license.sn;

    if(model.license.serial_b.isEmpty() || model.license.serial_b.isNull())
    {
        ui->frame_5->setVisible(false);
        ui->frame_6->setVisible(false);;
        ui->frame_7->move(0, 274);
    }
    else
    {
        ui->label_active_seq->setText(model.license.serial_b);

        if(model.license.product_seq.isEmpty() || model.license.product_seq.isNull())
        {
            ui->frame_6->setVisible(true);
            ui->frame_6->move(0, 274);
            ui->frame_7->move(0, 322);
            ui->frame_6->setStyleSheet("QFrame{background: #E5F1FF; border-radius: 6px;}");
            ui->frame_7->setStyleSheet("QFrame{background: #FFFFFF; border-radius: 6px;}");
        }
        else
        {
            ui->label_product_seq->setText(model.license.product_seq);
            ui->frame_5->setVisible(true);
            ui->frame_6->move(0, 322);
            ui->frame_7->move(0, 370);
            ui->frame_6->setStyleSheet("QFrame{background: #FFFFFF; border-radius: 6px;}");
            ui->frame_7->setStyleSheet("QFrame{background: #E5F1FF; border-radius: 6px;}");
        }
    }

    ui->label_due_time->setText(model.license.valid_date);
}

void status_widget::set_version(const QString &ver)
{
    ui->label_ident->setText(ver);
}

void status_widget::create_load()
{
    load_ptr = new FormLoading(this);
    load_ptr->setWindowFlags(Qt::CustomizeWindowHint | Qt::FramelessWindowHint);
    load_ptr->move(180, 290);
}

void status_widget::create_menu()
{
    pMenu = new QMenu;
    pAction_info = new QAction("关于", this);
    pAction_set = new QAction("激活服务器设置", this);
    pAction_react = new QAction("重新激活", this);
    pMenu->addAction(pAction_set);
    pMenu->addAction(pAction_react);
    pMenu->addAction(pAction_info);
    ui->pushButton_set->setMenu(pMenu);
    ui->pushButton_set->setStyleSheet("QPushButton::menu-indicator{image: none;}"
                                      "QPushButton{border: none; image: url(:/icons/deepin/builtin/icons/btn_set.png);}");

    connect(pAction_set, SIGNAL(triggered()), this, SLOT(show_setting_widget()));
    connect(pAction_info, SIGNAL(triggered()), this, SLOT(show_information_widget()));
    connect(pAction_react, SIGNAL(triggered()), this, SLOT(show_reactivation_widget()));
}

void status_widget::show_setting_widget()
{
    read_file();
    set_widget_ptr->init(defaulturl, bRet, confCustomUrl);
    set_widget_ptr->move(this->pos().x() - 63, this->pos().y() + 158);
    set_widget_ptr->show();
}

void status_widget::show_information_widget()
{
    Info_widget_ptr->move(this->pos().x() - 32, this->pos().y() + 158);
    Info_widget_ptr->show();
}

void status_widget::show_reactivation_widget()
{
    if(Is_import_start_productD)
    {
       return;
    }

    read_file();
    load_ptr->show();

    QEventLoop loop;
    QTimer::singleShot(100, &loop, SLOT(quit()));
    loop.exec();

    Is_react_click = true;
    QString url = defaulturl;
    url.section(QRegExp("[/]"), 0, 2);
    emit request_network_state(url);
}

//获取产品名称
QString status_widget::get_product_name()
{
    return ui->label_product_name->text();
}

//confCustomUrl + qrcodepath + "?mac=" + mac_id
QString status_widget::get_customurl()
{
    return customurl;
}

//获取默认服务器url
QString status_widget::get_default_url()
{
    return defaulturl;
}

QString status_widget::get_label()
{
    return label;
}

void status_widget::receiveServerConfUrl(const QVariantMap &serverCon)
{
//    confCustomUrl = serverCon.value("confCustomUrl").toString();
//    QString mac_id = ui->label_match_id->text();
//    bRet = serverCon.value("confDefault").toBool();
//    QString qrcodepath = serverCon.value("qrcodePath").toString();
//    customurl = confCustomUrl + qrcodepath + "?mac=" + mac_id;
//    defaulturl = serverCon.value("defaultUrl").toString();
}

void status_widget::read_file()
{
    QString path = "/var/lib/nfs-license/.license.json";
    QFile file(path);

    if(!file.open(QIODevice::ReadOnly))
    {
        return;
    }

    QByteArray Data = file.readAll();
    file.close();

    QJsonParseError jsonError;
    QJsonDocument jsonDoc(QJsonDocument::fromJson(Data, &jsonError));
    if(jsonError.error != QJsonParseError::NoError)
    {
        return;
    }

    QJsonObject rootObj = jsonDoc.object();

    confCustomUrl = rootObj["confCustomUrl"].toString();
    bRet = rootObj["confDefault"].toBool();
    defaulturl = rootObj["defaultUrl"].toString();
    qrcodepath = rootObj["qrcodePath"].toString();
    customurl = defaulturl + qrcodepath + "?mac=" + mac + "&sn=" + sn + "&disk=" + disk;
}

void status_widget::change_button(const QString str)
{
    ui->pushButton_input_seq->setText(str);
}

void status_widget::on_pushButton_input_file_clicked()
{
    Is_file_active_click = true;
    setWindowFlag(Qt::WindowStaysOnTopHint, false);
    showNormal();

    widget = ui->scrollArea->takeWidget();
    ui->scrollArea->setWidget(active_file_widget_ptr);
}

void status_widget::on_pushButton_clicked()
{
    showMinimized();
}

void status_widget::recover_style()
{
    if(bIsclick)
    {
        widget = ui->scrollArea->takeWidget();
        recover_style_sheet();
        bIsclick = false;
    }
    else if(active_file_widget_ptr->bIsclick)
    {
        take_active_file_widget();
        active_file_widget_ptr->recover_style_sheet();
        active_file_widget_ptr->bIsclick = false;
    }
    else if(act_widget_ptr->bIsclick)
    {
        take_active_widget();
        act_widget_ptr->recover_style_sheet();
        act_widget_ptr->bIsclick = false;
    }
    else if(offline_active_widget_ptr->bIsclick)
    {
        take_offline_active_widget();
        offline_active_widget_ptr->recover_style_sheet();
        offline_active_widget_ptr->bIsclick = false;
    }
    else if(react_active_widget_ptr->Is_click)
    {
        take_react_active_widget();
        react_active_widget_ptr->recover_style_sheet();
        react_active_widget_ptr->Is_click = false;
    }
    else if(Is_react_click)
    {
        if(Is_file_active_click)
        {
            take_active_file_widget();
            recover_style_sheet();
        }
        else if(Is_input_click)
        {
            take_active_widget();
            recover_style_sheet();
        }
        else
        {
            widget = ui->scrollArea->takeWidget();
            recover_style_sheet();
            Is_react_click = false;
        }
    }
}

void status_widget::show_error_widget(const QString msg)
{
    recover_style();
    load_ptr->hide();
    active_status_ptr->init(false, msg);
    ui->scrollArea->setWidget(active_status_ptr);
    active_status_ptr->run();
}

void status_widget::show_success_widget(const LicenseModel &licenseModel)
{
    recover_style();

    load_ptr->hide();
    init(licenseModel, true);
    active_status_ptr->init(true, "");
    ui->scrollArea->setWidget(active_status_ptr);
    active_status_ptr->run();
}

void status_widget::get_network_state(bool bState)
{
    if(Is_react_click && bState == false)
    {
        load_ptr->hide();
        Is_react_click = false;
        widget = ui->scrollArea->takeWidget();
        QString url = customurl + "&label=" + label;
        react_active_widget_ptr->init_react_active_widget(url);
        show_react_active_widget();
    }
    else if(Is_react_click && bState == true)
    {
        Is_import_start_productD = true;
        emit importStartProductD(defaulturl, label);
        load_ptr->show();

        ui->pushButton_input_seq->setStyleSheet("QPushButton{background: #A6D1FF; color: #FFFFFF; font-size: 14px; border-radius: 6px;}");
        ui->pushButton_input_file->setStyleSheet("QPushButton{color: #B8B8B8; border: 1px solid #E4E4E4; font-size: 14px; border-radius: 6px;}");
        ui->pushButton_input_seq->setEnabled(false);
        ui->pushButton_input_file->setEnabled(false);
    }
    else if(set_widget_ptr->Is_modify_kms)
    {
        emit sendKmsNetworkState(bState);
    }
    else
    {
        Is_input_click = false;
        emit sendNetworkState(bState);
    }
}
