#include "RsaCrypt.h"
#include "licensemodel.h"

QString RsaCrypt::privateEncryptBase64(const QString &strClearData, const QString &strPriKey)
{
    QByteArray priKeyArry = strPriKey.toUtf8();
    const char *pPriKey = priKeyArry.data();
    BIO *pKeyBio = BIO_new_mem_buf(pPriKey, strPriKey.length());

    if (Q_NULLPTR == pKeyBio)
        return Q_NULLPTR;

    RSA *pRsa = RSA_new();
    pRsa = PEM_read_bio_RSAPrivateKey(pKeyBio, &pRsa, Q_NULLPTR, Q_NULLPTR);

    if (Q_NULLPTR == pRsa) {
        BIO_free_all(pKeyBio);
        return Q_NULLPTR;
    }

    int nLen = RSA_size(pRsa);
    char *pEncryptBuf = new char[ulong(nLen)];
    memset(pEncryptBuf, 0, size_t(nLen));

    QByteArray clearDataArry = strClearData.toUtf8();
    int nClearDataLen = clearDataArry.length();
    uchar *pClearData = static_cast<uchar *>(static_cast<void *>(const_cast<char *>(clearDataArry.data())));
    uchar *encryptBuf = static_cast<uchar *>(static_cast<void *>(pEncryptBuf));
    int nSize = RSA_private_encrypt(nClearDataLen,
                                    pClearData,
                                    encryptBuf,
                                    pRsa,
                                    RSA_PKCS1_PADDING);
    QString strEncryptData;

    if (nSize >= 0) {
        QByteArray arry(pEncryptBuf, nSize);
        strEncryptData = arry.toBase64();
    }

    delete[] pEncryptBuf;
    BIO_free_all(pKeyBio);
    RSA_free(pRsa);

    return strEncryptData;
}

QString RsaCrypt::publicDecryptBase64(const QString &strDecryptData, const QString &strPubKey)
{
    QByteArray pubKeyArry = strPubKey.toUtf8();
    const char *pPubKey = pubKeyArry.data();
    BIO *pKeyBio = BIO_new_mem_buf(pPubKey, strPubKey.length());

    if (Q_NULLPTR == pKeyBio)
        return Q_NULLPTR;

    RSA *pRsa = RSA_new();

    if (strPubKey.contains(BEGIN_RSA_PUBLIC_KEY)) {
        pRsa = PEM_read_bio_RSAPublicKey(pKeyBio, &pRsa, Q_NULLPTR, Q_NULLPTR);
    } else {
        pRsa = PEM_read_bio_RSA_PUBKEY(pKeyBio, &pRsa, Q_NULLPTR, Q_NULLPTR);
    }

    if (Q_NULLPTR == pRsa) {
        BIO_free_all(pKeyBio);
        return Q_NULLPTR;
    }

    int nLen = RSA_size(pRsa);
    char *pClearBuf = new char[ulong(nLen)];
    memset(pClearBuf, 0, size_t(nLen));

    QByteArray decryptDataArry = strDecryptData.toUtf8();
    decryptDataArry = QByteArray::fromBase64(decryptDataArry);
    int nDecryptDataLen = decryptDataArry.length();
    uchar *pDecryptData = static_cast<uchar *>(static_cast<void *>(const_cast<char *>(decryptDataArry.data())));
    uchar *clearBuf = static_cast<uchar *>(static_cast<void *>(pClearBuf));
    int nSize = RSA_public_decrypt(nDecryptDataLen,
                                   pDecryptData,
                                   clearBuf,
                                   pRsa,
                                   RSA_PKCS1_PADDING);
    QString strClearData = Q_NULLPTR;

    if (nSize >= 0)
        strClearData = QByteArray(pClearBuf, nSize);

    delete[] pClearBuf;
    BIO_free_all(pKeyBio);
    RSA_free(pRsa);

    return strClearData;
}

QString RsaCrypt::publicEncryptBase64(const QString &strClearData, const QString &strPubKey)
{
    QByteArray pubKeyArry = strPubKey.toUtf8();
    const char *pPubKey = pubKeyArry.data();
    BIO *pKeyBio = BIO_new_mem_buf(pPubKey, pubKeyArry.length());

    if (Q_NULLPTR == pKeyBio)
        return Q_NULLPTR;

    RSA *pRsa = RSA_new();

    if (strPubKey.contains(BEGIN_RSA_PUBLIC_KEY)) {
        pRsa = PEM_read_bio_RSAPublicKey(pKeyBio, &pRsa, Q_NULLPTR, Q_NULLPTR);
    } else {
        pRsa = PEM_read_bio_RSA_PUBKEY(pKeyBio, &pRsa, Q_NULLPTR, Q_NULLPTR);
    }

    if (Q_NULLPTR == pRsa) {
        BIO_free_all(pKeyBio);
        return Q_NULLPTR;
    }

    int nLen = RSA_size(pRsa);
    char *pEncryptBuf = new char[ulong(nLen)];
    memset(pEncryptBuf, 0, size_t(nLen));

    QByteArray clearDataArry = strClearData.toUtf8();
    int nClearDataLen = clearDataArry.length();
    uchar *pClearData = static_cast<uchar *>(static_cast<void *>(const_cast<char *>(clearDataArry.data())));
    uchar *encryptBuf = static_cast<uchar *>(static_cast<void *>(pEncryptBuf));
    int nSize = RSA_public_encrypt(nClearDataLen,
                                   pClearData,
                                   encryptBuf,
                                   pRsa,
                                   RSA_PKCS1_PADDING);
    QString strEncryptData = Q_NULLPTR;

    if (nSize >= 0) {
        QByteArray arry(pEncryptBuf, nSize);
        strEncryptData = arry.toBase64();
    }

    delete[] pEncryptBuf;
    BIO_free_all(pKeyBio);
    RSA_free(pRsa);

    return strEncryptData;
}

QString RsaCrypt::privateDecryptBase64(const QString &strDecryptData, const QString &strPriKey)
{
    QByteArray priKeyArry = strPriKey.toUtf8();
    const char *pPriKey = priKeyArry.data();
    BIO *pKeyBio = BIO_new_mem_buf(pPriKey, priKeyArry.length());

    if (Q_NULLPTR == pKeyBio)
        return Q_NULLPTR;

    RSA *pRsa = RSA_new();
    pRsa = PEM_read_bio_RSAPrivateKey(pKeyBio, &pRsa, Q_NULLPTR, Q_NULLPTR);

    if (Q_NULLPTR == pRsa) {
        BIO_free_all(pKeyBio);
        return Q_NULLPTR;
    }

    int nLen = RSA_size(pRsa);
    char *pClearBuf = new char[ulong(nLen)];
    memset(pClearBuf, 0, size_t(nLen));

    QByteArray decryptDataArry = strDecryptData.toUtf8();
    decryptDataArry = QByteArray::fromBase64(decryptDataArry);
    int nDecryptDataLen = decryptDataArry.length();
    uchar *pDecryptData = static_cast<uchar *>(static_cast<void *>(const_cast<char *>(decryptDataArry.data())));
    uchar *clearBuf = static_cast<uchar *>(static_cast<void *>(pClearBuf));
    int nSize = RSA_private_decrypt(nDecryptDataLen,
                                    pDecryptData,
                                    clearBuf,
                                    pRsa,
                                    RSA_PKCS1_PADDING);
    QString strClearData = Q_NULLPTR;

    if (nSize >= 0)
        strClearData = QByteArray(pClearBuf, nSize);

    delete[] pClearBuf;
    BIO_free_all(pKeyBio);
    RSA_free(pRsa);

    return strClearData;
}

QByteArray RsaCrypt::publicEncrypt(const QByteArray &encryptData, const QByteArray &pubKeyArry)
{
    QByteArray strClearData;
    const char *pPubKey = pubKeyArry.data();
    BIO *pKeyBio = BIO_new_mem_buf(pPubKey, pubKeyArry.length());

    if (Q_NULLPTR == pKeyBio)
        return strClearData;

    RSA *pRsa = RSA_new();

    if (pubKeyArry.contains(BEGIN_RSA_PUBLIC_KEY)) {
        pRsa = PEM_read_bio_RSAPublicKey(pKeyBio, &pRsa, Q_NULLPTR, Q_NULLPTR);
    } else {
        pRsa = PEM_read_bio_RSA_PUBKEY(pKeyBio, &pRsa, Q_NULLPTR, Q_NULLPTR);
    }

    if (Q_NULLPTR == pRsa) {
        BIO_free_all(pKeyBio);
        return strClearData;
    }

    int nLen = RSA_size(pRsa);
    char *pEncryptBuf = new char[ulong(nLen)];
    memset(pEncryptBuf, 0, size_t(nLen));

    int nClearDataLen = encryptData.length();
    uchar *pClearData = static_cast<uchar *>(static_cast<void *>(const_cast<char *>(encryptData.data())));
    uchar *encryptBuf = static_cast<uchar *>(static_cast<void *>(pEncryptBuf));
    int nSize = RSA_public_encrypt(nClearDataLen,
                                   pClearData,
                                   encryptBuf,
                                   pRsa,
                                   RSA_PKCS1_PADDING);

    if (nSize > 0)
        strClearData = QByteArray(pEncryptBuf, nSize);

    delete[] pEncryptBuf;
    BIO_free_all(pKeyBio);
    RSA_free(pRsa);

    return strClearData;
}

QByteArray RsaCrypt::privateDecrypt(const QByteArray &data, const QByteArray &priKeyArry)
{
    QByteArray strClearData;
    const char *pPriKey = priKeyArry.data();
    BIO *pKeyBio = BIO_new_mem_buf(pPriKey, priKeyArry.length());

    if (Q_NULLPTR == pKeyBio)
        return Q_NULLPTR;

    RSA *pRsa = RSA_new();
    pRsa = PEM_read_bio_RSAPrivateKey(pKeyBio, &pRsa, Q_NULLPTR, Q_NULLPTR);

    if (Q_NULLPTR == pRsa) {
        BIO_free_all(pKeyBio);
        return Q_NULLPTR;
    }

    int nLen = RSA_size(pRsa);
    char *pClearBuf = new char[ulong(nLen)];
    memset(pClearBuf, 0, size_t(nLen));

    int nDecryptDataLen = data.length();
    uchar *pDecryptData = static_cast<uchar *>(static_cast<void *>(const_cast<char *>(data.data())));
    uchar *clearBuf = static_cast<uchar *>(static_cast<void *>(pClearBuf));
    int nSize = RSA_private_decrypt(nDecryptDataLen,
                                    pDecryptData,
                                    clearBuf,
                                    pRsa,
                                    RSA_PKCS1_PADDING);

    if (nSize >= 0)
        strClearData = QByteArray(pClearBuf, nSize);

    delete[] pClearBuf;
    BIO_free_all(pKeyBio);
    RSA_free(pRsa);

    return strClearData;
}
