/*
* Copyright (C) 2019 ~ 2020 Uniontech Software Technology Co.,Ltd.
*
* Author:     helei <helei@uniontech.com>
*
* Maintainer: helei <helei@uniontech.com>
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

// RSA 加密解密工具

#ifndef RSA_CRYPT_H
#define RSA_CRYPT_H

#include <openssl/rsa.h>
#include <openssl/pem.h>
#include <openssl/err.h>

#include <QString>

class RsaCrypt
{
public:
    //私钥加密,strClearData 明文,strPriKey 私钥,返回加密后数据(base64格式)
    static QString privateEncryptBase64(const QString &strClearData, const QString &strPriKey);

    //公钥解密,strDecrypt 待解密数据(base64格式),strPubKey 公钥钥,返回明文
    static QString publicDecryptBase64(const QString &strDecryptData, const QString &strPubKey);

    //公钥加密,strClearData 明文,strPubKey 私钥,返回加密后数据(base64格式)
    static QString publicEncryptBase64(const QString &strClearData, const QString &strPubKey);

    //私钥解密,strDecrypt 待解密数据(base64格式),strPriKey 私钥,返回明文
    static QString privateDecryptBase64(const QString &strDecryptData, const QString &strPriKey);

    //公钥加密，data 待解密数据，pubKeyArry 公钥
    static QByteArray publicEncrypt(const QByteArray &data, const QByteArray &pubKeyArry);

    //私钥加密，data 待解密数据，pubKeyArry 私钥
    static QByteArray privateDecrypt(const QByteArray &data, const QByteArray &priKeyArry);
};

#endif // RSA_CRYPT_H
