#ifndef BASE32_H
#define BASE32_H

#include <inttypes.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

void base32_encode(unsigned char *pucInBuf, unsigned int inBufSize, char *encodeBuf)
{
    unsigned int ulLoop;
    unsigned int codeVal;
    unsigned char *codeAddr;
    unsigned int ulCodeCount;
    const char *maps = "ABCDEFGHJKLMNPQRSTUVWXYZ23456789";

    //计算编码的长度
    if (0 == (inBufSize * 8) % 5) {
        ulCodeCount = (inBufSize * 8) / 5;

    } else {
        ulCodeCount = (inBufSize * 8) / 5 + 1;
    }

    for (ulLoop = 0, codeAddr = pucInBuf; ulLoop < ulCodeCount; ulLoop++) {
        //对结果进行赋值
        switch (ulLoop % 8) {
        case 0:
            codeVal = (*codeAddr) & 0x1f;
            break;

        case 1:
            codeVal = (*codeAddr) >> 5;
            codeAddr++;
            codeVal |= ((*codeAddr) & 0x03) << 3;
            break;

        case 2:
            codeVal = ((*codeAddr) >> 2) & 0x1f;
            break;

        case 3:
            codeVal = ((*codeAddr) >> 7) & 0x01;
            codeAddr++;
            codeVal |= ((*codeAddr) & 0x0f) << 1;
            break;

        case 4:
            codeVal = (*codeAddr) >> 4;
            codeAddr++;
            codeVal |= ((*codeAddr) & 0x01) << 4;
            break;

        case 5:
            codeVal = ((*codeAddr) >> 1) & 0x1f;
            break;

        case 6:
            codeVal = ((*codeAddr) >> 6) & 0x03;
            codeAddr++;
            codeVal |= ((*codeAddr) & 0x07) << 2;
            break;

        case 7:
            codeVal = (*codeAddr) >> 3;
            codeAddr++;
            break;
        }

        //对编码输出字符串赋值
        encodeBuf[ulLoop] = maps[codeVal];
    }

    encodeBuf[ulCodeCount] = 0;
    return;
}

unsigned int base32_decode(char *toDecodeStr, unsigned char *pucOutBuf)
{
    unsigned int ulLoop;
    unsigned int mapIndex;
    unsigned char *retAddr;
    const char *maps = "ABCDEFGHJKLMNPQRSTUVWXYZ23456789";

    for (ulLoop = 0, retAddr = pucOutBuf; ulLoop < strlen((char *)toDecodeStr); ulLoop++) {
        for (mapIndex = 0; mapIndex < 32; mapIndex++) {
            if (toDecodeStr[ulLoop] == maps[mapIndex])
                break;
        }

        //对结果进行赋值
        switch (ulLoop % 8) {
        case 0:
            *retAddr = mapIndex;
            break;

        case 1:
            *retAddr |= (mapIndex & 0x07) << 5;
            retAddr++;
            *retAddr = mapIndex >> 3;
            break;

        case 2:
            *retAddr |= mapIndex << 2;
            break;

        case 3:
            *retAddr |= mapIndex << 0x07;
            retAddr++;
            *retAddr = (mapIndex >> 1) & 0x0F;
            break;

        case 4:
            *retAddr |= mapIndex << 0x04;
            retAddr++;
            *retAddr = (mapIndex >> 4) & 0x01;
            break;

        case 5:
            *retAddr |= mapIndex << 0x01;
            break;

        case 6:
            *retAddr |= mapIndex << 0x06;
            retAddr++;
            *retAddr = mapIndex >> 2;
            break;

        case 7:
            *retAddr |= mapIndex << 3;
            retAddr++;
            break;
        }
    }

    return (unsigned int)(retAddr - pucOutBuf);
}
#endif // BASE32_H
