#include "https.h"
#include "logencryption.h"

#include <QJsonDocument>
#include <QJsonObject>
#include <QDateTime>
#include "easylogging++.h"

const qint64 POSTLIMITTIME = 1 * 1000; //limit time 1s

Https::Https(QObject *parent)
    : QObject(parent)
    , m_previousUrl(QString(""))
{
    connect(&m_manager, &QNetworkAccessManager::finished, this, &Https::replyFinished);
}

void Https::replyFinished(QNetworkReply *reply)
{
    LOG(INFO) << "[replyFinished START] network reply list size : " << m_networkReplyList.size();
    QVariant statusCode = reply->attribute(QNetworkRequest::HttpStatusCodeAttribute);

    //如果是http超时，不做处理，因为有外部处理操作操作(整个流程为8s)
    if(QNetworkReply::TimeoutError == reply->error()) {
        qInfo() << "TimeoutError return" << LogEncryption::encryptionMessage(reply->url().fileName());
        updateNetworkReplyList(reply);
    }

    QVariantMap headerDataMap ;
    QList<QPair<QByteArray, QByteArray>> headerDataList= reply->rawHeaderPairs();
    for (auto curData : headerDataList) {
        headerDataMap[QString(curData.first)] = curData.second;
    }

    if (QNetworkReply::NoError != reply->error())
        LOG(INFO) << "NetworkReply Failed: " << statusCode.toInt() << reply->error() << reply->errorString().toStdString();
        // qInfo() << "NetworkReply Failed: " << statusCode.toInt() << reply->error() << reply->errorString();

    if ("health" == reply->url().fileName()) {
        emit receiveHealth(headerDataMap, statusCode.toInt());
    } else {
        QByteArray returnByte = reply->readAll();
        LOG(INFO) << "returnByte" << QString(returnByte).toStdString();
        // qInfo() << "returnByte" << LogEncryption::encryptionMessage(returnByte);
        emit receiveHttpsData(statusCode.toInt(), headerDataMap, returnByte, reply->url().toString());
    }

    updateNetworkReplyList(reply);
    qInfo() << "[replyFinished END] network reply list size : " << m_networkReplyList.size();
}

void Https::sendGet(const QString &url, QString token)
{
    qint64 currentTime = QDateTime::currentMSecsSinceEpoch();
    if (!m_previousUrl.isEmpty() && m_previousUrlTime != 0) {
        // 1s内只允许发送一次同一个接口请求
        if (url.compare(m_previousUrl) == 0) {
            qInfo() << "currentUrl: " << LogEncryption::encryptionMessage(url);
            qint64 timeTo = currentTime - m_previousUrlTime;
            if(timeTo < POSTLIMITTIME && timeTo > 0) {
                qInfo() << "url get send in 1s";
                qInfo() << "previousTime: " << m_previousUrlTime;
                qInfo() << "currentTime: " << currentTime;
                qInfo() << "timeTo: " << timeTo;
                return;
            }
        }
    }

    m_previousUrl = url;
    m_previousUrlTime = currentTime;

    QNetworkRequest request;
    //判断一下网络状态， 如果为NotAccessible 重新设置一下
    if(QNetworkAccessManager::NotAccessible == m_manager.networkAccessible()){
        LOG(INFO) << "m_manager.networkAccessible() NotAccessible";
        m_manager.setNetworkAccessible(QNetworkAccessManager::Accessible);
    }

    if (!token.isEmpty()) {
        token = "Bearer " + token;
        request.setRawHeader("AuthToken", token.toLatin1());
    }

    // header
    QString timestamp = QString::number(QDateTime::currentDateTime().toMSecsSinceEpoch() / 1000);
    QString apiKey = "nFs*D5A62CBD8C04^d=484A43EC18E864CF8AE7";  // 公钥
    QString salt = "y8tnqh6o6du71zj87g208shkznrkpbh1uk9r10";  // 盐
    QString token_all = apiKey + salt + timestamp; // 生成的md5
    QString md5 = QString(QCryptographicHash::hash(token_all.toLatin1(), QCryptographicHash::Md5).toHex());
    request.setRawHeader("Timestamp", timestamp.toLatin1());
    request.setRawHeader("Token", md5.toLatin1());

    request.setUrl(QUrl(url));
    request.setHeader(QNetworkRequest::ContentTypeHeader, "application/json");
    QNetworkReply *networkReply = m_manager.get(request);
    m_networkReplyList.append(networkReply);
    networkReply = Q_NULLPTR;
    qInfo() << "http sendPost url:" << LogEncryption::encryptionMessage(url);
    qInfo() << "http sendPost token:" << LogEncryption::encryptionMessage(token);
}

void Https::sendPost(const QString &url, const QByteArray &body, QString token)
{
    qint64 currentTime = QDateTime::currentMSecsSinceEpoch();
    if (!m_previousUrl.isEmpty() && m_previousUrlTime != 0) {
        // 1s内只允许发送一次同一个接口请求
        if (url.compare(m_previousUrl) == 0) {
            qInfo() << "currentUrl: " << LogEncryption::encryptionMessage(url);
            qint64 timeTo = currentTime - m_previousUrlTime;
            if(timeTo < POSTLIMITTIME && timeTo > 0) {
                qInfo() << "url post send in 1s";
                qInfo() << "previousTime: " << m_previousUrlTime;
                qInfo() << "currentTime: " << currentTime;
                qInfo() << "timeTo: " << timeTo;
                return;
            }
        }
    }

    m_previousUrl = url;
    m_previousUrlTime = currentTime;

    QNetworkRequest request;
    //判断一下网络状态， 如果为NotAccessible 重新设置一下
    if(QNetworkAccessManager::NotAccessible == m_manager.networkAccessible()){
        LOG(INFO) << "m_manager.networkAccessible() NotAccessible";
        m_manager.setNetworkAccessible(QNetworkAccessManager::Accessible);
    }

    if (!token.isEmpty()) {
        token = "Bearer " + token;
        request.setRawHeader("AuthToken", token.toLatin1());
    }
    // header
    QString timestamp = QString::number(QDateTime::currentDateTime().toMSecsSinceEpoch() / 1000);
    QString apiKey = "nFs*D5A62CBD8C04^d=484A43EC18E864CF8AE7";  // 公钥
    QString salt = "y8tnqh6o6du71zj87g208shkznrkpbh1uk9r10";  // 盐
    QString token_all = apiKey + salt + timestamp; // 生成的md5
    QString md5 = QString(QCryptographicHash::hash(token_all.toLatin1(), QCryptographicHash::Md5).toHex());
    request.setRawHeader("Timestamp", timestamp.toLatin1());
    request.setRawHeader("Token", md5.toLatin1());


    request.setUrl(QUrl(url));
    request.setHeader(QNetworkRequest::ContentTypeHeader, "application/json");
    QNetworkReply *networkReply = m_manager.post(request, body);
    m_networkReplyList.append(networkReply);
    networkReply = Q_NULLPTR;
    qInfo() << "http sendPost url:" << LogEncryption::encryptionMessage(url);
    qInfo() << "http sendPost token:" << LogEncryption::encryptionMessage(token);
    qInfo() << "body" << LogEncryption::encryptionMessage(body);
}

void Https::timeOutDeleteReply()
{
    if (!m_networkReplyList.isEmpty()) {
        for (int num = 0; num < m_networkReplyList.size(); ++num) {
            if (m_networkReplyList.at(num) != Q_NULLPTR) {
                m_networkReplyList.at(num)->abort();
                LOG(INFO) << "Network Reply Abort : " << "network reply list size : " << m_networkReplyList.size();
            }
        }
    }
}

void Https::updateNetworkReplyList(QNetworkReply *reply)
{
    if (reply == Q_NULLPTR)
        return;

    if (m_networkReplyList.contains(reply)) {
        for (int var = 0; var < m_networkReplyList.size(); ++var) {
            if (m_networkReplyList.at(var) == reply) {
                reply->deleteLater();
                reply = Q_NULLPTR;
                m_networkReplyList.removeAt(var);
                break;
            }
        }
    }
}

Https::~Https()
{
}
