#ifndef KMS_H
#define KMS_H

#include <QObject>
#include <QByteArray>
#include <QDebug>

#define KMS_CONFIG "/usr/share/kms/kmsserver.conf"

static const QString LicenseServer = "https://license.chinauos.com";
static const QString License1Server = "https://license1.chinauos.com";
static const QString TeLicenseServer = "https://telicense.chinauos.com";
static const QString TeLicense1Server = "https://telicense1.chinauos.com";
static const QString PreLicenseServer = "https://prelicense.chinauos.com";
static const int TIMELIMIT = 60 * 1000; // 1 min
static const int KMSHEART = 20 * 1000; // 20 sec
static const int CONNECTTIMES = 5; // 默认重连次数为５次

struct KmsConfigInfo {
    QString authCode;
    QString serialNumber;
    QString defaultIP;
    QString BackupIP;
    uint defaultPort = 8600; // 服务器默认端口
    uint BackupPort = 8800; // 备份服务器默认端口
};

struct LicenseFileInfo {
    QString activeTime;
    QString activeCode;
    QString activeObject;
    quint32 authObjectHidden;
    quint32 authorizationState;
    quint32 activeMethod;
    QString uuid;
    QString validityEnd;
    quint32 actived;
    QString serverAddress;
    QString serverTime;
    QString systemId;
    QString serverA6;
    QString serverBoardCpu;
    QString systemMac;
    QString kmsDomainName;
};

enum KmsMessageType {
    NullOperation = 0, // 无操作
    VerifyPassword, // 校验口令
    VerifyPasswordResult, // 校验口令返回的结果
    SendSystemInfo, // 发送硬件信息
    GetActivateInfo, // 获取激活信息
    ActiveSignVerify, // 签名验证
    WriteLicense, // 写.license.key文件

    HeartPkgSend = 100
};

struct KMSHeader {
    int len;
    KmsMessageType messageType;
};

enum KmsErrorMsgType {
    Normal, // 正常状态
    NoActivationCode, // 数据库无激活码
    ActivationCodeExpired, // 数据库激活码已过期
    EarlierThanEffectiveTime, // 激活时间早于有效时间
    PasscodeUnmatch, // 口令不匹配
    MachineHasActived, // 设备已激活
    MachineInactive, // 设备占用了授权码，但未激活
    VersionUnmatch,  // 版本类型不匹配
    DbError // 数据库错误
};

enum ReturnValue {
    True = 1,
    False
};

enum VerifyValue {
    VerifySucess = 0,
    VerifyFaild
};

#endif // KMS_H

