#include "licenseagent.h"
#include "licenseinfo.h"
#include "licensekernel.h"
#include "https.h"
#include "licensejson.h"
#include "systeminfo.h"
#include "licensekey.h"
#include "licensemodel.h"

#include <QFile>
#include <QDebug>
#include <QDir>
#include <QTimer>
#include <QDateTime>
#include <QTemporaryFile>
#include <QSysInfo>
#include <QSettings>
#include <QProcess>
#include <QCoreApplication>
#include <QNetworkConfigurationManager>
#include <iostream>
#include "easylogging++.h"

class LicenseAgentPrivate
{
public:
    LicenseAgentPrivate(LicenseAgent *parent)
        : q_ptr(parent)
    {
    }

    LicenseKernel *m_kernel = Q_NULLPTR;
    LicenseInfo *m_licenseInfo = Q_NULLPTR;

    LicenseAgent *q_ptr;
    Q_DECLARE_PUBLIC(LicenseAgent)
};

LicenseAgent::LicenseAgent(LicenseKernel *kernel, LicenseInfo *licenseInfo, QObject *parent)
    : QObject(parent)
    , dd_ptr(new LicenseAgentPrivate(this))
{
    Q_D(LicenseAgent);
    d->m_kernel = kernel;
    d->m_licenseInfo = licenseInfo;
}

LicenseAgent::~LicenseAgent()
{
}

//设置自定义激活服务器地址
void LicenseAgent::ImportCustomServerUrl(const QString &customServerUrl)
{
    qDebug() << endl << endl << "***Import Custom ServerUrl [START]***";
    Q_D(LicenseAgent);
    // d->m_kernel->m_networkStatus = networkStatus;
    d->m_kernel->setCustomServerUrl(customServerUrl);
}

//设置默认激活服务器地址
void LicenseAgent::ImportDefaultServerUrl()
{
    qDebug() << endl << endl << "***Import Default ServerUrl [START]***";
    Q_D(LicenseAgent);
    d->m_kernel->setDefaultServerUrl();
}

//获取服务器配置
QByteArray LicenseAgent::requestServerConfUrl()
{
    Q_D(LicenseAgent);
    return d->m_kernel->getServerConfUrl();
}

void LicenseAgent::ImportOnlineSerialA(const QString &online_url, const QString& serial, const QString& product_label) {
    Q_D(LicenseAgent);

    LOG(INFO) << online_url.toStdString() << "\t" << serial.toStdString();
    d->m_kernel->activateWithOnlineURL(online_url, serial, product_label);
}

void LicenseAgent::ImportInlineURL(const QString &inline_url, const QString &type) {
    Q_D(LicenseAgent);

    LOG(INFO) << "inline_url:" << inline_url.toStdString();
    d->m_kernel->activateWithInlineURL(inline_url, type);
}

void LicenseAgent::ImportSerialB(const QString &license_b, const QString &product_name, const QString &serial) {
    Q_D(LicenseAgent);

    LOG(INFO) << "serial_b:" << license_b.toStdString();
    LOG(INFO) << "product_name:" << product_name.toStdString();
    LOG(INFO) << "serial:" << serial.toStdString();
    d->m_kernel->activateWithSerialB(license_b, product_name, serial);
}

void LicenseAgent::ImportStartProductC(const QString &kms_url, const QString &serial, const QString &sn, const QString &note)
{
    Q_D(LicenseAgent);

    LOG(INFO) << "kms_url:" << kms_url.toStdString() << ", serial:" << serial.toStdString() << ", sn:" << sn << ", note:" << note.toStdString();

    d->m_kernel->ImportStartProductC(kms_url, serial, sn, note);
}

void LicenseAgent::ImportStartProductD(const QString &online_url, const QString &type)
{
    Q_D(LicenseAgent);

    LOG(INFO) << online_url.toStdString();
    d->m_kernel->activateWithStartProductD(online_url, type);
}
