#include "licensefileoperate.h"
#include "licensekey.h"
#include "logencryption.h"

#include <QFile>
#include <QDir>
#include <QUuid>

LicenseFileOperate::LicenseFileOperate()
{
}

QStringList LicenseFileOperate::getLicenseFileData(const QByteArray& readFileData)
{
    // 读取加密id的文件
    QByteArray curReadFileData = readFileData;
    activation_key_dencrypt(reinterpret_cast<unsigned char *>(curReadFileData.data()), static_cast<unsigned long>(curReadFileData.size()));
    QString readBuff = QString(curReadFileData);

    QStringList dataList;
    // 对第一版本配置文件做兼容处理，如果没有$字符，则直接使用#分割
    if (!readBuff.contains(LICENSE_PACKAGE_DIVIDER)) {
        dataList.push_back(readBuff);
    } else {
        dataList = readBuff.split(LICENSE_PACKAGE_DIVIDER, QString::SkipEmptyParts);
    }
    return dataList;

}

void LicenseFileOperate::licenseDataFill(QStringList &fileData)
{
    // 去除末端空格
    if (fileData.last().isEmpty())
        fileData.removeLast();

    // 删除sp1版本本地激活文件中的machineCode，便于后续存储uuid
    if (fileData.size() < (SP2_LICENSE_FILE_LENGTH - 1) && SP1_MACHINECODE_LENGTH == fileData.first().length()) {
        QUuid id = QUuid::createUuid();
        fileData.first() = id.toString().remove("{").remove("-").mid(0, 16);
    }

    // sp2之前版本需添加systemid
    int valideLength = SP2_LICENSE_FILE_LENGTH - 1;
    while (fileData.size() < valideLength) {
        // 5.1.9(1000)版本本地激活文件中没有服务器地址，默认添加为当前服务器地址
        if ((PRESP1_LICENSE_FILE_LENGTH) == fileData.size())
            fileData.append(m_currenServerUrl);

        if (fileData.size() == valideLength -1) {
            fileData.append(m_hardWareInfo.systemId);
        } else {
            fileData.append("");
        }
        // 兼容SP1激活状态
        int curActived = fileData.value(LicenseNameActived, "").toInt();
        if (0 == curActived)
            fileData.replace(LicenseNameActived,"2");
    }

    // sp2之后sp3之前版本需以空格补齐
    while (fileData.size() < SP3_LICENSE_FILE_LENGTH -1)
        fileData.append("");

    fileData.append(m_hardWareInfo.a6);
    fileData.append(m_hardWareInfo.board_cpu);
    fileData.append(m_hardWareInfo.mac);


    QRegExp rxDateFormat(QString("[0-9]{4}-[0-9]{1,2}-[0-9]{1,2}"));
    QString validityEnd = fileData.value(LicenseNameFileValidityEnd, "");
    if (!validityEnd.isEmpty() && rxDateFormat.exactMatch(validityEnd))
        fileData.replace(LicenseNameFileValidityEnd, validityEnd.simplified() + " 23:59:59");

    QString serverTime = fileData.value(LicenseNameServerTime, "");
    if (!serverTime.isEmpty() && rxDateFormat.exactMatch(serverTime))
        fileData.replace(LicenseNameServerTime, serverTime.simplified() + " 00:00:00");

}

// 写激活文件
bool LicenseFileOperate::writeActiveFile(const QByteArray &data, const QString &filePath)
{
    QDir dir = QDir("/var/nfs");

    if (!dir.exists())
        dir.mkpath("/var/nfs");

    QFile file(filePath);

    if (!file.open(QIODevice::Truncate | QIODevice::ReadWrite)) {
        qWarning() << filePath << "File open error";
        return false;
    }

    // 清空文件中的原有内容
    file.resize(0);
    file.write(data);
    file.close();
    return true;
}

// 读激活文件
QByteArray LicenseFileOperate::readActiveFile(const QString &path)
{
    QByteArray fileData;
    QFile file(path);

    if (file.open(QIODevice::ReadOnly)) {
        fileData = file.readAll();
        file.close();
    } else {
        qWarning() << path << " " << file.errorString();
    }

    return fileData;
}
