#include "licenseinfo.h"
#include "licensekernel.h"
#include "licensejson.h"
#include "licensemodel.h"
#include "logencryption.h"

#include <QDateTime>
#include <QDebug>
#include <QNetworkConfigurationManager>
#include <QStringList>
#include <QJsonObject>
#include <QJsonDocument>
#include <iostream>
#include "check-reg.h"
#include <QProcess>
#include "easylogging++.h"

LicenseInfo::LicenseInfo(LicenseKernel *kernel, QObject *parent)
    : QObject(parent)
{
    Q_ASSERT(kernel != Q_NULLPTR);
    m_kernel = kernel;
    connect(m_kernel, &LicenseKernel::activeStateChange, this, &LicenseInfo::LicenseStateChange);
    connect(m_kernel, &LicenseKernel::licenseSuccess, this, &LicenseInfo::licenseSuccess);
    connect(m_kernel, &LicenseKernel::sendActivateErrorInfo, [this](QString error_info) {
                // qDebug() << "------FailMsg------" << error_info;
                std::cout << error_info.toStdString() << std::endl;
                emit sendActivateErrorInfo(error_info);
            });

    m_pHttpMgr = new QNetworkAccessManager();

//    connect(m_kernel, &LicenseKernel::errorMessage, this, &LicenseInfo::errorMessage);
//    connect(m_kernel, &LicenseKernel::heartbeatNotify, this, &LicenseInfo::heartbeatNotify);
//    connect(m_kernel, &LicenseKernel::activeMessageChange, [this](quint32 message, QString startTime) {
//        qDebug() << "------FailMsg------" << message;
//        qInfo() << "------startTime------" << LogEncryption::encryptionMessage(startTime);
//        emit sendActiveMsg(message, startTime);
//    });
//    connect(m_kernel, &LicenseKernel::serverActiveMessageChange, [this](quint32 showScene, ServerMessageType code, QVariantMap errData) {
//        qDebug() << "showScene:" << showScene << " code:" << code;
//        qInfo() <<" errdata:" << LogEncryption::encryptionMessage(QVariant(errData).toString());
//        emit sendServerMsg(showScene, static_cast<quint32>(code), errData);
//    });
//    connect(m_kernel, &LicenseKernel::sendQrcodeUrl, this, &LicenseInfo::sendQrcodeUrl);
//    connect(m_kernel, &LicenseKernel::serverReturn, this, &LicenseInfo::serverReturn);
//    connect(m_kernel, &LicenseKernel::diagnoseFinishedReturn, this, &LicenseInfo::diagnoseFinishedReturn);
}

LicenseInfo::~LicenseInfo()
{
    if(m_pHttpMgr != nullptr)
    {
        delete m_pHttpMgr;
        m_pHttpMgr = nullptr;
    }
}

void LicenseInfo::Show()
{
    emit show();
}

void LicenseInfo::Notify()
{
    emit notify();
}

QByteArray LicenseInfo::ProductInfo(QString type) {
    QList<ProductInfoEntry> list = m_kernel->getLicenseB(type);
    QVariantMap dataMap;
    QString mac = m_kernel->getMacAddress();
    QString serial = m_kernel->getSerial();
    QString sn = m_kernel->getSn();
    dataMap.insert("mac", mac);
    dataMap.insert("disk", serial);
    dataMap.insert("sn", sn);
    if (list.length() >= 1) {
        // 查询授权状态
        ProductInfoEntry res = list[0];
        dataMap.insert("product_label", type);
        dataMap.insert("product_name", res.product_name);
        // dataMap.insert("product_version", res.product_version);

        dataMap.insert("serial", res.serial);
        dataMap.insert("license_b", res.license_b);
        dataMap.insert("expiration", res.expiration);
        dataMap.insert("start_time", res.start_time);

        int res_mac = is_register(toCStr(mac), toCStr(res.license_b), NULL);
        int res_check_serial = -1, res_check_sn = -1;
        if(serial != "")
        {
            res_check_serial = is_register(toCStr(serial), toCStr(res.license_b), NULL);
        }

        if(sn != "")
        {
            res_check_sn = is_register(toCStr(sn), toCStr(res.license_b), NULL);
        }

        if (res_mac == 1 || res_check_serial == 1 || res_check_sn == 1) {
            int state = m_kernel->getAuthorizationState(res.license_b);
            dataMap.insert("state", state);
        } else {
            dataMap.insert("state", Notauthorized);
        }
    } else {
        // 查询试用状态
        QList<TrialInfoEntry> list = m_kernel->getTrialInfo(type);

        if (list.length() == 0) {
            // 未试用
            QDateTime current_date = QDateTime::currentDateTime();
            TrialInfoEntry init_trial;
            init_trial.product_label = type;
            init_trial.current_date = current_date.toString("yyyyMMdd");
            init_trial.remain_days = trial_days;
            init_trial.due_date = current_date.addDays(trial_days).toString("yyyyMMdd");
            init_trial.start_date = current_date.toString("yyyyMMdd");
            m_kernel->insertTrialInfo(init_trial, false);

            dataMap.insert("product_label", type);
            dataMap.insert("product_name", "");
            // dataMap.insert("product_version", "");

            dataMap.insert("serial", "");
            dataMap.insert("license_b", "");
            dataMap.insert("expiration", init_trial.due_date);
            dataMap.insert("state", TrialAuthorized);
        } else {
            // 已试用
            TrialInfoEntry current_trial = list[0];
            QDateTime current_date = QDateTime::currentDateTime();
            QDateTime record_current_date = QDateTime::fromString(current_trial.current_date, "yyyyMMdd");
            QDateTime due_date = QDateTime::fromString(current_trial.due_date, "yyyyMMdd");
            if (current_date.daysTo(record_current_date) > 0) {
                 dataMap.insert("state", TrialExpired);
            } else {
                if (current_date.daysTo(due_date) >= 0) {
                    dataMap.insert("state", TrialAuthorized);
                } else {
                    dataMap.insert("state", TrialExpired);
                }
                current_trial.current_date = current_date.toString("yyyyMMdd");
            }
            m_kernel->insertTrialInfo(current_trial, true);

            dataMap.insert("product_label", type);
            dataMap.insert("product_name", "");
            // dataMap.insert("product_version", "");

            dataMap.insert("serial", "");
            dataMap.insert("license_b", "");
            dataMap.insert("expiration", current_trial.due_date);
        }
    }
    QJsonDocument docObject = QJsonDocument::fromVariant(dataMap);
    QByteArray data = docObject.toJson();
    LOG(INFO) << QString(data).toStdString();
    return data;
}

QString LicenseInfo::GET_License_B(const QString &type)
{
    QByteArray product_info = ProductInfo(type);
    QString info = QString::fromLatin1(product_info);

    if(info.isEmpty() || info.isNull())
    {
        return "";
    }

    QJsonParseError jsonParseError;
    QJsonDocument document = QJsonDocument::fromJson(info.toUtf8(), &jsonParseError);

    if(jsonParseError.error != QJsonParseError::NoError)
    {
        LOG(INFO) << "release data error!";
        return "";
    }

    QJsonObject object = document.object();
    return object["license_b"].toString();
}

void LicenseInfo::ShowLicenseActivator(const QString &product_label, const QString &product_name, const QString &product_version, bool showProbationPeriod)
{
     // LOG(INFO) << product_label.toStdString() << product_name.toStdString();
     emit showActivator(product_label, product_name, product_version, showProbationPeriod);

//    QProcess* pro = new QProcess(this);
//    QStringList arg;
//    arg.append(label);
//    arg.append(productName);
//    LOG(INFO) << pro->execute("nfs-activator", arg);
//    // pro->setArguments(arg);
//    // pro->start("nfs-activator", arg);
//    LOG(INFO) << "5555555555";

}


QString LicenseInfo::GetLicenseStatus(const QString &type) {
    QList<ProductInfoEntry> list = m_kernel->getLicenseB(type);
    QVariantMap dataMap;
    QString mac = m_kernel->getMacAddress();
    QString serial = m_kernel->getSerial();
    QString sn = m_kernel->getSn();
    if (list.length() >= 1) {
        // 查询授权状态
        ProductInfoEntry res = list[0];
        dataMap.insert("expiration", QString::number(QDateTime::fromString(res.expiration, "yyyyMMdd").toTime_t(), 10));
        if(res.start_time.isEmpty())
        {
            QList<TrialInfoEntry> trial_list = m_kernel->getTrialInfo(type);
            dataMap.insert("start_time", QString::number(QDateTime::fromString(trial_list[0].start_date, "yyyyMMdd").toTime_t(), 10));
        }
        else
        {
            dataMap.insert("start_time", QString::number(QDateTime::fromString(res.start_time, "yyyyMMdd").toTime_t(), 10));
        }

        int res_mac = is_register(toCStr(mac), toCStr(res.license_b), NULL);
        int res_check_serial = -1, res_check_sn = -1;
        if(serial != "")
        {
            res_check_serial = is_register(toCStr(serial), toCStr(res.license_b), NULL);
        }

        if(sn != "")
        {
            res_check_sn = is_register(toCStr(sn), toCStr(res.license_b), NULL);
        }

        if (res_mac == 1 || res_check_serial == 1 || res_check_sn == 1) {
            int state = m_kernel->getAuthorizationState(res.license_b);
            dataMap.insert("state", state);
        } else {
            dataMap.insert("state", Notauthorized);
        }
    } else {
        // 查询试用状态
        QList<TrialInfoEntry> list = m_kernel->getTrialInfo(type);

        if (list.length() == 0) {
            // 未试用
            QDateTime current_date = QDateTime::currentDateTime();
            TrialInfoEntry init_trial;
            init_trial.product_label = type;
            init_trial.current_date = current_date.toString("yyyyMMdd");
            init_trial.remain_days = trial_days;
            init_trial.due_date = current_date.addDays(trial_days).toString("yyyyMMdd");
            init_trial.start_date = current_date.toString("yyyyMMdd");
            m_kernel->insertTrialInfo(init_trial, false);

            dataMap.insert("expiration", QString::number(QDateTime::fromString(init_trial.due_date, "yyyyMMdd").toTime_t(), 10));
            dataMap.insert("state", TrialAuthorized);
        } else {
            // 已试用
            TrialInfoEntry current_trial = list[0];
            QDateTime current_date = QDateTime::currentDateTime();
            QDateTime record_current_date = QDateTime::fromString(current_trial.current_date, "yyyyMMdd");
            QDateTime due_date = QDateTime::fromString(current_trial.due_date, "yyyyMMdd");
            if (current_date.daysTo(record_current_date) > 0) {
                 dataMap.insert("state", TrialExpired);
            } else {
                if (current_date.daysTo(due_date) >= 0) {
                    dataMap.insert("state", TrialAuthorized);
                } else {
                    dataMap.insert("state", TrialExpired);
                }
                current_trial.current_date = current_date.toString("yyyyMMdd");
            }
            m_kernel->insertTrialInfo(current_trial, true);
            dataMap.insert("expiration", QString::number(QDateTime::fromString(current_trial.due_date, "yyyyMMdd").toTime_t(), 10));
        }
    }
    QJsonDocument docObject = QJsonDocument::fromVariant(dataMap);
    QString data = docObject.toJson();
    return data;
}

int LicenseInfo::GetLicenseState(const QString &type)
{
    QString data = GetLicenseStatus(type);

    if(data.isEmpty() || data.isNull())
    {
        return -1;
    }

    QJsonParseError jsonParseError;
    QJsonDocument document = QJsonDocument::fromJson(data.toUtf8(), &jsonParseError);

    if(jsonParseError.error != QJsonParseError::NoError)
    {
        LOG(INFO) << "release data error!";
        return -1;
    }

    QJsonObject object = document.object();
    int state = object["state"].toInt();
    return state;
}

uint LicenseInfo::GetLicenseExpiration(const QString &type)
{
    QString data = GetLicenseStatus(type);

    if(data.isEmpty() || data.isNull())
    {
        return 1;
    }

    QJsonParseError jsonParseError;
    QJsonDocument document = QJsonDocument::fromJson(data.toUtf8(), &jsonParseError);

    if(jsonParseError.error != QJsonParseError::NoError)
    {
        LOG(INFO) << "release data error!";
        return 1;
    }

    QJsonObject object = document.object();
    QString expiration = object["expiration"].toString();
    uint time = expiration.toUInt();
    LOG(INFO) << "expiration=" << expiration << " ,time=" << time;
    return time;
}

bool LicenseInfo::check_net_work(const QString &url)
{
    QTimer timer;
    timer.setInterval(10000);
    timer.setSingleShot(true);

    QString send_url = url + "/api/ping/";
    QNetworkRequest requestInfo;
    requestInfo.setUrl(QUrl(send_url));
    QEventLoop eventLoop;
    QNetworkReply *reply = m_pHttpMgr->get(requestInfo);
    connect(&timer, &QTimer::timeout, &eventLoop, &QEventLoop::quit);
    connect(reply, &QNetworkReply::finished, &eventLoop, &QEventLoop::quit);
    timer.start();
    eventLoop.exec();

    if(timer.isActive())
    {
        timer.stop();
        if (reply->error() == QNetworkReply::NoError)
         {
             qDebug() << "request protobufHttp NoError";
         }
         else
         {
             qDebug()<<"request protobufHttp handle errors here";
             QVariant statusCodeV = reply->attribute(QNetworkRequest::HttpStatusCodeAttribute);
             //statusCodeV是HTTP服务器的相应码，reply->error()是Qt定义的错误码，可以参考QT的文档
             qDebug( "request protobufHttp found error ....code: %d %d\n", statusCodeV.toInt(), (int)reply->error());
             qDebug(qPrintable(reply->errorString()));
             LOG(INFO) << "request protobufHttp handle errors here" << statusCodeV.toInt() << " " << (int)reply->error();
             return false;
         }
         //请求返回的结果
         QByteArray responseByte = reply->readAll();

         LOG(INFO) << "network state reply is " << QString(responseByte);

         if(QString(responseByte).contains("pong"))
         {
             return true;
         }
         else
         {
             return false;
         }
    }
    else
    {
        disconnect(reply, &QNetworkReply::finished, &eventLoop, &QEventLoop::quit);
        reply->abort();
        reply->deleteLater();
        qDebug() << "Timeout";
        LOG(INFO) << "check_net_work Timeout";
        return false;
    }
}
