#include "licensejson.h"
#include "https.h"
#include "licensekernel.h"
#include "systeminfo.h"
#include "logencryption.h"

#include <QCoreApplication>
#include <QJsonArray>
#include <QJsonDocument>
#include <QJsonObject>
#include <QJsonParseError>
#include <QJsonValue>
#include <QString>
#include <QDebug>
#include <QFile>
#include <QDateTime>
#include <QDir>
#include <QNetworkConfigurationManager>
#include "easylogging++.h"

LicenseJson::LicenseJson(QObject *parent)
    : QObject(parent)
{
    m_https = new Https;
    //针对批量离线激活，相同的接口，回来的数据有所区分
    connect(m_https, &Https::receiveHttpsData, this, &LicenseJson::getServerData);
}

LicenseJson::~LicenseJson()
{
    m_https->deleteLater();
}

//解析接受的json
QVariantMap LicenseJson::parseJsonData(const QJsonDocument &jsonDoc)
{
    if (!jsonDoc.isObject())
        return QVariantMap();
    QJsonObject jsonObject = jsonDoc.object();
    return jsonObject.toVariantMap();
}

QJsonDocument LicenseJson::transQBytearrayToJsonDocument(const QByteArray &byte)
{
    QJsonParseError jsonpe;
    QJsonDocument jsonDocument = QJsonDocument::fromJson(byte, &jsonpe);

    if (jsonpe.error == QJsonParseError::NoError)
        return jsonDocument;

    LOG(INFO) << "QJsonParseError " << jsonpe.errorString() << ", jsonDocument:" << QString(jsonDocument.toJson());
    return jsonDocument;
}

QString LicenseJson::getQrcodeSendStr(const QString &curQrcodeStr)
{
    QString path = m_serverQrCodePath.arg(curQrcodeStr);
    QString qrodeUrl = m_serverUrlConfig + path;
    qInfo() << "getQrSendStr:" << LogEncryption::encryptionMessage(qrodeUrl);
    return qrodeUrl;
}

void LicenseJson::sendOnlineActivatePublic(const QString &online_url,
                                           const QString &serial,
                                           const QString &mac,
                                           const QString &disk,
                                           const QString &sn,
                                           const QString &product_label) {
    QJsonObject json;
    json.insert("serial", serial);
    json.insert("mac", mac);
    json.insert("disk", disk);
    json.insert("sn", sn);
    json.insert("product_label", product_label);
    QJsonDocument document;
    document.setObject(json);
    QString url = online_url + m_onlinePulicPath;
    qInfo() << "sendOnlineActivatePublic" << LogEncryption::encryptionMessage(url);

    LOG(INFO) << "license_a:\t" << serial.toStdString();
    LOG(INFO) << "mac:\t" << mac.toStdString();
    LOG(INFO) << "url:\t" << url.toStdString();

    m_https->sendPost(QString(url), document.toJson(QJsonDocument::Compact)); //访问服务器
}

void LicenseJson::sendStartProductC(const QString &kms_url,
                                    const QString &serial,
                                    const QString &sn,
                                    const QString &disk,
                                    const QString &mac,
                                    const QString &note)
{
    QJsonObject json;
    json.insert("serial", serial);
    json.insert("sn", sn);
    json.insert("disk", disk);
    json.insert("mac", mac);
    json.insert("note", note);
    QJsonDocument document;
    document.setObject(json);
    QString url = kms_url + "/api/product/start-product-C/";
    qInfo() << "sendStartProductC" << LogEncryption::encryptionMessage(url);
    LOG(INFO) << QString(QJsonDocument(json).toJson(QJsonDocument::Compact)).toStdString();
    LOG(INFO) << url;

    m_https->sendPost(QString(url), document.toJson(QJsonDocument::Compact)); //访问服务器
}

void LicenseJson::sendOnlineActivatePrivate(const QString &domain_url,
                                            const QString &product_label,
                                            const QString &mac,
                                            const QString &disk,
                                            const QString &sn) {
    QJsonObject json;
    json.insert("product_label", product_label);
    json.insert("mac", mac);
    json.insert("disk", disk);
    json.insert("sn", sn);
    QJsonDocument document;
    document.setObject(json);
    QString url = domain_url + m_onlinePrivatePath;
    qInfo() << "sendOnlineActivatePrivate" << LogEncryption::encryptionMessage(url);
    m_https->sendPost(QString(url), document.toJson(QJsonDocument::Compact)); //访问服务器
}

void LicenseJson::sendStartProductD(const QString &react_url,
                                    const QString &mac,
                                    const QString &sn,
                                    const QString &disk,
                                    const QString &product_label)
{
    QJsonObject json;
    json.insert("mac", mac);
    json.insert("sn", sn);
    json.insert("disk", disk);
    json.insert("label", product_label);
    QJsonDocument document;
    document.setObject(json);

    LOG(INFO) << "url:" << react_url << ", data:" << QString(document.toJson(QJsonDocument::Compact));
    m_https->sendPost(QString(react_url), document.toJson(QJsonDocument::Compact)); //访问服务器
}

void LicenseJson::closeNetworkRequest()
{
    m_https->timeOutDeleteReply();
}

//获取配置服务器地址
void LicenseJson::getServerUrlConf(QVariantMap map)
{
    // const QString onlinePrivatePathName = "onlinePrivatePath";
    // const QString onlinePublicPathName = "onlinePublicPath";
    m_onlinePulicPath = map.value("onlinePublicPath").toString();;
    m_onlinePrivatePath = map.value("onlinePrivatePath").toString();;

    // 服务器二维码路径
    m_serverQrCodePath = map.value("qrcodePath").toString();

    // 是否采用默认设置
    if (map.value("confDefault", true).toBool()) {
        //自定义地址
        m_serverUrlConfig = map.value("defaultUrl").toString();

    } else {
        m_serverUrlConfig = map.value("confCustomUrl").toString();
    }

    //自定义地址
    qInfo() << "server-url= " << LogEncryption::encryptionMessage(m_serverUrlConfig);
}

void LicenseJson::getServerData(int httpCode, const QVariantMap &headerData, const QByteArray &returnData,
                                const QString &postName)

{
    if (returnData.isEmpty())
        qInfo() << "getServerData is empty.";

    LOG(INFO) << "returnData" <<returnData.toStdString();

    if(returnData.isEmpty() || returnData.isNull())
    {
        return;
    }

    QJsonDocument jsonDoc = transQBytearrayToJsonDocument(returnData);
    QVariantMap info = parseJsonData(jsonDoc);

    if (m_https == dynamic_cast<Https *>(sender()))
        emit sendHttpsData(httpCode, headerData, info, postName);
}
