#include "licensekernel.h"
#include "licensekey.h"
#include "systeminfo.h"
// #include "licensemodel.h"
#include "RsaCrypt.h"
#include "sysdevice.h"
#include "base32.h"
#include "logencryption.h"
#include "systemfileoperate.h"
#include "monitornetwork.h"

#include <sqlite3.h>

#include <QDebug>
#include <QTemporaryFile>
#include <QJsonValue>
#include <QJsonArray>
#include <QJsonDocument>
#include <QJsonObject>
#include <QTimer>
#include <QProcess>
#include <QReadWriteLock>
#include <QDBusInterface>
#include <QtDBus>
#include <QDBusReply>

#include <inttypes.h>
#include <QCryptographicHash>
#include <iostream>
#include <dlfcn.h>
#include "check-reg.h"
#include <QSqlQuery>
#include <QVariant>
#include <QNetworkConfigurationManager>
#include "easylogging++.h"

namespace {
const QString unionTechHostName = "http://detect.uniontech.com";
const QString kmsHostPrefix = "kms://";
const QString oemkey = "key";
const QString clientKeyType = "key_type";
const QString serverKeyType = "keyType";
constexpr qint32 minutesToMilliseconds = 60 * 1000;
constexpr qint32 systemRunningTimeHeartBeat = 900000;
constexpr qint32 twentyFiveSerialNumberLength = 25;
constexpr qint32 licenseServerIndex = 0;

const QString licenseDBPath = "/var/nfs/license.db";
}

class LicenseKernelPrivate
{
public:
    LicenseKernelPrivate(LicenseKernel *parent)
        : q_ptr(parent)
    {
        m_licenseModel.license.actived = Unactivated;
        m_licenseModel.license.authorizationState = Notauthorized;
    }

    struct LicenseModel m_licenseModel;
    struct ServerSystemInfo m_serverSystemInfo;
    struct OSVersionInfo m_osVersionInfo;
    QString m_activeCode;
    QString m_serverTime;
    QString m_safeVerifyTime;
    QString m_systemLanguage;
    QDateTime m_serverDateTime;
    LicenseJson *m_licenseJson;
    QVariantMap m_licenseJsonConfMap;
    void *m_eccCurveInst = Q_NULLPTR;
    bool eccInitFlag;
    bool eccSerInitFlag;
    LicenseKernel *q_ptr;
    Q_DECLARE_PUBLIC(LicenseKernel)
};

LicenseKernel::LicenseKernel(QObject *parent)
    : QObject(parent)
    , m_hasVerifyKey(true)
    , dd_ptr(new LicenseKernelPrivate(this))
    , m_isNeedLogin(true)
    , m_isDiagnostics(false)
    , m_helthRequestNormal(false)
    , m_kmsActiveCodeUsedTimes(-1)
    , m_previousAuthState(Notauthorized)
{
    Q_D(LicenseKernel);
    // database
    m_licenseDB = new QSqlCipher(licenseDBPath, this);
    // initServer

    m_networkStatus = InitNetworkState;
    m_systemInfo = new SystemInfo();
    m_hardWareInfo = m_systemInfo->getHardWareInfo();
    d->m_licenseJson = new LicenseJson;
    m_isNetworkConnecting = false;
    m_isTimeout = false;
    m_msgType = True;

    // serverConfCompatiter();
    m_openModel = true;
    SystemFileOperate::readOsVersionInfo(d->m_osVersionInfo);
    SystemFileOperate::readServerSystemInfo(d->m_serverSystemInfo);
    SystemFileOperate::writeConfigIniFile();
    parseSystemType();
    //获取配置服务器地址
    initServerUrlConf();
    parseServerConf();

    initLanguage();
    m_key.clear();
    m_operationType = NoOperation;
    m_fileValidityEnd = QString("");
    // checkNetworkTimeout
    m_checkNetworkTimer = new QTimer(this);
    m_kmsTcpConnect = new KMSTcpClient(this);
    m_kmsNetworkTimer = new QTimer(this);
    m_kmsHeartTimer = new QTimer(this);
    m_systemRunningTimer = new QTimer(this);
    connect(m_checkNetworkTimer, SIGNAL(timeout()), this, SLOT(checkNetworkTimeout()));
    connect(m_kmsNetworkTimer, SIGNAL(timeout()), this, SLOT(kmsNetworkTimeout()));
    connect(m_kmsHeartTimer, SIGNAL(timeout()), this, SLOT(checkKMSServer()));
    connect(m_systemRunningTimer, SIGNAL(timeout()), this, SLOT(writeTrialInfoFile()));
    m_unactivatedTipTimer = new QTimer(this);
//    connect(m_unactivatedTipTimer, SIGNAL(timeout()), this, SLOT(unactivatedBalloonTip()));
//    m_unactivatedTipTimer->start(300000);
    connect(d->m_licenseJson, &LicenseJson::sendHttpsData, this, &LicenseKernel::parseHttpsData);
    connect(m_kmsTcpConnect, SIGNAL(receData()), this, SLOT(parseKmsInitInfo()));
    d->m_licenseModel.system.netState = InitNetworkState;

    m_previousServerAddr = m_currentServerUrl;
    m_previousKmsDomain = m_previousServerAddr;
    m_vaildActiveCode = m_activeFileData.value(1, "");
    initABRecovery();
    initCheckUuid();
    // startGetLicenseStateTimer();
    // writeAuthConf();
}

LicenseKernel::~LicenseKernel()
{
    Q_D(LicenseKernel);
    if (m_checkNetworkTimer != nullptr) {
        m_checkNetworkTimer->deleteLater();
        m_checkNetworkTimer = nullptr;
    }
    if (m_unactivatedTipTimer != nullptr) {
        m_unactivatedTipTimer->deleteLater();
        m_unactivatedTipTimer = nullptr;
    }
    if (m_systemInfo != nullptr) {
        m_systemInfo->deleteLater();
        m_systemInfo = nullptr;
    }

    releaseEccResources();
}

void LicenseKernel::activateWithInlineURL(const QString &url, const QString &product_lab)
{
    Q_D(LicenseKernel);
    // http://172.30.81.226:81
    if (isNetWorkOnline()) {
        QString mac = m_systemInfo->getMacAddress(false).toLower();
        QString disk = getSerial();
        QString sn = getSn();
        d->m_licenseJson->sendOnlineActivatePrivate(url, product_lab, mac, disk, sn);
    } else {
        QString error_info = "网络无法连接";
        emit sendActivateErrorInfo(error_info);
    }
}

void LicenseKernel::activateWithOnlineURL(const QString& online_url, const QString& serial, const QString& product_label) {
    Q_D(LicenseKernel);

    if (isNetWorkOnline()) {
        QString mac = m_systemInfo->getMacAddress(false).toLower();
        QString disk = getSerial();
        QString sn = getSn();
        LOG(INFO) << "mac=" << mac << ", disk=" << disk << ", sn=" << sn;
        d->m_licenseJson->sendOnlineActivatePublic(online_url, serial, mac, disk, sn, product_label);
    } else {
        QString error_info = "网络无法连接";
        emit sendActivateErrorInfo(error_info);
    }

}

bool LicenseKernel::activateWithSerialB(const QString& license_b, const QString& product_name, const QString &serial) {
    Q_D(LicenseKernel);
    LOG(INFO) << "serial_b:" << license_b.toStdString();
    LOG(INFO) << "mac" << getMacAddress().toStdString();
    int res = is_register(toCStr(getMacAddress()), toCStr(license_b), NULL); 
    LOG(INFO) << "res = " << res << "," << getMacAddress().toStdString() << "," << license_b.toStdString();

    int res_check_serial = -1, res_check_sn = -1;
    if(getSerial() != "")
    {
        res_check_serial = is_register(toCStr(getSerial()), toCStr(license_b), NULL);
        LOG(INFO) << "res_check_serial = " << res_check_serial << "," << getSerial().toStdString() << "," << license_b.toStdString();
    }

    QFileInfo dir("/run/live/");
    if(dir.isDir())
    {
        res_check_sn = is_register(toCStr(m_sn), toCStr(license_b), NULL);
        LOG(INFO) << "res_check_sn = " << res_check_sn << ",m_sn = " << m_sn << "," << license_b.toStdString();
    }
    else
    {
        if(getSn() != "")
        {
            res_check_sn = is_register(toCStr(getSn()), toCStr(license_b), NULL);
            LOG(INFO) << "res_check_sn = " << res_check_sn << "," << getSn().toStdString() << "," << license_b.toStdString();
        }
    }

    if (res == 1 || res_check_serial == 1 || res_check_sn == 1) {
        LOG(INFO) << "success:";
        if (getAuthorizationState(license_b) == Authorized) {
            ProductInfoEntry productInfo;
            productInfo.product_label = get_product_label(toCStr(license_b));
            productInfo.product_name = product_name;
            // productInfo.product_version = product_version;
            productInfo.serial =serial;
            productInfo.license_b = license_b;
            productInfo.expiration = get_expiration(toCStr(license_b));
            QDateTime current_date = QDateTime::currentDateTime();
            productInfo.start_time = current_date.toString("yyyyMMdd");
            productInfo.offline_time = current_date.toString("yyyyMMddhhmmss.zzz");
            productInfo.recheck_duration = "";
            productInfo.recheck = 1;
            writeLicenseDB(productInfo);
            emit activeStateChange();
            emit licenseSuccess(productInfo.product_label);
            return true;
        } else {
            QString error_info = "激活已过期";
            emit sendActivateErrorInfo(error_info);
            return false;
        }
    } else {
        LOG(INFO) << "failed:";
        QString error_info = "校验失败";
        emit sendActivateErrorInfo(error_info);
        return false;
    }
}

QString LicenseKernel::getSerial()
{
    QProcess process;
    process.start("lsblk -J -bno NAME,SERIAL,MOUNTPOINT,UUID");
    process.waitForFinished(-1);
    QString data = process.readAllStandardOutput();
    process.close();
    process.kill();

    if(data.isNull() || data.isEmpty())
    {
        return "";
    }

    QString serial = "";
    QJsonParseError Err;

    QJsonDocument doc = QJsonDocument::fromJson(data.toLocal8Bit(), &Err);
    if(!Err.error == QJsonParseError::NoError)
    {
        LOG(INFO) << "json格式解析错误";
        return "";
    }

    QJsonObject obj = doc.object();
    if(obj.contains("blockdevices"))
    {
        QJsonValue value = obj.value("blockdevices");
        if(value.isArray())
        {
            LOG(INFO) << "blockdevices is array";
            QJsonArray obj_array = value.toArray();
            for(int i=0; i < obj_array.size(); i++)
            {
                QJsonObject device_obj = obj_array[i].toObject();
                QJsonValue children = device_obj.value("children");
                if(children.isArray())
                {
                    LOG(INFO) << "children is array";
                    QJsonArray array = children.toArray();
                    for(int j=0; j < array.size(); j++)
                    {
                        QJsonValue nameArray = array[j];
                        QJsonObject key = nameArray.toObject();
                        QString mountpoint = key.value("mountpoint").toString();
                        LOG(INFO) << "mountpoint=" << mountpoint;
                        if(mountpoint == "/")
                        {
                            serial = device_obj.value("serial").toString();
                            LOG(INFO) << "serial=" << serial;
                            if(serial == "")
                            {
                                serial = key.value("uuid").toString();
                                LOG(INFO) << "serial=null, uuid=" << serial;
                                return serial;
                            }
                        }
                        else
                        {
                            if(mountpoint.contains("/tmp/calamares-root"))
                            {
                                serial = device_obj.value("serial").toString();
                                LOG(INFO) << "calamares-root serial=" << serial;
                                return serial;
                            }
                        }
                    }
                }
            }
        }
    }
    if(serial == "")
    {
        FILE* fd=NULL;
        char  buff[1024]={0};
        char  disk_serial[1024]={0};
        fd = popen("lsblk -nbo NAME,UUID,SERIAL,MOUNTPOINT","r");
        if (NULL != fd)
        {
            int flag = 0;
            while (fgets(buff, sizeof(buff), fd) != NULL)
            {
                if(isalpha(buff[0]))
                {
                    sscanf(buff, "%*s %s", disk_serial);
                    continue;
                }
                else
                {
                    char tmp = buff[strlen(buff)-2];
                    if(tmp == '/')
                    {
                        // printf("%s\n", disk_serial);
                        serial = QString::fromStdString(std::string(disk_serial));
                        break;
                    }
                }
            }
        }
        pclose(fd);
    }
    return serial;
}

bool LicenseKernel::executeCmd(const QString &cmd, QString &result)
{
    QProcess proc;
    proc.start("bash", {"-c", cmd});
    proc.waitForFinished();
    result = QString::fromLatin1(proc.readAll());
    proc.close();

    return (!result.isEmpty());
}

QString LicenseKernel::getSn()
{
//    QProcess process;
//    process.start("dmidecode -s system-serial-number");
//    process.waitForFinished(-1);
//    QString sn = process.readAllStandardOutput();
//    process.close();
//    process.kill();

//    if(sn.isEmpty() || sn.isNull())
//    {
//        return "";
//    }

//    return sn.remove("\n");
    QString sn;
    QString cmdOut;
    QString cmd = "dmidecode -s system-serial-number";
    executeCmd(cmd, cmdOut);

    if (cmdOut.isEmpty())
        return sn;

    sn = cmdOut.remove("\n");
    return sn;
}

void LicenseKernel::ImportStartProductC(const QString &kms_url, const QString &serial, const QString &sn, const QString &note)
{
    Q_D(LicenseKernel);

    if (isNetWorkOnline()) {
        QString mac = m_systemInfo->getMacAddress(false).toLower();
        QString disk = getSerial();
        m_sn = sn;
        LOG(INFO) << "mac=" << mac << ", disk=" << disk << ", serial=" << serial << ", note=" << note;

        d->m_licenseJson->sendStartProductC(kms_url, serial, sn, disk, mac, note);
    } else {
        QString error_info = "网络无法连接";
        emit sendActivateErrorInfo(error_info);
    }
}

void LicenseKernel::activateWithStartProductD(const QString &url, const QString &product_lab)
{
    Q_D(LicenseKernel);

    if (isNetWorkOnline()) {
        QString mac = m_systemInfo->getMacAddress(false).toLower();
        QString disk = getSerial();
        QString sn = getSn();
        QString react_url = url + "/api/product/start-product-D/";
        LOG(INFO) << "react_url" << react_url << ", sn=" << sn;

        d->m_licenseJson->sendStartProductD(react_url, mac, sn, disk, product_lab);
    } else {
        QString error_info = "网络无法连接";
        emit sendActivateErrorInfo(error_info);
    }
}

bool LicenseKernel::isNetWorkOnline() {
    QNetworkConfigurationManager mgr;
    return mgr.isOnline();
}

bool LicenseKernel::checkOsLicense()
{
    Q_D(LicenseKernel);
    SystemFileOperate::readServerSystemInfo(d->m_serverSystemInfo);
    if (!verifyProductId())
        return false;

    SystemFileOperate::readOsVersionInfo(d->m_osVersionInfo);

    if (d->m_serverSystemInfo.version.isEmpty() || d->m_osVersionInfo.majorVersion.isEmpty()
            || d->m_serverSystemInfo.productName.isEmpty() || d->m_osVersionInfo.productTypeCH.isEmpty()
            || d->m_serverSystemInfo.systemChinese.isEmpty() || d->m_osVersionInfo.editionNameCH.isEmpty()) {
        qInfo() << "osfile field is empty!";
        return false;
    }

    if (0 != d->m_serverSystemInfo.version.compare(d->m_osVersionInfo.majorVersion)
            || 0 != d->m_serverSystemInfo.productName.compare(d->m_osVersionInfo.productTypeCH)
            || 0 != d->m_serverSystemInfo.systemChinese.compare(d->m_osVersionInfo.editionNameCH)) {
        qInfo() << "Compare osfile diffent!";
        return false;
    }

    return true;
}

bool LicenseKernel::verifyProductId()
{
    Q_D(const LicenseKernel);
    QMap<int, QString> uosVersionMap;
    uosVersionMap.insert(0, "20");
    QMap<int, QString> productTypeMap;
    productTypeMap.insert(0, "桌面");
    productTypeMap.insert(1, "服务器");
    productTypeMap.insert(2, "专用设备");
    QMap<int, QStringList> systemDesktopTypeMap;
    systemDesktopTypeMap.insert(0, {"专业版"});
    systemDesktopTypeMap.insert(1, {"个人版"});
    systemDesktopTypeMap.insert(2, {"社区版"});
    systemDesktopTypeMap.insert(3, {"军用版"});
    systemDesktopTypeMap.insert(4, {"专用设备版"});
    QMap<int, QStringList> systemServerTypeMap;
    systemServerTypeMap.insert(0, {"企业版", "d"});
    systemServerTypeMap.insert(1, {"行业版", "a"});
    systemServerTypeMap.insert(2, {"欧拉版", "e"});
    systemServerTypeMap.insert(3, {"军用版"});
    systemServerTypeMap.insert(4, {"专用设备版"});
    systemServerTypeMap.insert(5, {"行业版", "c"});
    QByteArray ba = d->m_serverSystemInfo.productId.toLocal8Bit();
    QByteArray productIDSwap(ba.size(), 0);
    memset(productIDSwap.data(), 0xFF, static_cast<size_t>(ba.size()));
    base32_decode(ba.data(), reinterpret_cast<unsigned char *>(productIDSwap.data()));
    if (productIDSwap.size() < 4) {
        qInfo() << "productId length is invalid!";
        return false;
    }

    productIDSwap.data()[3] &= 0X0F;

    int uosType = productIDSwap.data()[0] & 0x0f; //总类（操作系统）
    int verison = (productIDSwap.data()[0] >> 4) & 0x0f; // verison
    int productType = productIDSwap.data()[1] & 0x0f; // product type
    int systemType = (productIDSwap.data()[1] >> 4) & 0x0f; // system type
    int archType = productIDSwap.data()[2] & 0x0f; // instruction type
    int oem = ((productIDSwap.data()[2] >> 4) & 0x0F) | ((productIDSwap.data()[3] << 4) & 0xF0); // OEM

    Q_UNUSED(uosType);
    Q_UNUSED(archType);
    Q_UNUSED(oem);

    if (uosVersionMap.value(verison) != d->m_serverSystemInfo.version) {
        qInfo() << "Compare majorVersion diffent!" << verison;
        return false;
    }

    if (productTypeMap.value(productType) != d->m_serverSystemInfo.productName) {
        qInfo() << "Compare product_name diffent!" << productType;
        return false;
    }

    if (0 == productType) {
        //桌面版
        if (!systemDesktopTypeMap[systemType].contains(d->m_serverSystemInfo.systemChinese)) {
            qDebug() << "Compare desktopProduct diffent!" << systemType;
            return false;
        }
    } else if (1 == productType) {
        //服务器版
        if (!systemServerTypeMap[systemType].contains(d->m_serverSystemInfo.systemChinese)) {
            qDebug() << "Compare serverProduct diffent!" << systemType;
            return false;
        }
    } else {
        //专用设备
    }

    return true;
}

quint32 LicenseKernel::activateStatus() const
{
    Q_D(const LicenseKernel);
    return d->m_licenseModel.license.actived;
}

void LicenseKernel::sendQrcodeUrlAddress()
{
    Q_D(LicenseKernel);

    // HW_KLU中缺失cid不允许进行激活操作
    if (m_systemInfo->getIsKLU() && m_systemInfo->getKLUCID().isEmpty()) {
        emit activeMessageChange(HWKLUSystemLostCID);
        return;
    }

    //auto qrcodeMap = getOfflineUniqueUuidMap();
    QString tempQrcodeStr = ""; // operateMap(qrcodeMap);
    QString qrcodeUrl = d->m_licenseJson->getQrcodeSendStr(tempQrcodeStr);
    qInfo() << "emit sendurl(qrcodeUrl) = " << LogEncryption::encryptionMessage(qrcodeUrl);
    emit sendQrcodeUrl(qrcodeUrl);
}

void LicenseKernel::sendReplaceServerUnconnectMessage()
{
    m_checkNetworkTimer->stop();
    m_isNetworkConnecting = false;
    Q_EMIT activeMessageChange(ReplaceUnconnect);
}

void LicenseKernel::initLanguage()
{
    Q_D(LicenseKernel);

    QString userLanguage = QString();
    QStringList environment = QProcess::systemEnvironment();
    foreach (auto env, environment) {
        if (env.contains("LANG=")) {
            userLanguage = env.split("=").at(1);
            break;
        }
    }

    qDebug() << "system Language:" << userLanguage;

    if (userLanguage.isEmpty()) {
        userLanguage = "zh_CN";
    } else {
        QStringList userNameList = userLanguage.split(".");
        if (userNameList.size() < 2) {
            userLanguage = "zh_CN";
        } else {
            userLanguage = userNameList[0];
        }
    }

    d->m_systemLanguage = userLanguage;
}

unsigned int LicenseKernel::eccVerifySerialNumber(const QString &key)
{
    Q_D(LicenseKernel);

//    if (key.isEmpty()) {
//        qInfo() << "key is empty";
//        return ECC_ERR_UNKNOWEN;
//    }

//    QByteArray ba = d->m_serverSystemInfo.productId.toLocal8Bit();
//    QByteArray productIDSwap(ba.size(), 0);
//    memset(productIDSwap.data(), 0xFF, static_cast<size_t>(ba.size()));
//    base32_decode(ba.data(), reinterpret_cast<unsigned char *>(productIDSwap.data()));
//    if (productIDSwap.size() < 4) {
//        qInfo() << "productId length is invalid!";
//        return ECC_ERR_UNKNOWEN;
//    }

//    productIDSwap.data()[3] &= 0X0F;
//    unsigned int ret = ecc_verify_ser(reinterpret_cast<unsigned char *>(productIDSwap.data()),
//                                      reinterpret_cast<char *>(const_cast<char *>(key.toLocal8Bit().data())));
//    if (ret != ECC_SUCC) {
//        qInfo() << "verify key defeat! " << ret;
//        return ret;
//    }

//    return ret;
    return 0;
}

quint32 LicenseKernel::compatiblePreviousAuthobjectDisplayed(const quint32 &authObjectHidden)
{
    Q_D(LicenseKernel);
    if (NullAuthObjectHidden == authObjectHidden) {
        if (!d->m_licenseModel.license.activeObject.isEmpty())
            return AuthObjectShowed;
        else
            return AuthObjectHidden;
    }

    return authObjectHidden;
}

bool LicenseKernel::isHardwareInfoValid()
{
    Q_D(LicenseKernel);

    auto isMacVerify = [&](const QString &systemMac, const QString &curSystemMac) {
        QStringList macList = systemMac.split(",");
        qSort(macList.begin(), macList.end());
        QStringList curMacList = curSystemMac.split(",");
        qSort(curMacList.begin(), curMacList.end());
        if (macList.join("").compare(curMacList.join("")) == 0) {
            return true;
        }
        qInfo() << "mac unmatch: ";
        qInfo() << "file :" << LogEncryption::encryptionMessage(systemMac);
        qInfo() << "current :" << LogEncryption::encryptionMessage(curSystemMac);
        return false;
    };

    if (m_hardWareInfo.is_vm == 1) {
        QString systemSystemID = m_activeFileData.value(LicenseNameSystemID, "");
        QString curSystemSystemID = m_hardWareInfo.systemId;
        QString systemMac = m_activeFileData.value(LicenseNameMac, "");
        QString curSystemMac = m_hardWareInfo.mac;
        if (curSystemSystemID.compare(systemSystemID) == 0 && isMacVerify(systemMac, curSystemMac))
            return true;
        qInfo() << "VM SystemID unmatch:";
        qInfo() << "file :" << LogEncryption::encryptionMessage(systemSystemID);
        qInfo() << "current :" << LogEncryption::encryptionMessage(curSystemSystemID);
    } else {
        if (SystemType::Server == d->m_licenseModel.system.systemType
            || SystemType::ServerDevice == d->m_licenseModel.system.systemType) {
            QString systemBoardCpu = m_activeFileData.value(LicenseNameBoardCpu, "");
            QString curSystemBoardCpu = m_hardWareInfo.board_cpu;
            QString systemMac = m_activeFileData.value(LicenseNameMac, "");
            QString curSystemMac = m_hardWareInfo.mac;
            if (curSystemBoardCpu.compare(systemBoardCpu) == 0 && isMacVerify(systemMac, curSystemMac))
                return true;
            qInfo() << "Server BoardCpu unmatch: ";
            qInfo() << "file :" << LogEncryption::encryptionMessage(systemBoardCpu);
            qInfo() << "current :" << LogEncryption::encryptionMessage(curSystemBoardCpu);
        } else {
            QString systemA6 = m_activeFileData.value(LicenseNameA6, "");
            QString curSystemA6 = m_hardWareInfo.a6;
            if (curSystemA6.compare(systemA6) == 0)
                return true;
            qInfo() << "A6 unmatch: ";
            qInfo() << "file a6 :" << LogEncryption::encryptionMessage(systemA6);
            qInfo() << "current a6 :" << LogEncryption::encryptionMessage(curSystemA6);
        }
    }
    return false;
}

bool LicenseKernel::isProductIdValid()
{
    Q_D(LicenseKernel);

    SystemFileOperate::readServerSystemInfo(d->m_serverSystemInfo);

    QString systemProductId;
    if (m_currentServerUrl.startsWith(kmsHostPrefix))
        systemProductId = m_activeFileData.value(KmsLicenseNameProductId, "");
    else
        systemProductId = m_activeFileData.value(LicenseNameProductId, "");

    if(systemProductId.isEmpty())
        return true;

    QString curProductId = d->m_serverSystemInfo.productId;
    if (systemProductId.compare(curProductId) == 0)
        return true;

    qInfo() << "ProductId unmatch: ";
    qInfo() << "file ProductId :" << LogEncryption::encryptionMessage(systemProductId);
    qInfo() << "current ProductId :" << LogEncryption::encryptionMessage(curProductId);
    return false;
}

QByteArray LicenseKernel::activationFileDencrypt(QByteArray src)
{
    const QChar key = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/';

    for (int i = 0; i < src.count(); i++)
        src[i] = src[i] ^ key.toLatin1();

    return src;
}

//剩余天数（试用期）
QString LicenseKernel::validityTrial() const
{
    Q_D(const LicenseKernel);
    return d->m_licenseModel.license.validityTrial;
}

QString LicenseKernel::validityEnd() const
{
    Q_D(const LicenseKernel);
    return d->m_licenseModel.license.validityEnd;
}

quint32 LicenseKernel::authorizationState() const
{
    Q_D(const LicenseKernel);
    return d->m_licenseModel.license.authorizationState;
}

quint32 LicenseKernel::activeResetState() const
{
    Q_D(const LicenseKernel);
    return d->m_licenseModel.license.activeResetState;
}

quint32 LicenseKernel::activeMethod() const
{
    Q_D(const LicenseKernel);
    return d->m_licenseModel.license.activeMethod;
}

quint32 LicenseKernel::activeMessageType() const
{
    Q_D(const LicenseKernel);
    return d->m_licenseModel.license.activeMsgType;
}

QString LicenseKernel::activeOverActionState() const
{
    Q_D(const LicenseKernel);
    return d->m_licenseModel.license.activeOverAction;
}

QString LicenseKernel::activeSign() const
{
    Q_D(const LicenseKernel);
    return d->m_licenseModel.license.activeSign;
}

QString LicenseKernel::activeCode() const
{
    Q_D(const LicenseKernel);
    return d->m_activeCode;
}

QString LicenseKernel::activeObject() const
{
    Q_D(const LicenseKernel);
    return d->m_licenseModel.license.activeObject;
}

quint32 LicenseKernel::sendAuthObjectHidden() const
{
    Q_D(const LicenseKernel);
    qDebug() << "sendAuthObjectHidden:" << d->m_licenseModel.license.authObjectHidden;
    return d->m_licenseModel.license.authObjectHidden;
}

int LicenseKernel::systemType() const
{
    Q_D(const LicenseKernel);
    return d->m_licenseModel.system.systemType;
}

QString LicenseKernel::machineCode() const
{
    Q_D(const LicenseKernel);
    return d->m_licenseModel.license.uuid;
}

QString LicenseKernel::authorizeMode() const
{
    Q_D(const LicenseKernel);
    return d->m_serverSystemInfo.keyType.isEmpty() ? openKeyType : d->m_serverSystemInfo.keyType;
}

QString LicenseKernel::authorizeVersion() const
{
    //return CVERSION;
    return "nfs";
}
QString LicenseKernel::authorizeUrl() const
{
    return m_currentServerUrl;
}

void LicenseKernel::getServerUrlConf(QVariantMap map)
{
    Q_D(LicenseKernel);
    //获取配置服务器地址
    d->m_licenseJson->getServerUrlConf(map);
}

void LicenseKernel::initServerUrlConf()
{
    Q_D(LicenseKernel);

    bool confDefault = true;
    QString defaultUrl("http://cloudaccount.os.nfschina.com");
    QString confCustomUrl("");
    QString qrcodePath("/mainweb/activateQRCode/");
    QString onlinePublicPath("/api/product/start-product-on-line-other/");
    QString onlinePrivatePath("/api/product/start-product-off-line-A/");

    QJsonObject jsonObject;
    QByteArray fileData;
    fileData = m_licenseFileOperate.readActiveFile(SERVER_CONF);
    if (parsingJson(fileData, jsonObject)) {
        if (jsonObject.contains(confDefaultName))
            confDefault = jsonObject.value(confDefaultName).toBool();

        if (jsonObject.contains(defaultUrlName))
            defaultUrl = jsonObject.value(defaultUrlName).toString();

        if (jsonObject.contains(confCustomUrlName))
            confCustomUrl = jsonObject.value(confCustomUrlName).toString();

        if (jsonObject.contains(qrcodePathName))
            qrcodePath = jsonObject.value(qrcodePathName).toString();

        if (jsonObject.contains(onlinePublicPathName))
            onlinePublicPath = jsonObject.value(onlinePublicPathName).toString();

        if (jsonObject.contains(onlinePrivatePathName))
            onlinePrivatePath = jsonObject.value(onlinePrivatePathName).toString();
    }

    d->m_licenseJsonConfMap.insert(confDefaultName, confDefault);    
    d->m_licenseJsonConfMap.insert(defaultUrlName, defaultUrl);
    d->m_licenseJsonConfMap.insert(confCustomUrlName, confCustomUrl);
    d->m_licenseJsonConfMap.insert(qrcodePathName, qrcodePath);
    d->m_licenseJsonConfMap.insert(onlinePublicPathName, onlinePublicPath);
    d->m_licenseJsonConfMap.insert(onlinePrivatePathName , onlinePrivatePath);

    getServerUrlConf(d->m_licenseJsonConfMap); //获取配置服务器地址
}

void LicenseKernel::setCustomServerUrl(const QString &customServerUrl) {
    Q_D(LicenseKernel);

    QFile file(SERVER_CONF);
    if (!file.open(QIODevice::WriteOnly | QIODevice::Text)) {
        qWarning() << "license.json open failed.";
        emit errorMessage(SetServerUrlFail);
        return;
    }

    d->m_licenseJsonConfMap.insert(confDefaultName, false);
    d->m_licenseJsonConfMap.insert(confCustomUrlName, customServerUrl);
    //获取配置服务器地址
    getServerUrlConf(d->m_licenseJsonConfMap);

    QJsonDocument jsonDoc = QJsonDocument::fromVariant(QVariant(d->m_licenseJsonConfMap));
    QByteArray data = jsonDoc.toJson();

    if (-1 == file.write(data)) {
        qWarning() << "license.json write failed.";
        emit errorMessage(SetServerUrlFail);
        file.close();
        return;
    }

    file.close();
}

//设置默认激活服务器地址
void LicenseKernel::setDefaultServerUrl()
{
    Q_D(LicenseKernel);

    QFile file(SERVER_CONF);

    if (!file.open(QIODevice::WriteOnly | QIODevice::Text)) {
        qWarning() << "license.json open failed.";
        emit errorMessage(SetServerUrlFail);
        return;
    }

    d->m_licenseJsonConfMap.insert("confDefault", true);
    //获取配置服务器地址
    getServerUrlConf(d->m_licenseJsonConfMap);
    QJsonDocument jsonDoc = QJsonDocument::fromVariant(QVariant(d->m_licenseJsonConfMap));
    QByteArray data = jsonDoc.toJson();

    if (-1 == file.write(data)) {
        qWarning() << "license.json write failed.";
        emit errorMessage(SetServerUrlFail);
        file.close();
        return;
    }

    file.close();
}

void LicenseKernel::setServerUrlInit()
{
    m_key.clear();
    m_kmsActiveCodeUsedTimes = -1;
    m_previousServerAddr = m_currentServerUrl;
    m_msgType = True;

    if (m_kmsTcpConnect->isConnected()) {
        m_kmsTcpConnect->disconnectServer();
    }
}

//获取服务器配置
QByteArray LicenseKernel::getServerConfUrl()
{    
    Q_D(LicenseKernel);

    initServerUrlConf();

    QVariantMap dataMap;
    dataMap.insert(confDefaultName, d->m_licenseJsonConfMap.value(confDefaultName));
    dataMap.insert(confCustomUrlName, d->m_licenseJsonConfMap.value(confCustomUrlName));
    dataMap.insert(defaultUrlName, d->m_licenseJsonConfMap.value(defaultUrlName));
    dataMap.insert(qrcodePathName, d->m_licenseJsonConfMap.value(qrcodePathName));
    QJsonDocument docObject = QJsonDocument::fromVariant(dataMap);
    QByteArray data = docObject.toJson();
    LOG(INFO) << "66666666666";
    LOG(INFO) << QString(data);
    return data;
}

QString LicenseKernel::getIdentifyUrl(const QString &fullUrl)
{
    if (fullUrl.startsWith("kms://")) {
        if (m_kmsDomainName.isEmpty())
            return "https://license.chinauos.com";
        else
            return getIdentifyUrl(m_kmsDomainName);
    } else {
        return fullUrl;
    }
}

void LicenseKernel::addNewTokenField(QMap<QString, QVariant> &token)
{
    Q_D(LicenseKernel);

    static const QMap<AuthorizationState, AptAuthorizationState> authorizationStateMap = {
        { Notauthorized, IsNotauthorized },
        { Authorized, IsAuthorized },
        { Expired, IsExpired },
        { TrialAuthorized, IsTrialAuthorized },
        { TrialExpired, IsTrialExpired }
    };
    AuthorizationState authorizationState =
            static_cast<AuthorizationState>(d->m_licenseModel.license.authorizationState);
    if (d->m_licenseModel.license.activeMethod == OffLineTrialActive) {
        token.insert("x", Apt_OffLineTrialActive);
        token.insert("u", authorizationStateMap[authorizationState]);
    } else if (d->m_licenseModel.license.activeMethod == VolumeActive) {
        token.insert("x", Apt_VolumeOfflineActive);
    }
}

void LicenseKernel::addOpenOemField(QMap<QString, QVariant> &token)
{
    Q_D(LicenseKernel);
    token.insert("w", d->m_serverSystemInfo.keyType);
    token.insert("aa", d->m_serverSystemInfo.key);
}

void LicenseKernel::addIsResetField(QMap<QString, QVariant> &token)
{
    Q_D(LicenseKernel);
    if (d->m_licenseModel.license.activeResetState == IsResetState)
        token.insert("ac", IsResetState);
}

void LicenseKernel::initCheckUuid()
{
    Q_D(LicenseKernel);

    //判断文件存在
    QString uuid = m_activeFileData.value(0, "");
    //本地激活文件为sp2及以上版本且其中的uuid有效则直接使用
    if (isLicenseFileValid() && SP2_LICENSE_FILE_LENGTH <= m_activeFileData.size()
            && (UUID_LENGTH == uuid.size() || TEMP_UUID_LENGTH == uuid.size())) {
        d->m_licenseModel.license.uuid = uuid;
        qInfo() << "formal uuid is valid";
        if (writeClientCheckNumFile())
            m_hasVerifyKey = true;
        return;
    }

    // uuid不为16位的临时uuid则需重置
    if (TEMP_UUID_LENGTH != d->m_licenseModel.license.uuid.size()) {
        QUuid id = QUuid::createUuid();
        d->m_licenseModel.license.uuid = id.toString().remove("{").remove("-").mid(0, 16);
    }

    if (writeClientCheckNumFile())
        m_hasVerifyKey = true;
    m_isNeedLogin = true;
}

bool LicenseKernel::writeClientCheckNumFile()
{
    //判断文件存在
    QFile file(CLIENT_VERIFY);

    QString checkSum = makeClientCheckSum(m_vaildActiveCode);
    QDir dir = QDir("/var/nfs");

    if (!dir.exists()) {
        dir.mkpath("/var/nfs");
    }

    if (!file.open(QIODevice::Truncate | QIODevice::ReadWrite)) {
        qWarning() << "ClientCheckNumFile open error";
        return false;
    }

    // 清空文件中的原有内容
    file.resize(0);
    file.write(checkSum.toLocal8Bit());
    file.close();

    return true;
}

QString LicenseKernel::makeClientCheckSum(QString tempActive)
{
    Q_D(LicenseKernel);
    QString productId = d->m_serverSystemInfo.productId;
    QString product_name = d->m_serverSystemInfo.productName;
    QString instruction_type = d->m_serverSystemInfo.instructionType;
    QString version = d->m_serverSystemInfo.version;
    QString minorVersion = d->m_serverSystemInfo.minorVersion;
    QString buildId = d->m_serverSystemInfo.buildId;
    QString systemCn = d->m_serverSystemInfo.systemChinese;
    QString systemEn = d->m_serverSystemInfo.systemEnglish;
    QString serverSum = d->m_serverSystemInfo.serverSum;
    QString systemId = m_hardWareInfo.systemId;
    QString key = d->m_serverSystemInfo.key;
    QString keyType = d->m_serverSystemInfo.keyType;
    //组织MD5
    QString uuid = d->m_licenseModel.license.uuid;
    QString curData = buildId + systemCn + version.toLatin1().toHex() + minorVersion + productId.toLatin1().toHex()
                      + uuid + serverSum.toLatin1().toHex() + systemEn + tempActive + systemId.toLatin1().toHex();
    if (!m_openModel) {
        curData = buildId + systemCn + key + keyType + version.toLatin1().toHex() + minorVersion
                  + productId.toLatin1().toHex() + uuid + serverSum.toLatin1().toHex() + systemEn + tempActive
                  + systemId.toLatin1().toHex();
    }
    qDebug() << "buildId:" << buildId << "systemCn:" << systemCn << "version:" << version
             << "minorVersion:" << minorVersion << "productId:" << productId << "serverSum:" << serverSum
             << "systemEn:" << systemEn << "systemId:" << systemId << "uuid:" << LogEncryption::encryptionMessage(uuid);
    qInfo() << "tempActive:" << LogEncryption::encryptionMessage(tempActive);
    QByteArray arrdata = curData.toLocal8Bit();
    QByteArray bamd5 = QCryptographicHash::hash(arrdata, QCryptographicHash::Md5).toHex();
    qDebug() << "clientsum qstring:" << QString(bamd5);
    return QString(bamd5);
}

void LicenseKernel::calculateActiveState()
{
    Q_D(LicenseKernel);

    QDateTime curTime = QDateTime::currentDateTime();
    if (!m_activeFileData.value(7, "").isEmpty()) {
        QString strServerTime = m_activeFileData.value(7, "");
        QDateTime serverTime = QDateTime::fromString(strServerTime, QString("yyyy-MM-dd hh:mm:ss"));
        int timeTo = static_cast<int>(serverTime.secsTo(curTime));

        //如果当前时间早于服务器最后一次请求时间，则说明用户修改了本地时间
        if (timeTo < 0) {
            curTime = serverTime;
        }
    }

    QDateTime validityTime = QDateTime::fromString(m_activeFileData.value(4, ""), QString("yyyy-MM-dd hh:mm:ss"));
    // qt返回值bug，当前日期大于截止日期返回的quint64为一个很大的值，无法进行比较，转成int便于if判断
    //过期返回值为负值
    int secsTo = static_cast<int>(curTime.secsTo(validityTime));
    if (secsTo < 0) {
        // 0 day
        d->m_licenseModel.license.validityTrial = "0";

        if (d->m_licenseModel.license.authorizationState == Authorized
                || d->m_licenseModel.license.authorizationState == Expired) {
            // 已过期
            d->m_licenseModel.license.actived = Unactivated;
            d->m_licenseModel.license.authorizationState = Expired;

        } else if (d->m_licenseModel.license.authorizationState == TrialAuthorized
                   || d->m_licenseModel.license.authorizationState == TrialExpired) {
            // 试用期过期
            d->m_licenseModel.license.authorizationState = TrialExpired;

        } else {
            d->m_licenseModel.license.authorizationState = Notauthorized;
        }
    } else {
        // 未过期，计算到期时间
        if (d->m_licenseModel.license.validityEnd.isEmpty()) {
            d->m_licenseModel.license.validityTrial.clear();
        } else {
            int daysTo = static_cast<int>(curTime.daysTo(validityTime));
            // 剩余天数（试用期）
            d->m_licenseModel.license.validityTrial = QString::number(daysTo);
            // 剩余秒数（试用期)
            d->m_licenseModel.license.validityTrialSecond = curTime.secsTo(validityTime);
        }
    }
}

void LicenseKernel::resetHeartbeatTimerInterval()
{
    ConfigIniInfo configInfo;
    SystemFileOperate::readConfigIniFile(configInfo);
    if ((m_heartbeatTimer->isActive()) && (m_heartbeatTimer->interval()
                                           != configInfo.heartBeatInfo.offlineTime * minutesToMilliseconds)) {
        m_heartbeatTimer->setInterval(configInfo.heartBeatInfo.offlineTime * minutesToMilliseconds);
    }
    qDebug() << "[resetHeartbeatTimerInterval] set offline heartBeat time : "
             << configInfo.heartBeatInfo.offlineTime * minutesToMilliseconds;
}

void LicenseKernel::parseServerConf()
{
    m_currentServerUrl = QString("");
    QByteArray data = getServerConfUrl();
    QJsonDocument docObject = QJsonDocument::fromJson(data);
    QVariantMap serverConf = docObject.toVariant().toMap();
    // QVariantList options = serverConf.value(confOptions).toList();

    if (serverConf.size() < 4)
        return;

    bool bDefault = serverConf.value(confDefaultName, true).toBool();

    if (bDefault) {
        m_currentServerUrl = serverConf.value(defaultUrlName).toString();
    } else {
        m_currentServerUrl = serverConf.value(confCustomUrlName, "").toString();
    }
}

bool LicenseKernel::isKeyAndKeyTypeMatch(const QVariantMap &data)
{
    Q_D(LicenseKernel);

    QString key = data[oemkey].toString();
    QString keyType = data[serverKeyType].toString();
    if (key.isEmpty() && keyType.isEmpty() && authorizeMode() == openKeyType)
        return true;

    if (key == d->m_serverSystemInfo.key && keyType == d->m_serverSystemInfo.keyType)
        return true;

    return false;
}

void LicenseKernel::sendKmsVerifyData(int verifyFlag, int messageType, QVariantMap &data)
{
    QJsonObject varifyObj;
    QJsonObject sendObj;
    if (False == messageType) {
        varifyObj.insert("deadLine", data["deadLine"].toString());
        // offlineActiveTime:允许脱网天数
        varifyObj.insert("offlineActiveTime", data["offlineActiveTime"].toString());
    }
    varifyObj.insert("msgType", messageType);
    varifyObj.insert("flag", verifyFlag);
    sendObj.insert("data", varifyObj);
    m_kmsTcpConnect->sendData(sendObj, ActiveSignVerify);
    m_kmsNetworkTimer->start(CHECK_NETTIME);
}

void LicenseKernel::sendSystemInfo()
{
    Q_D(LicenseKernel);

    QJsonDocument jsonDocSend = QJsonDocument::fromVariant(QVariant(getUniqueUuidMap()));
    QJsonObject systemInfo;
    QJsonObject sendObj;
    systemInfo = QJsonObject(QJsonDocument::fromJson(jsonDocSend.toJson()).object());
    systemInfo.insert("system_id", m_hardWareInfo.systemId);
    systemInfo.insert("system_type", d->m_licenseModel.system.systemType);
    systemInfo.insert("build_id", d->m_serverSystemInfo.buildId);
    systemInfo.insert("minor_version", d->m_serverSystemInfo.minorVersion);
    systemInfo.insert("auth_product", d->m_serverSystemInfo.productId);
    if (authorizeMode() != openKeyType) {
        systemInfo.insert(oemkey, d->m_serverSystemInfo.key);
        systemInfo.insert(clientKeyType, d->m_serverSystemInfo.keyType);
    }
    sendObj.insert("data", systemInfo);
    m_kmsTcpConnect->sendData(sendObj, SendSystemInfo);
    m_kmsNetworkTimer->start(CHECK_NETTIME);
}

bool LicenseKernel::isKmsProductLineActive()
{
    // kms量产地址(kms://127.0.0.1:6000/abcd/123)
    QRegExp kmsProductLine(QString("^([\\w-]+\\.)+([\\w-]+):[\\d]{1,5}/[^\u4e00-\u9fa5]+$"));
    QString serverUrl = m_currentServerUrl;
    if (kmsProductLine.exactMatch(serverUrl.remove("kms://")))
        return true;

    return false;
}

// ====================================================================================================
void LicenseKernel::writeLicenseDB(const ProductInfoEntry & productInfo) {

    QString create_sql;

    QString select_sql = "SELECT count(*) FROM LicenseKey;";
    if (!m_licenseDB->IstableExist(select_sql))
    {
        create_sql = "CREATE TABLE IF NOT EXISTS LicenseKey "
                                 "(product_label VARCHAR(255), product_name VARCHAR(255),"
                                 "serial VARCHAR(255), license_b VARCHAR(255), expiration VARCHAR(255), start_time VARCHAR(255), offline_time VARCHAR(255), recheck_duration VARCHAR(255), recheck BOOLEAN, "
                                 "PRIMARY KEY(product_label));";

        if (!m_licenseDB->createTable(create_sql))
        {
            LOG(INFO) << "CREATE TABLE IF NOT EXISTS LicenseKey Failed";
            return;
        }
    }
    else
    {
        create_sql = "ALTER TABLE LicenseKey ADD COLUMN start_time;";
        select_sql = "SELECT count(*) FROM LicenseKey WHERE start_time;";

        if(!m_licenseDB->IstableExist(select_sql))
        {
            if (!m_licenseDB->createTable(create_sql))
            {
                LOG(INFO) << "alter table LicenseKey Failed";
                return;
            }
        }
    }

    QString insert_sql;
    insert_sql = QString("INSERT OR REPLACE INTO LicenseKey(product_label, product_name, serial, license_b, expiration, start_time, offline_time, recheck_duration, recheck) "
                         "VALUES (\"%1\", \"%2\", \"%3\", \"%4\", \"%5\", \"%6\", \"%7\", \"%8\", %9);")
            .arg(productInfo.product_label, productInfo.product_name, productInfo.serial, productInfo.license_b, productInfo.expiration, productInfo.start_time, productInfo.offline_time, productInfo.recheck_duration, QString("1"));

    if (!m_licenseDB->insertData(insert_sql)) {
        LOG(INFO) << "INSERT OR REPLACE INTO LicenseKey Failed";
        return;
    }
}

QList<ProductInfoEntry> LicenseKernel::getLicenseB(const QString& type) {
    // const char *errmsg;
    QList<ProductInfoEntry> list;

    QString sql = "SELECT * From LicenseKey WHERE product_label = \"" + type + "\"";
    QSqlQuery q = m_licenseDB->getData(sql);
    while(q.next()) {
        ProductInfoEntry productInfo;
        productInfo.product_label = q.value(0).toString();
        productInfo.product_name = q.value(1).toString();
        // productInfo.product_version = q.value(2).toString();

        productInfo.serial = q.value(2).toString();
        productInfo.license_b = q.value(3).toString();
        productInfo.expiration = q.value(4).toString();
        productInfo.start_time = q.value(5).toString();
        list.append(productInfo);
    }
    return list;
}

int LicenseKernel::getAuthorizationState(const QString &license_b) {
    QString expiration = get_expiration(toCStr((license_b)));

    QDateTime date_expire = QDateTime::fromString(expiration, "yyyyMMdd");
    QDateTime current_date = QDateTime::currentDateTime();

    if (current_date.daysTo(date_expire) >= 0) {
        return Authorized;
    } else {
        return Expired;
    }
}

QList<TrialInfoEntry> LicenseKernel::getTrialInfo(const QString &type) {
    QList<TrialInfoEntry> list;
    QString sql = "SELECT * From TrialInfo WHERE product_label = \"" + type + "\"";
    QSqlQuery q = m_licenseDB->getData(sql);
    while(q.next()) {
        TrialInfoEntry trial_info;
        trial_info.product_label = q.value(0).toString();
        trial_info.due_date = q.value(1).toString();
        trial_info.current_date = q.value(2).toString();
        trial_info.start_date = q.value(3).toString();
        trial_info.remain_days = q.value(4).toInt();
        list.append(trial_info);
    }
    return list;
}

bool LicenseKernel::insertTrialInfo(const TrialInfoEntry &trialInfo, bool is_exist) {
    QString create_sql = "CREATE TABLE IF NOT EXISTS TrialInfo ("
          "product_label VARCHAR(2)"
          ",due_date VARCHAR(255)"
          ",current_date VARCHAR(255)"
          ",start_date VARCHAR(255)"
          ",remain_days BIGINT(20)"
          ",PRIMARY KEY(product_label));";
    if (!m_licenseDB->createTable(create_sql)) {
        return false;
    }

    QString insert_sql;
    if (!is_exist) {
        insert_sql = QString("INSERT OR REPLACE INTO TrialInfo(product_label, due_date, current_date, start_date, remain_days) "
                                         "VALUES (\"%1\", \"%2\", \"%3\", \"%4\", %5);")
                    .arg(trialInfo.product_label, trialInfo.due_date, trialInfo.current_date, trialInfo.start_date, QString::number(trialInfo.remain_days));
    } else {
        insert_sql = QString("UPDATE TrialInfo SET current_date = \"%1\", remain_days= \"%2\" WHERE product_label = \"%3\";")
                .arg(trialInfo.current_date, QString::number(trialInfo.remain_days), trialInfo.product_label);
    }

    if (!m_licenseDB->insertData(insert_sql)) {
        return false;
    }
    return true;
}

QString LicenseKernel::getMacAddress() {
    return m_systemInfo->getMacAddress(false).toLower();
}
// =====================================

bool LicenseKernel::scanLicenseList(const QString &serverUrl, const QStringList &licenseList)
{
    for (QString list : licenseList) {
        if (list.contains(serverUrl))
            return false;
    }

    return true;
}

void LicenseKernel::parseServerTimeInfo(const QByteArray &time)
{
    Q_D(LicenseKernel);

    //服务器时间
    if (!time.isEmpty()) {
        //服务器时间--批量离线激活使用
        d->m_serverDateTime = QDateTime::fromSecsSinceEpoch(time.toLongLong());
        //服务器时间
        d->m_serverTime = QDateTime::fromSecsSinceEpoch(time.toLongLong()).toString(QString("yyyy-MM-dd hh:mm:ss"));
        //安全认证对时--服务器安全认证时间
        d->m_safeVerifyTime = QString(time);
    } else {
        //服务器时间--批量离线激活使用--取当前时间
        d->m_serverDateTime = QDateTime::currentDateTime();
        d->m_serverTime = QString("");
        d->m_safeVerifyTime = QString("");
    }
}

void LicenseKernel::parseHttpsData(int httpCode, QVariantMap headerData, const QVariantMap &info, const QString &curUrl)
{
    Q_D(LicenseKernel);

    LOG(INFO) << "httpCode:" << httpCode;
    LOG(INFO) << curUrl.toStdString();

    if (httpCode != 200) {
        QString error_info = "网络连接失败";
        emit sendActivateErrorInfo(error_info);
        return;
    }

    // ----------------------------
    QStringList url_list = curUrl.split("/");
    QString activateName = url_list[url_list.length() - 2];
    if (httpCode == 200) {
        if (info["code"].toUInt() == 2000) {
            if (activateName == "start-product-off-line-A") {               
                QVariantMap data = info["license_b_info"].toMap();
                activateWithSerialB(info["license_b"].toString().remove("-"), data["product_name"].toString(), data["serial"].toString());
            } else if (activateName == "start-product-on-line-other") {
                QVariantMap data = info["license_b_info"].toMap();
                activateWithSerialB(info["license_b"].toString().remove("-"), data["product_name"].toString(), data["serial"].toString());
            }else if(activateName == "start-product-D") {
                QVariantMap data = info["license_b_info"].toMap();
                activateWithSerialB(info["license_b"].toString().remove("-"), data["product_name"].toString(), data["serial"].toString());
            }
            else if(activateName == "start-product-C")
            {
                QVariantMap data = info["license_b_info"].toMap();
                activateWithSerialB(info["license_b"].toString().remove("-"), data["product_name"].toString(), data["serial"].toString());
            }
        } else {
            QString error_info = info["msg"].toString();
            emit sendActivateErrorInfo(error_info);
        }
    }
}

QMap<QString, QVariant> LicenseKernel::getUniqueUuidMap()
{
    QMap<QString, QVariant> major_map;
    QMap<QString, QVariant> disk_map;
    disk_map.insert("a1", m_hardWareInfo.a1);
    disk_map.insert("a2", m_hardWareInfo.a2);
    disk_map.insert("a3", m_hardWareInfo.a3);
    disk_map.insert("a4", m_hardWareInfo.a4);
    disk_map.insert("a5", m_hardWareInfo.a5);
    disk_map.insert("a6", m_hardWareInfo.a6);
    major_map.insert("disk", disk_map);
    major_map.insert("board_cpu", m_hardWareInfo.board_cpu);
    major_map.insert("mac", m_hardWareInfo.mac);
    major_map.insert("is_vm", m_hardWareInfo.is_vm);
    major_map.insert("is_raid", m_hardWareInfo.is_raid);
    return major_map;
}

void LicenseKernel::parseAuthorizationState(quint32 activeState, quint32 authState, quint32 authType)
{
    Q_D(LicenseKernel);

    if (Activated == activeState) {
        if (ServerAuthorized == authState) {
            //授权
            d->m_licenseModel.license.authorizationState = Authorized;
        }

    } else if (Unactivated == activeState) {
        if (ServerAuthorized == authState) {
            //试用期
            d->m_licenseModel.license.authorizationState = TrialAuthorized;

        } else if (ServerNotAuthorized == authState) {
            //未授权
            d->m_licenseModel.license.authorizationState = Notauthorized;

        } else if (ServerExpired == authState) {
            if (TryFullVersionActivate == authType) {
                d->m_licenseModel.license.authorizationState = TrialExpired;

            } else {
                d->m_licenseModel.license.authorizationState = Expired;
            }
        }
    }
}

bool LicenseKernel::parseVolumeActiveOfflineInfo(const QVariantMap &info)
{
    Q_D(LicenseKernel);

    QByteArray createDateSrc = QByteArray::fromHex(info["createdat"].toByteArray());
    QString createDate = RsaCrypt::privateDecrypt(createDateSrc, m_rsaKey);
    QDateTime validStartTime = QDateTime::fromSecsSinceEpoch(createDate.toLongLong());

    QByteArray fileDaysSrc = QByteArray::fromHex(info["filedays"].toByteArray());
    QString fileDays = RsaCrypt::privateDecrypt(fileDaysSrc, m_rsaKey);
    QDateTime validEndTime = QDateTime::fromSecsSinceEpoch(fileDays.toLongLong());
    //批量激活文件超过使用期限
    QDateTime curTime = QDateTime::currentDateTime();
    qint64 timeToValidStart = validStartTime.secsTo(curTime);
    qint64 timeToValidEnd = curTime.secsTo(validEndTime);
    if (timeToValidStart < 0 || timeToValidEnd <= 0) {
        Q_EMIT activeMessageChange(VolumeActiveFileTimeUnvalid);
        return false;
    }

    QByteArray currentTimeSrc = QByteArray::fromHex(info["currenttime"].toByteArray());
    QString currentTime = RsaCrypt::privateDecrypt(currentTimeSrc, m_rsaKey);
    QDateTime serverTime = QDateTime::fromSecsSinceEpoch(currentTime.toLongLong());
    qint64 timeToServer = serverTime.secsTo(curTime);

    //如果当前时间早于服务器请求时间，则说明用户修改了本地时间
    if (timeToServer < 0)
        curTime = serverTime;

    //到期时间
    QString asofat;
    //若激活码为设定起始时间和截止时间方式
    QByteArray validDaysSrc = QByteArray::fromHex(info["days"].toByteArray());
    QString validDays = RsaCrypt::privateDecrypt(validDaysSrc, m_rsaKey);
    if (validDays == "0") {
        QByteArray startAtTimeSrc = QByteArray::fromHex(info["startat"].toByteArray());
        QString startAtTime = RsaCrypt::privateDecrypt(startAtTimeSrc, m_rsaKey);
        QDateTime startTime = QDateTime::fromSecsSinceEpoch(startAtTime.toLongLong());
        QString strStartTime = startTime.toString(QString("yyyy-MM-dd"));
        qint64 timeToStart = startTime.secsTo(curTime);

        if (timeToStart < 0) {
            Q_EMIT activeMessageChange(ActiveTimeUnvalid, strStartTime);
            return false;
        }

        QByteArray asofatSrc = QByteArray::fromHex(info["asofat"].toByteArray());
        asofat = RsaCrypt::privateDecrypt(asofatSrc, m_rsaKey);
        if (asofat == "9999999999") {
            asofat = VALID_FOR_LIFE;
            m_fileValidityEnd = VALID_FOR_LIFE;
        } else {
            QDateTime expireDate = QDateTime::fromSecsSinceEpoch(asofat.toLongLong());
            qint64 timeDis = curTime.secsTo(expireDate);
            //若当前时间已超过到期时间，则批量激活码无效
            if (timeDis <= 0) {
                Q_EMIT activeMessageChange(VolumeActiveCodeExpired);
                return false;
            }
            asofat = expireDate.toString(QString("yyyy-MM-dd"));
            m_fileValidityEnd = expireDate.toString(QString("yyyy-MM-dd hh:mm:ss"));
        }
    } else {
        time_t disSec = curTime.toTime_t() + validDays.toLongLong() * 24 * 60 * 60;
        m_fileValidityEnd = QDateTime::fromSecsSinceEpoch(disSec).toString(QString("yyyy-MM-dd hh:mm:ss"));
        asofat = QDateTime::fromSecsSinceEpoch(disSec).toString(QString("yyyy-MM-dd"));
    }

    d->m_activeCode = m_volumeActiveCode;

    QByteArray activedSrc = QByteArray::fromHex(info["activatestate"].toByteArray());
    d->m_licenseModel.license.actived = RsaCrypt::privateDecrypt(activedSrc, m_rsaKey).toUInt();

    QByteArray authorizationStateSrc = QByteArray::fromHex(info["authostate"].toByteArray());
    uint authorizationState = RsaCrypt::privateDecrypt(authorizationStateSrc, m_rsaKey).toUInt();
    parseAuthorizationState(d->m_licenseModel.license.actived, authorizationState, VolumeActive);

    d->m_licenseModel.license.validityEnd = asofat;
    d->m_licenseModel.license.activeMethod = VolumeActive;

    QByteArray activeObjectSrc = QByteArray::fromHex(info["authoobject"].toByteArray());
    d->m_licenseModel.license.activeObject = RsaCrypt::privateDecrypt(activeObjectSrc, m_rsaKey);

    QByteArray activeObjectHiddenSrc = QByteArray::fromHex(info["authoobject_hidden"].toByteArray());
    d->m_licenseModel.license.authObjectHidden =
            compatiblePreviousAuthobjectDisplayed(RsaCrypt::privateDecrypt(activeObjectHiddenSrc, m_rsaKey).toUInt());
    qInfo() << "authoobject_hidden" << d->m_licenseModel.license.authObjectHidden;
    //记录激活时间
    d->m_serverTime = curTime.toString(QString("yyyy-MM-dd hh:mm:ss"));
    qDebug() << "authorizationState" << d->m_licenseModel.license.authorizationState
             << " actived :" << d->m_licenseModel.license.actived;
    return true;
}

bool LicenseKernel::parseVolumeActiveOnlineInfo(const QVariantMap &info, QDateTime curTime, QString &volumeActiveCode)
{
    QByteArray createDateSrc = QByteArray::fromHex(info["createdat"].toByteArray());
    QString createDate = RsaCrypt::privateDecrypt(createDateSrc, m_rsaKey);
    QDateTime validStratTime = QDateTime::fromSecsSinceEpoch(createDate.toLongLong());

    QByteArray fileDaysSrc = QByteArray::fromHex(info["filedays"].toByteArray());
    QString fileDays = RsaCrypt::privateDecrypt(fileDaysSrc, m_rsaKey);
    QDateTime validEndTime = QDateTime::fromSecsSinceEpoch(fileDays.toLongLong());
    //批量激活文件超过使用期限

    qint64 timeToValidStart = validStratTime.secsTo(curTime);
    qint64 timeToValidEnd = curTime.secsTo(validEndTime);
    if (timeToValidStart < 0 || timeToValidEnd <= 0) {
        Q_EMIT activeMessageChange(VolumeActiveFileTimeUnvalid);
        return false;
    }

    QByteArray currentTimeSrc = QByteArray::fromHex(info["currenttime"].toByteArray());
    QString currentTime = RsaCrypt::privateDecrypt(currentTimeSrc, m_rsaKey);
    QDateTime serverTime = QDateTime::fromSecsSinceEpoch(currentTime.toLongLong());
    qint64 timeToServer = serverTime.secsTo(curTime);

    //如果当前时间早于服务器请求时间，则说明用户修改了本地时间
    if (timeToServer < 0)
        curTime = serverTime;

    //到期时间
    QString asofat;
    //若激活码为设定起始时间和截止时间方式
    QByteArray validDaysSrc = QByteArray::fromHex(info["days"].toByteArray());
    QString validDays = RsaCrypt::privateDecrypt(validDaysSrc, m_rsaKey);
    if (validDays == "0") {
        QByteArray startAtTimeSrc = QByteArray::fromHex(info["startat"].toByteArray());
        QString startAtTime = RsaCrypt::privateDecrypt(startAtTimeSrc, m_rsaKey);
        QDateTime startTime = QDateTime::fromSecsSinceEpoch(startAtTime.toLongLong());
        QString strStartTime = startTime.toString(QString("yyyy-MM-dd"));
        qint64 timeToStart = startTime.secsTo(curTime);

        if (timeToStart < 0) {
            Q_EMIT activeMessageChange(ActiveTimeUnvalid, strStartTime);
            return false;
        }

        QByteArray asofatSrc = QByteArray::fromHex(info["asofat"].toByteArray());
        asofat = RsaCrypt::privateDecrypt(asofatSrc, m_rsaKey);
        if (asofat == "9999999999") {
            asofat = VALID_FOR_LIFE;
            m_fileValidityEnd = VALID_FOR_LIFE;
        } else {
            QDateTime expireDate = QDateTime::fromSecsSinceEpoch(asofat.toLongLong());
            qint64 timeDis = curTime.secsTo(expireDate);
            //若当前时间已超过到期时间，则批量激活码无效
            if (timeDis <= 0) {
                Q_EMIT activeMessageChange(VolumeActiveCodeExpired);
                return false;
            }
        }
    }

    return true;
}

bool LicenseKernel::checkActiveCode(const QString &key, bool isVolume)
{
    Q_D(LicenseKernel);

    if (!matchModelAndCode(key, isVolume))
        return false;

    unsigned int ret = eccVerifySerialNumber(key);
//    if (ret == ECC_ERR_PRODUCT) {
//        //激活码版本不正确
//        Q_EMIT activeMessageChange(ActiveVersionError);
//        qInfo() << "VolumeActiveKey version error:" << LogEncryption::encryptionMessage(key);
//        return false;
//    } else if (ret != ECC_SUCC) {
//        if (isVolume) {
//            qInfo() << "Invalid universal batch activation code"
//                    << "VolumeActiveKey " << LogEncryption::encryptionMessage(key);
//            Q_EMIT activeMessageChange(VolumeActiveFileUnvalid);
//        } else {
//            //激活码、激活文件、离线激活码无效
//            Q_EMIT activeMessageChange(ActiveCodeUnvalid);
//            qInfo() << "ActiveCodeUnvalid key:" << LogEncryption::encryptionMessage(key);
//        }

//        return false;
//    }
    if (!isVolume)
        d->m_activeCode = key;

    return true;
}

bool LicenseKernel::checkActiveServerAddress()
{
    if (m_currentServerUrl.startsWith("kms")) {
        qWarning() << "The current server is kms server, please check the server address";
        Q_EMIT activeMessageChange(ActiveServerUnmatch);
        return false;
    }

    return true;
}

bool LicenseKernel::checkLocalNetwork()
{
    int unionTechStatus = -1;

    if (!networkCheck.getIsEnter())
        unionTechStatus = networkCheck.getNetworkStatus(unionTechHostName);

    if ((CheckNetworkState::NetworkNormal == unionTechStatus)
        || (CheckNetworkState::NetworkConnectionTimeout == unionTechStatus) || networkCheck.getNetWorkInterfaceStatus())
        return true;

    qInfo() << "Current local network exception.";
    return false;
}

void LicenseKernel::sendStateChange()
{
    Q_D(LicenseKernel);

    if (Heartbeat == m_operationType || False == m_msgType) {
        emit heartbeatNotify();
        if (d->m_licenseModel.license.authorizationState != m_previousAuthState)
            Q_EMIT activeStateChange(m_operationType);

        m_operationType = NoOperation;
    } else {
        emit serverReturn(m_serverReturn);
        emit activeStateChange();
    }
}

void LicenseKernel::unactivatedBalloonTip()
{
    quint32 authState = authorizationState();
    //判断cmd进程是否运行
    QProcess process;
    process.start("bash", {"-c", "ps -ef |grep -w uos-activator-cmd |grep -v grep|wc -l"});
    process.waitForFinished();
    QString cmdResult = QString::fromLatin1(process.readAll()).simplified();

    if ((authState == Notauthorized || authState == Expired || authState == TrialExpired) && cmdResult == "0") {
        QString stringCmd = "wall -n Your system is not activated. Please activate as soon as possible for normal use.";
        process.start("bash", QStringList() << "-c" << stringCmd);
        process.waitForFinished();
    }
    process.close();
}

void LicenseKernel::checkKMSServer()
{
    QJsonObject sendObj;
    if (m_kmsTcpConnect->isConnected()) {
        m_kmsTcpConnect->sendData(sendObj, HeartPkgSend);
    } else {
        qInfo() << "server is disconnect";
        m_kmsHeartTimer->stop();
    }
}

void LicenseKernel::parseSystemType()
{
    Q_D(LicenseKernel);

    if (d->m_osVersionInfo.productType == "Server") {
        if (d->m_osVersionInfo.editionName == "Device")
            d->m_licenseModel.system.systemType = SystemType::ServerDevice;
        else
            d->m_licenseModel.system.systemType = SystemType::Server;
    } else if (d->m_osVersionInfo.productType == "Desktop") {
        if (d->m_osVersionInfo.editionName == "Home")
            d->m_licenseModel.system.systemType = SystemType::Personal;
        else if (d->m_osVersionInfo.editionName == "Device")
            d->m_licenseModel.system.systemType = SystemType::DesktopDevice;
        else
            d->m_licenseModel.system.systemType = SystemType::Professional;
    } else {
        //
    }
}

bool LicenseKernel::isLicenseOrLicense1Server(const QString &serverUrl)
{
    if (serverUrl.contains(LicenseServer) || serverUrl.contains(License1Server)
            || serverUrl.contains(PreLicenseServer) || serverUrl.startsWith("kms") || serverUrl.isEmpty()) {
        return true;
    }

    return false;
}

void LicenseKernel::initABRecovery()
{
    m_ABRecoveryInfo.isAbRecovery = isABRecovery();
    qDebug() << "ab-Recovery is:" << m_ABRecoveryInfo.isAbRecovery;
    if (m_ABRecoveryInfo.isAbRecovery) {
        m_ABRecoveryInfo.A.systemId = m_systemInfo->getInitSystemID(m_ABRecoveryInfo.A.name);
        m_ABRecoveryInfo.B.systemId = m_systemInfo->getInitSystemID(m_ABRecoveryInfo.B.name);
        QString systemIdAFile;
        QString systemIdBFile;
        if (SP3_LICENSE_FILE_LENGTH <= m_activeFileData.size() && !m_activeFileData.value(10, "").isEmpty()) {
            systemIdAFile = m_activeFileData.value(9, "");
            systemIdBFile = m_activeFileData.value(10, "");
            if (m_hardWareInfo.systemId == systemIdBFile && !systemIdAFile.isEmpty() && !systemIdBFile.isEmpty()) {
                m_hardWareInfo.systemId = systemIdAFile;
                qDebug() << "ab-Recovery exchange systemId. m_hardWareInfo.systemId " << m_hardWareInfo.systemId
                         << " = systemIdAFile " << systemIdAFile << " and systemIdBFile is:" << systemIdBFile;
            }
        }
    }
}

bool LicenseKernel::isABRecovery()
{
    QByteArray data = readABRecoveryJsonFile();

    if(data.isEmpty() || data.isNull())
    {
        return false;
    }

    QVariantMap abMap = parseABRecoveryJson(data);
    if (abMap.isEmpty()) {
        LOG(INFO) << "ab-RecoveryJson is empty!";
        return false;
    }

    QString currentUuid = abMap.find("Current")->toString();
    QString backUpUuid = abMap.find("Backup")->toString();
    if (currentUuid.isEmpty() || backUpUuid.isEmpty()) {
        qInfo() << "currentUuid or backUpUuid is empty!";
        return false;
    }

    getABRecoveryDiskDevice();
    for (auto &dev : m_ABRecoveryDiskDevices) {
        if (dev.diskUuid == currentUuid) {
            m_ABRecoveryInfo.A.diskUuid = dev.diskUuid;
            m_ABRecoveryInfo.A.name = dev.name;
        }
        if (dev.diskUuid == backUpUuid) {
            m_ABRecoveryInfo.B.diskUuid = dev.diskUuid;
            m_ABRecoveryInfo.B.name = dev.name;
        }
    }

    QString diskName = m_systemInfo->getRootDiskPartName();
    if (m_ABRecoveryInfo.B.diskUuid.isEmpty()) {
        qInfo() << "m_ABRecoveryInfo.B.diskUuid.isEmpty";
        return false;
    }

    if (m_ABRecoveryInfo.A.name != diskName) {
        if (m_ABRecoveryInfo.B.name == diskName) {
            QString tempName = m_ABRecoveryInfo.A.name;
            m_ABRecoveryInfo.A.name = m_ABRecoveryInfo.B.name;
            m_ABRecoveryInfo.B.name = tempName;
            QString tempUuid = m_ABRecoveryInfo.A.diskUuid;
            m_ABRecoveryInfo.A.diskUuid = m_ABRecoveryInfo.B.diskUuid;
            m_ABRecoveryInfo.B.diskUuid = tempUuid;
        } else {
            qDebug() << "m_ABRecoveryInfo.A.name " << m_ABRecoveryInfo.A.name << " m_ABRecoveryInfo.A.diskUuid "
                     << m_ABRecoveryInfo.A.diskUuid << "!= m_pSystemInfo->getRootDiskPartName() " << diskName;
            return false;
        }
    }

    QString diskUuid = m_ABRecoveryInfo.A.diskUuid;
    if (diskUuid == currentUuid || diskUuid == backUpUuid)
        return true;

    qInfo() << "m_ABRecoveryInfo.A.diskUuid is not in ab-Recovery Json file.";
    return false;
}

QByteArray LicenseKernel::readABRecoveryJsonFile()
{
    QString filePath = "/etc/deepin/ab-recovery.json";
    QFile file(filePath);

    if (!file.open(QIODevice::ReadOnly)) {
        LOG(INFO) << "ab-RecoveryJsonFile open failed!";
        return "";
    }

    QByteArray data = file.readAll();

    if (data.isEmpty()) {
        LOG(INFO) << "ab-RecoveryJsonFile empty!";
        file.close();
        return "";
    }

    file.close();
    return data;
}

QVariantMap LicenseKernel::parseABRecoveryJson(const QByteArray &data)
{
    Q_D(const LicenseKernel);
    QJsonDocument jsonDoc = d->m_licenseJson->transQBytearrayToJsonDocument(data);
    QVariantMap abMap = d->m_licenseJson->parseJsonData(jsonDoc);
    if (abMap.isEmpty()) {
        LOG(INFO) << "ab-RecoveryJson is empty!";
        return abMap;
    }

    return abMap;
}

void LicenseKernel::getABRecoveryDiskDevice()
{
    QString cmd = "lsblk -J -bno name,uuid";
    QString result;
    QProcess proc;
    proc.start("bash", QStringList() << "-c" << cmd);
    proc.waitForFinished();
    result = QString::fromLatin1(proc.readAll());
    proc.close();
    if (result.isEmpty())
        return;

    QJsonParseError err;
    QJsonDocument jsonDoc = QJsonDocument::fromJson(result.toUtf8(), &err);
    if (err.error != QJsonParseError::NoError) {
        LOG(INFO) << "ABRecoveryDiskDevice QJsonParseError";
        return;
    }

    if (jsonDoc.isNull())
        return;

    QJsonObject jsonObj = jsonDoc.object();
    QJsonArray devArray = jsonObj.value("blockdevices").toArray();

    for (int i = 0; i < devArray.size(); i++) {
        ABRecoveryDiskInfo dev = {};
        QJsonObject devObj = devArray.at(i).toObject();
        dev.diskUuid = devObj["uuid"].toString();
        dev.name = devObj["name"].toString();
        dev.root = dev.name;
        if (!dev.diskUuid.isEmpty())
            m_ABRecoveryDiskDevices.insert(dev.diskUuid, dev);
        diskDevice(devObj, dev.root, dev.name);
    }
}

void LicenseKernel::diskDevice(QJsonObject &json, QString root, QString parent)
{
    if (!json["children"].isNull()) {
        QJsonArray devArray = json.value("children").toArray();
        for (int i = 0; i < devArray.size(); i++) {
            ABRecoveryDiskInfo dev = {};
            QJsonObject devObj = devArray.at(i).toObject();
            dev.diskUuid = devObj["uuid"].toString();
            dev.name = devObj["name"].toString();
            dev.parent = parent;
            dev.root = root;
            if (!dev.diskUuid.isEmpty())
                m_ABRecoveryDiskDevices.insert(dev.diskUuid, dev);
            diskDevice(devObj, root, dev.name);
        }
    }
}

bool LicenseKernel::isNeedTrialActive()
{
    if (m_systemInfo->getIsKLU() && m_systemInfo->getKLUCID().isEmpty()) {
        emit activeMessageChange(HWKLUSystemLostCID);
        return false;
    }

    QStringList defaultServerUrls = {LicenseServer, License1Server, TeLicenseServer, TeLicense1Server};

    if (m_activeFileData.isEmpty() || m_activeFileData.value(0, "").isEmpty()) {
        if (defaultServerUrls.contains(m_currentServerUrl)) {
            return true;
        } else {
            if (m_operationType == RequestTrialActive)
                emit activeMessageChange(ActiveServerUnmatch);
        }
    } else {
        if (isLicenseFileValid()) {
            uint activedState = m_activeFileData.value(2, "0").toUInt();
            uint authorizationState = m_activeFileData.value(3, "0").toUInt();
            if (Unactivated == activedState && Notauthorized == authorizationState) {
                if (defaultServerUrls.contains(m_currentServerUrl)) {
                    return true;
                } else {
                    if (m_operationType == RequestTrialActive)
                        emit activeMessageChange(ActiveServerUnmatch);
                }
            }
        } else {
            if (m_operationType == RequestTrialActive)
                emit activeMessageChange(SystemFilesUnvalid);
            qInfo() << ".lcense.key invalid, trial active failed";
        }
    }
    return false;
}

qint64 LicenseKernel::calculateTrialRemainTime(const QStringList &activeFileData)
{
    QDateTime curTime = QDateTime::currentDateTime();
    if (!activeFileData.value(7, "").isEmpty()) {
        QString strServerTime = activeFileData.value(7, "");
        QDateTime serverTime = QDateTime::fromString(strServerTime, QString("yyyy-MM-dd hh:mm:ss"));
        int timeTo = static_cast<int>(serverTime.secsTo(curTime));

        // 如果当前时间早于服务器最后一次请求时间，则说明用户修改了本地时间
        if (timeTo < 0) {
            curTime = serverTime;
        }
    }

    QString validityEnd = activeFileData.value(4, "");
    QDateTime validityTime = QDateTime::fromString(validityEnd, QString("yyyy-MM-dd hh:mm:ss"));
    if (!validityTime.isValid()) {
        qInfo() << "file validityEnd is valid" << validityEnd;
        return 0;
    }
    // qt返回值bug，当前日期大于截止日期返回的quint64为一个很大的值，无法进行比较，转成int便于if判断, 过期返回值为负值
    qint64 validityTrialSecond = 0;
    int secsTo = static_cast<int>(curTime.secsTo(validityTime));
    qInfo() << "validityTime: " << LogEncryption::encryptionMessage(validityTime.toString());
    qInfo() << "curTime: " << LogEncryption::encryptionMessage(curTime.toString());
    qInfo() << "secsTo: " << LogEncryption::encryptionMessage(QString::number(secsTo));
    // 试用期剩余秒数
    if (secsTo > 0)
        validityTrialSecond = curTime.secsTo(validityTime);
    return validityTrialSecond;
}

QJsonObject LicenseKernel::writeTrialInfoAsServer(const QString &serverUrl, const qint64 &usedTrialTime)
{
    QJsonObject trialInfoObject;
    trialInfoObject.insert("trialtime", m_trialTimes);
    trialInfoObject.insert("systemrunningtime", m_systemRunningTime);
    trialInfoObject.insert("lastreboottime", m_lastRebootTime);
    trialInfoObject.insert("lastshutdowntime", m_lastShutDownTime);
    trialInfoObject.insert("usedtrialtime", usedTrialTime);
    trialInfoObject.insert("currentserverurl", serverUrl);
    return trialInfoObject;
}

void LicenseKernel::updateLicenseAndLicense1(QJsonArray &newTrialInfoObjectArray, const QString &serverAddress,
                                             qint64 &usedTrialTime)
{
    newTrialInfoObjectArray.push_back(writeTrialInfoAsServer(serverAddress, usedTrialTime));
    if (serverAddress == LicenseServer)
        newTrialInfoObjectArray.push_back(writeTrialInfoAsServer(License1Server, usedTrialTime));
    else if (serverAddress == License1Server)
        newTrialInfoObjectArray.push_back(writeTrialInfoAsServer(LicenseServer, usedTrialTime));
    else if (serverAddress == TeLicenseServer)
        newTrialInfoObjectArray.push_back(writeTrialInfoAsServer(TeLicense1Server, usedTrialTime));
    else if (serverAddress == TeLicense1Server)
        newTrialInfoObjectArray.push_back(writeTrialInfoAsServer(TeLicenseServer, usedTrialTime));
}

bool LicenseKernel::parsingJson(QByteArray fileData, QJsonObject &jsonObject)
{
    if (fileData.isEmpty()) {
        LOG(INFO) << "fileData isEmpty :";
        return false;
    }

    QJsonParseError jsonpe;
    QJsonDocument jsonDocument = QJsonDocument::fromJson(fileData, &jsonpe);
    if (jsonpe.error == QJsonParseError::NoError) {
        if (jsonDocument.isObject()) {
            jsonObject = jsonDocument.object();
        } else {
            LOG(INFO) << "json data is not object";
            return false;
        }
    } else {
        LOG(INFO) << "json data error :" << jsonpe.errorString();
        return false;
    }

    return true;
}

bool LicenseKernel::writeFile(QString path, QIODevice::OpenMode flags, const QByteArray &data)
{
    QFile file(path);

    if (file.open(flags)) {
        if (-1 == file.write(data)) {
            qWarning() << path << " " << file.errorString();
            file.close();
            return false;
        }

        file.close();
    } else {
        qWarning() << path << " " << file.errorString();
        return false;
    }

    return true;
}

void LicenseKernel::switchRsaKey(const QString &serverUrl)
{
    if (isLicenseOrLicense1Server(serverUrl)) {
        m_rsaKey = FORMAL_RSA_KEY;
    } else {
        m_rsaKey = TEST_RSA_KEY;
    }
}

bool LicenseKernel::isLicenseFileValid()
{
    if (m_activeFileData.size() <= SP3_LICENSE_FILE_LENGTH)
        return false;

    if (!isHardwareInfoValid() || !isProductIdValid())
        return false;

    return true;
}

QString LicenseKernel::operateMap(const QMap<QString, QVariant> &map)
{
    QMap<QString, QVariant>::const_iterator it = map.begin();
    QString tempQrcodeStr;
    while (it != map.end()) {
        tempQrcodeStr += it.key();
        tempQrcodeStr += "=";
        tempQrcodeStr += it.value().toString();
        tempQrcodeStr += "&";
        it++;
    }
    tempQrcodeStr = tempQrcodeStr.left(tempQrcodeStr.length() - 1);
    return tempQrcodeStr;
}

bool LicenseKernel::matchModelAndCode(const QString &key, const bool &isVolumeActive)
{
    if (!m_openModel && (twentyFiveSerialNumberLength > key.length())) {
        if (isVolumeActive) {
            Q_EMIT activeMessageChange(VolumeActiveFileUnvalid);
            qInfo() << "VolumeActiveCodeUnvalid key:" << LogEncryption::encryptionMessage(key);
        } else {
            Q_EMIT activeMessageChange(ActiveCodeUnvalid);
            qInfo() << "ActiveCodeUnvalid key:" << LogEncryption::encryptionMessage(key);
        }

        return false;
    }

    return true;
}

void LicenseKernel::releaseEccResources()
{
    Q_D(LicenseKernel);
//    if (d->eccInitFlag) {
//        ecc_uninit(d->m_eccCurveInst);
//        d->eccInitFlag = false;
//        d->m_eccCurveInst = Q_NULLPTR;
//    }

//    if (d->eccSerInitFlag) {
//        ecc_ser_uninit();
//        d->eccSerInitFlag = false;
//    }
}
