#ifndef LICENSEKERNEL_H
#define LICENSEKERNEL_H

#include "licensejson.h"
#include "licensemodel.h"
#include "kmstcpclient.h"
#include "licensefileoperate.h"
#include "licensefileoperate.h"
#include "networkconnectivitycheck.h"

#include <QObject>
#include <QTimer>
#include <QDir>
#include <QFile>
#include <QIODevice>
#include <QReadWriteLock>

#include "qsqlcipher.h"
// #include <license/uosdevicec.h>

#define toCStr(_s)   ((_s).isNull() ? NULL : (_s).toUtf8().data())

struct ProductInfoEntry {
    QString product_label;
    QString product_name;
    // QString product_version;
    QString serial;
    QString license_b;
    QString expiration;
    QString start_time;
    QString offline_time;
    QString recheck_duration;
    int recheck;
};

struct TrialInfoEntry {
    QString product_label;
    QString due_date;
    QString current_date;
    QString start_date;
    int remain_days;
};

class SystemInfo;
class LicenseKernelPrivate;
class LicenseKernel : public QObject
{
    Q_OBJECT
public:
    explicit LicenseKernel(QObject *parent = Q_NULLPTR);
    ~LicenseKernel();

    //
    void activateWithInlineURL(const QString& inline_url, const QString& product_lab);
    void activateWithOnlineURL(const QString& online_url, const QString& serial, const QString& product_label);
    bool activateWithSerialB(const QString& license_b, const QString &product_name, const QString &serial = "");
    bool isNetWorkOnline();
    void ImportStartProductC(const QString &kms_url, const QString &serial, const QString &sn, const QString &note = "");
    void activateWithStartProductD(const QString &url, const QString &product_lab);

    QList<ProductInfoEntry> getLicenseB(const QString& type);
    int getAuthorizationState(const QString &license_b);

    QString getSerial();    //获取硬盘序列号
    QString getSn();        //获取主板序列号

    // 将保存信息写入数据库
    void writeLicenseDB(const ProductInfoEntry& productInfo);

    QList<TrialInfoEntry> getTrialInfo(const QString &type);
    bool insertTrialInfo(const TrialInfoEntry &trialInfo, bool is_exist);
    //

    QString getMacAddress();

    quint32 isActivated() const;
    void sendQrcodeUrlAddress();
    void sendReplaceServerUnconnectMessage();

    quint32 activateStatus() const;
    QString validityTrial() const;
    QString validityEnd() const;
    quint32 authorizationState() const;
    quint32 activeResetState() const;
    quint32 activeMethod() const;
    quint32 activeMessageType() const;
    QString activeOverActionState() const;
    QString activeSign() const;
    QString activeCode() const;
    QString activeObject() const;
    quint32 sendAuthObjectHidden() const;
    int systemType() const;
    QString machineCode() const;
    QString authorizeMode() const;
    QString authorizeVersion() const;
    QString authorizeUrl() const;

    //初始化服务器配置
    // void serverConfCompatiter();
    //获取配置服务器地址
    void getServerUrlConf(QVariantMap map);
    void initServerUrlConf();
    //设置自定义激活服务器地址
    void setCustomServerUrl(const QString &customServerUrl);
    //设置默认激活服务器地址
    void setDefaultServerUrl();
    void setServerUrlInit();
    //获取服务器配置
    QByteArray getServerConfUrl();

    bool checkActiveCode(const QString &key, bool isVolume = false);
    bool checkActiveServerAddress();

    bool checkSerialB(const QString &serial_b);

    //试用期自动激活
    bool isNeedTrialActive();
    void resetHeartbeatTimerInterval();

    quint32 m_operationType;
    QString m_key;
    QString m_volumeActiveCode;
    quint32 m_networkStatus;
    QTimer *m_checkNetworkTimer;
    quint32 m_serverReturn;
    QString m_fileValidityEnd;
    qint32 m_isActiveFile;
    bool m_hasVerifyKey;
    bool m_isNetworkConnecting;
    bool m_isTimeout;
    QString m_sn;

private slots:
    //服务器返回数据
    void parseHttpsData(int httpCode, QVariantMap headerData, const QVariantMap &info, const QString &postName);

signals:
    //
    void sendActivateErrorInfo(QString error_info);
    //

    void activeStateChange(quint32 operationMseeage = NoOperation);
    void licenseSuccess(QString type);
    void activeMessageChange(quint32 activeMessageType, QString startTime = "");
    void serverActiveMessageChange(quint32 showScene, ServerMessageType code, QVariantMap errData);
    void sendQrcodeUrl(QString qrcodeUrl);
    void heartbeatNotify();
    void errorMessage(quint32 errorMessage);
    void serverReturn(quint32 serverReturn);
    void diagnoseFinishedReturn();

private slots:
    void unactivatedBalloonTip();
    void checkKMSServer();

private:
    bool executeCmd(const QString &cmd, QString &result);
    void parseSystemType();
    QString getIdentifyUrl(const QString &fullUrl);
    void addNewTokenField(QMap<QString, QVariant> &token);
    void addOpenOemField(QMap<QString, QVariant> &token);
    void addIsResetField(QMap<QString, QVariant> &token);
    void initCheckUuid();
    bool writeClientCheckNumFile();
    QString makeClientCheckSum(QString tempActive);
    bool checkOsLicense();
    bool verifyProductId();
    void parseServerConf();

    bool scanLicenseList(const QString &serverUrl, const QStringList &licenseList);
    bool isKeyAndKeyTypeMatch(const QVariantMap &data);
    void sendKmsVerifyData(int verifyFlag, int messageType, QVariantMap &data);
    void sendSystemInfo();
    bool isKmsProductLineActive();
    bool parseVolumeActiveOfflineInfo(const QVariantMap &info);
    bool parseVolumeActiveOnlineInfo(const QVariantMap &info, QDateTime serverTime, QString &volumeActiveCode);
    QByteArray activationFileDencrypt(QByteArray src);
    bool checkLocalNetwork();
    //发送激活状态改变信号
    void sendStateChange();
    QMap<QString, QVariant> getUniqueUuidMap();
    //判断激活码激活状态
    void parseAuthorizationState(quint32 activeState, quint32 authState, quint32 authType);
    //解析服务器时间
    void parseServerTimeInfo(const QByteArray &time);
    unsigned int eccVerifySerialNumber(const QString &key);
    quint32 compatiblePreviousAuthobjectDisplayed(const quint32 &authObjectHidden);

    //硬件信息校验是否有效
    bool isHardwareInfoValid();
    //产品ID校验是否有效
    bool isProductIdValid();
    //.license.key文件内容校验是否有效
    bool isLicenseFileValid();

    //初始化系统语言
    void initLanguage();
    bool isLicenseOrLicense1Server(const QString &serverUrl);
    // ab-recovery
    void initABRecovery();
    QByteArray readABRecoveryJsonFile();
    QVariantMap parseABRecoveryJson(const QByteArray &data);
    bool isABRecovery();
    void getABRecoveryDiskDevice();
    void diskDevice(QJsonObject &json, QString root, QString parent);
    //试用期自动激活
    void calculateActiveState();
    qint64 calculateTrialRemainTime(const QStringList &activeFileData);
    void updateLicenseAndLicense1(QJsonArray &newTrialInfoObjectArray, const QString &serverAddress,
                                  qint64 &usedTrialTime);
    QJsonObject writeTrialInfoAsServer(const QString &serverUrl, const qint64 &usedTrialTime);
    bool parsingJson(QByteArray fileData, QJsonObject &jsonObject);
    bool writeFile(QString path, QIODevice::OpenMode flags, const QByteArray &data);
    void switchRsaKey(const QString &serverUrl);
    QString operateMap(const QMap<QString, QVariant> &map);
    bool matchModelAndCode(const QString &key, const bool &isVolumeActive);
    //释放ECC申请资源
    void releaseEccResources();
    ABRecoveryInfo m_ABRecoveryInfo;
    QMap<QString, ABRecoveryDiskInfo> m_ABRecoveryDiskDevices;

    QScopedPointer<LicenseKernelPrivate> dd_ptr;
    QStringList m_activeFileData;
    QString m_currentServerUrl;
    Q_DECLARE_PRIVATE_D(qGetPtrHelper(dd_ptr), LicenseKernel)
    QTimer *m_unactivatedTipTimer;
    QTimer *m_kmsNetworkTimer;
    qint32 m_kmsOfflineValid;
    QTimer *m_kmsHeartTimer;
    QVariantMap m_volumeActiveinfo;
    QString m_token;
    KMSTcpClient *m_kmsTcpConnect;
    LicenseFileOperate m_licenseFileOperate;
    bool m_isNeedLogin;
    bool m_isDiagnostics;
    bool m_helthRequestNormal;
    qint64 m_kmsActiveCodeUsedTimes;
    QByteArray m_rsaKey;
    SystemInfo *m_systemInfo;
    QString m_systemID;
    QString m_vaildActiveCode;
    QString m_trueActiveCode;
    QString m_previousServerAddr;
    // 1是正常交互，2是服务端修改允许离线天数操作
    qint32 m_msgType;
    struct HardWareInfo m_hardWareInfo;
    QTimer *m_heartbeatTimer;
    QTimer *m_systemRunningTimer;
    qint64 m_systemRunningTime;
    qint64 m_lastRebootTime;
    qint64 m_lastShutDownTime;
    qint64 m_trialTimes;
    quint32 m_previousAuthState;
    QString m_previousKmsDomain;
    QString m_kmsDomainName;
    QReadWriteLock licenseLock;
    bool m_openModel;
    qint64 m_lastTime;
    NetworkConnectivityCheck networkCheck;

    // sqlite
    QSqlCipher* m_licenseDB;
};

#endif // LICENSEKERNEL_H
