#include "licenseagent.h"
#include "licenseinfo.h"
#include "licensekernel.h"

// #include <DLog>
#include <QCoreApplication>
#include <QtDBus>
#include <QDebug>
#include <iostream>

#include "easylogging++.h"
#define AGENT_LOG_PATH "/var/lib/nfs-license/license_agent_logging.conf"
INITIALIZE_EASYLOGGINGPP

#define SYSTEM

void initLog(int argc, char *argv[]) {

    START_EASYLOGGINGPP(argc, argv);
    el::Configurations conf(AGENT_LOG_PATH);
    // Reconfigure single logger
    el::Loggers::reconfigureLogger("default", conf);
    // Actually reconfigure all loggers instead
    el::Loggers::reconfigureAllLoggers(conf);
    QString hello = "hello agent";
    LOG(INFO) << hello;
}

int main(int argc, char *argv[])
{
    initLog(argc, argv);

    QTextCodec::setCodecForLocale(QTextCodec::codecForName("UTF-8"));

    QCoreApplication app(argc, argv);
    app.setApplicationName("nfs-license-agent");
#ifdef CVERSION
    app.setApplicationVersion(CVERSION);
    LOG(INFO) << "License Version:" << app.applicationVersion();
#endif

#ifdef SYSTEM
    if (!QDBusConnection::systemBus().isConnected()) {
        LOG(INFO) << "QDBusConnection::systemBus().isConnected() failed";
        return 0x0001;
    }

    if (!QDBusConnection::systemBus().registerService(SERVICE_NAME)) {
        //std::cout << QDBusConnection::systemBus().lastError().message().toStdString();
        LOG(INFO) << "registerService Error" << QDBusConnection::systemBus().lastError().message();
        return -1;
    }

    LicenseKernel licenseKernel;
    LicenseInfo licenseInfo(&licenseKernel);
    LicenseAgent licenseAgent(&licenseKernel, &licenseInfo);

    if (!QDBusConnection::systemBus().registerObject(AGENT_SERVICE_PATH, &licenseAgent, QDBusConnection::ExportAllSlots | QDBusConnection::ExportAllSignals | QDBusConnection::ExportAllProperties)) {
        LOG(INFO) << "registerObject LicenseDaemon Error" << QDBusConnection::systemBus().lastError().message();
        return -2;
    }

    if (!QDBusConnection::systemBus().registerObject(INFO_SERVICE_PATH, &licenseInfo, QDBusConnection::ExportAllSlots | QDBusConnection::ExportAllSignals | QDBusConnection::ExportAllProperties)) {
        LOG(INFO) << "registerObject LicenseInfo Error" << QDBusConnection::systemBus().lastError().message();
        return -3;
    }
#else
// ==========================================================
    if (!QDBusConnection::sessionBus().isConnected()) {
        qWarning() << "QDBusConnection::systemBus().isConnected() failed";
        return 0x0001;
    }

    if (!QDBusConnection::sessionBus().registerService(SERVICE_NAME)) {
        std::cout << QDBusConnection::sessionBus().lastError().message().toStdString();
        qWarning() << "registerService Error" << QDBusConnection::sessionBus().lastError();
        exit(0x0002);
    }

    LicenseKernel licenseKernel;
    LicenseInfo licenseInfo(&licenseKernel);
    LicenseAgent licenseAgent(&licenseKernel, &licenseInfo);

    if (!QDBusConnection::sessionBus().registerObject(AGENT_SERVICE_PATH, &licenseAgent, QDBusConnection::ExportAllSlots | QDBusConnection::ExportAllSignals | QDBusConnection::ExportAllProperties)) {
        qWarning() << "registerObject LicenseDaemon Error" << QDBusConnection::sessionBus().lastError();
        exit(0x0003);
    }

    if (!QDBusConnection::sessionBus().registerObject(INFO_SERVICE_PATH, &licenseInfo, QDBusConnection::ExportAllSlots | QDBusConnection::ExportAllSignals | QDBusConnection::ExportAllProperties)) {
        qWarning() << "registerObject LicenseInfo Error" << QDBusConnection::sessionBus().lastError();
        exit(0x0004);
    }

// ==========================================================
#endif
    app.exec();
    return 0;
}
