#include "monitornetwork.h"

#include <QDebug>

#define NETWORK_MANAGER_SERVER "org.freedesktop.NetworkManager"
#define NETWORK_MANAGER_PATH "/org/freedesktop/NetworkManager"
#define NETWORK_MANAGER_INTERFACE "org.freedesktop.NetworkManager"
#define NETWORK_MANAGER_CONNECT "org.freedesktop.NetworkManager.Connection.Active"
#define NETWORK_MANAGER_DEVICE "org.freedesktop.NetworkManager.Device"

Q_GLOBAL_STATIC(MonitorNetwork, monitorNetwork)

namespace  {
constexpr quint32 networkConnected = 70;
constexpr quint32 networkDeviceConnected = 100;
}

class MonitorNetworkPrivate
{
public:
    MonitorNetworkPrivate(MonitorNetwork *parent)
        : m_networkManager(new QDBusInterface(NETWORK_MANAGER_SERVER
                             , NETWORK_MANAGER_PATH
                             , NETWORK_MANAGER_INTERFACE
                             , QDBusConnection::systemBus()))
        ,q_ptr(parent)
    {

    }

    QScopedPointer<QDBusInterface> m_networkManager;
    MonitorNetwork *q_ptr;
    bool m_state = false;
    Q_DECLARE_PUBLIC(MonitorNetwork)
};

MonitorNetwork *MonitorNetwork::getInstance()
{
    return monitorNetwork();
}

void MonitorNetwork::currectNetworkState()
{
    Q_D(MonitorNetwork);
    d->m_state = false;
    QString primaryConnectionPath = getPrimaryConnection();
    if (!primaryConnectionPath.isEmpty() && primaryConnectionPath != "/") {
        QScopedPointer<QDBusInterface> activeConnectionsInterface (new QDBusInterface(NETWORK_MANAGER_SERVER
                                                                        , primaryConnectionPath
                                                                        , NETWORK_MANAGER_CONNECT
                                                                        , QDBusConnection::systemBus()));

        QList<QDBusObjectPath> devicesInterfacePath =
                qvariant_cast<QList<QDBusObjectPath>> (activeConnectionsInterface->property("Devices"));

        for (int index = 0; index < devicesInterfacePath.size(); ++index) {
            qInfo() << "device interface path : " << devicesInterfacePath.at(index).path();
            QString tempPath = devicesInterfacePath.at(index).path();
            QScopedPointer<QDBusInterface> deviceInterface(new QDBusInterface(NETWORK_MANAGER_SERVER
                                                                 , tempPath
                                                                 , NETWORK_MANAGER_DEVICE
                                                                 , QDBusConnection::systemBus()));
            qInfo() << "current device network state : " << deviceInterface->property("State").toUInt();
            quint32 state = deviceInterface->property("State").toUInt();

            if (state == networkDeviceConnected) {
                d->m_state = true;
                return;
            }
        }
    }

    QDBusReply<uint> reply  = d->m_networkManager->call(QDBus::AutoDetect, "state");

    if (reply.isValid() && reply.value() == networkConnected) {
        d->m_state = true;
        return;
    } else {
        qDebug() << "get network state error!";
    }
}

bool MonitorNetwork::getNetworkState()
{
    Q_D(MonitorNetwork);
    return d->m_state;
}

QString MonitorNetwork::getPrimaryConnection()
{
    Q_D(MonitorNetwork);
    QDBusObjectPath objectPath = qvariant_cast<QDBusObjectPath>
            (d->m_networkManager->property("PrimaryConnection"));
    qInfo() << "get primary connection path : " << objectPath.path();
    return objectPath.path();
}

void MonitorNetwork::onStateChanged(quint32 state)
{
    Q_D(MonitorNetwork);
    qInfo() << "***[START] current network state is changed ***: " << state;

    if (state == networkConnected) {
        d->m_state = true;
        emit networkStateChange(d->m_state);//发送信号
    }
}

MonitorNetwork::MonitorNetwork(QObject *parent)
    : QObject(parent)
    , dd_ptr(new MonitorNetworkPrivate(this))
{
    Q_D(MonitorNetwork);
    // connect(d->m_networkManager, SIGNAL(StateChanged(quint32)), this, SLOT(onStateChanged(quint32)));
    currectNetworkState();
}

MonitorNetwork::~MonitorNetwork()
{

}

