#include "qsqlcipher.h"
#include <QSqlDatabase>
#include <QSqlQuery>
#include <QSqlError>
#include <iostream>
#include <QString>
#include <QVariant>
#include <QFileInfo>
#include <QDir>

#include "easylogging++.h"

QSqlCipher::QSqlCipher(
        const QString& dbPath,
        QObject *parent)
    : QObject(parent)
    , m_dbPath(dbPath)
    , m_connectionName("licensedb")
{
    this->init();
}

QSqlCipher::~QSqlCipher()
{
    QSqlDatabase db = QSqlDatabase::database(m_connectionName, false);
    db.close();
}

bool QSqlCipher::init()
{
    QFileInfo fileInfo(m_dbPath);
    QDir dir = fileInfo.absoluteDir();
    if (!dir.exists()) {
        QString fullpath = dir.absolutePath();
        if (!dir.mkpath(fullpath)){
            LOG(INFO) << "创建目录失败:" << fullpath;
            return false;
        }
    }

    // Check that the driver exists
    if (!QSqlDatabase::isDriverAvailable("QSQLCIPHER")) {
        LOG(INFO) << "QSQLCIPHER Available False";
        return false;
    }
    // Set the database file
    m_db = QSqlDatabase::addDatabase("QSQLCIPHER", m_connectionName);
    m_db.setDatabaseName(m_dbPath);
    return true;
}

bool QSqlCipher::createTable(const QString &sql) {
    //LOG(INFO) << sql.toStdString();
    QSqlQuery q(QSqlDatabase::database(m_connectionName));
    QStringList queries;
    queries << "PRAGMA key='foobar'"
            << sql;
    for(const QString& qs : queries)
    {
        if (!q.exec(qs)) {
            LOG(INFO) << "sqlite exec error" << qs.toStdString();
            return false;
        }
    }
    return true;
}

bool QSqlCipher::insertData(const QString &sql) {
    //LOG(INFO) << sql.toStdString();
    QSqlQuery q(QSqlDatabase::database(m_connectionName));
    QStringList queries;
    queries << "PRAGMA key='foobar'";
    queries << sql;
    for(const QString& qs : queries)
    {
        if (!q.exec(qs)) {
            LOG(INFO) << "sqlite exec error" << qs.toStdString();
            return false;
        }
    }
    m_db.commit();
    return true;
}


bool QSqlCipher::IstableExist(const QString &sql) {
    //LOG(INFO) << sql.toStdString();
    QSqlQuery q(QSqlDatabase::database(m_connectionName));
    QStringList queries;
    queries << "PRAGMA key='foobar'" << sql;
    for(const QString& qs : queries)
    {
        if (!q.exec(qs)) {
            LOG(INFO) << "sqlite exec error" << qs.toStdString();
            return false;
        }
    }
    return true;
}

QSqlQuery QSqlCipher::getData(const QString &sql) {
    //LOG(INFO) << sql.toStdString();
    QSqlQuery q(QSqlDatabase::database(m_connectionName));
    QStringList queries;
    queries << "PRAGMA key='foobar'" << sql;
    for(const QString& qs : queries) {
        if (!q.exec(qs)) {
            LOG(INFO) << "sqlite exec error" << qs.toStdString();
            return q;
        }
    }
    return q;
}
