#ifndef SYSTEMDEVICE_H
#define SYSTEMDEVICE_H

#include <QProcess>
#include <QJsonObject>
#include <QMap>

#define MD5_DIGEST_LENGTH 16
#define KLUCID_FILE "/proc/bootdevice/cid"

typedef QMap<QString, QString> DiskInfo;

typedef struct
{
    bool isStartDisk = false;
    QString name;
    QString serial;
    QString type;
    QString model;
    QString mountpoint;
    QString parent;
    QString root;
} DiskDevice;

class SystemDevice
{
private:
    bool executeCmd(const QString &cmd, QString &result);
    void getDiskDevices();
    void parseChildDiskDevice(const QJsonObject &json, const QString &root, const QString &parent);
    QString diskID(const char *diskname);
    void findDiskPartName(const QString &mountPath, const QString &diskName);
    //5.1.9
    QList<DiskInfo> getPhyDisk();
    QList<DiskInfo> getPreviousSp1MountDisk();
    QString getPreviousSp1RootDisk();
    QList<DiskInfo> previousSp1ParseLsblk(const QString &lsblkOut);
    QString fillMachineIDAsMachineCode();
    //sp1(5.2.6)
    QString fillSp1MachineIDAsMachineCode();
    //sp1-update1(5.2.6.17)
    QString getDiskIdByDBus(QString &diskName);
    QString getDiskIdByLsblk(QString &parentName);
    void resetMachineId();
    QString getSp1Update1MountDisk();

public:
    QString getRootDiskPartName();
    QString getMountDiskName();
    QString getSystemBoardID();
    QString getSystemCpuID();
    QString getSystemDiskIDByIoctl();
    QString getSystemDiskIDByLsblk();
    QString generateDiskInfoBylsblk();
    QString generateDiskInfoByKluCid();
    QString getSystemInitField(const QString &diskName = "");
    //5.1.9
    QString stripValue(QString value);
    QString getPreviousSp1DiskInfo(bool &isGetRealDiskInfo);
    //sp1(5.2.6)
    QString getSp1DiskInfo(bool &isGetRealDiskInfo);
    //sp1-update1(5.2.6.17)
    QList<DiskInfo> parseLsblk(const QString &lsblkOut);
    bool isVirtualMachineSp1Update1();
    QString getSp1Update1DiskInfo(bool &isGetRealDiskInfo);

    qint64 getSystemRunningTime();
    qint64 getLastShutDownTime();
    qint64 getLastRebootTime();
    QMap<QString, DiskDevice> getSystemDiskDevices();
    bool isDiskRAID();
    bool isVirtualMachine();
    bool isKLU();
    QString readCIDFile();
    bool isUFSDiskType();

private:
    QMap<QString, DiskDevice> m_diskDevices;
    QMap<QString, QString> m_diskPartName;
};

#endif // SYSTEMDEVICE_H
