#include "systemfileoperate.h"

#include <QDebug>
#include <QFile>
#include <QSettings>

namespace  {
const QString systemOsLicensePath = "/var/nfs/os-license";
const QString systemOsVersionPath = "/etc/os-version";
// const QString systemOemVersionPath = "/etc/oem-version";
const QString licenseConfigIniPath = "/var/lib/nfs-license/licenseconfig.ini";
constexpr int defaultOfflineTime = 60;
constexpr int minOfflineTime = 10;
constexpr int maxOfflineTime = 24 * 60;
constexpr int defaultNetworkIntervalTime = 10;
constexpr int minNetworkIntervalTime = 0;
constexpr int maxNetworkIntervalTime = 24 * 60;
}

void SystemFileOperate::readServerSystemInfo(ServerSystemInfo &osLicenseInfo)
{
    //读取系统信息
    QString filePath(systemOsLicensePath);

    if (!QFile::exists(filePath)) {
        qDebug() << "uos-system non-existent: " << filePath;
        osLicenseInfo.productId.clear();
        osLicenseInfo.productName.clear();
        osLicenseInfo.instructionType.clear();
        osLicenseInfo.version.clear();
        osLicenseInfo.minorVersion.clear();
        osLicenseInfo.buildId.clear();
        osLicenseInfo.systemChinese.clear();
        osLicenseInfo.systemEnglish.clear();
        osLicenseInfo.nameVersion.clear();
        osLicenseInfo.serverSum.clear();
        osLicenseInfo.key.clear();
        osLicenseInfo.keyType.clear();
        return;
    }

    QSettings settings(filePath, QSettings::IniFormat);
    settings.setIniCodec("UTF8");
    settings.beginGroup("UOS-System");
    osLicenseInfo.productId = settings.value("product_id").toString();
    osLicenseInfo.productName = settings.value("product_name").toString();
    osLicenseInfo.instructionType = settings.value("instruction_type").toStringList().join(",");
    osLicenseInfo.version = settings.value("version").toString();
    osLicenseInfo.minorVersion = settings.value("minor_version").toString();
    osLicenseInfo.buildId = settings.value("build_id").toString();
    osLicenseInfo.systemChinese = settings.value("system_cn").toString();
    osLicenseInfo.systemEnglish = settings.value("system_en").toString();
    osLicenseInfo.nameVersion = settings.value("name_version", "").toString();
    osLicenseInfo.serverSum = settings.value("server_sum").toString();
    osLicenseInfo.key = settings.value("key").toString();
    osLicenseInfo.keyType = settings.value("key_type").toString();
    settings.endGroup();
}

void SystemFileOperate::readOsVersionInfo(OSVersionInfo &osVersionInfo)
{
    QString filePath(systemOsVersionPath);

    if (!QFile::exists(filePath)) {
        qDebug() << "uos-version non-existent: " << filePath;
        osVersionInfo.systemName.clear();
        osVersionInfo.systemNameCH.clear();
        osVersionInfo.productType.clear();
        osVersionInfo.productTypeCH.clear();
        osVersionInfo.editionName.clear();
        osVersionInfo.editionNameCH.clear();
        osVersionInfo.majorVersion.clear();
        osVersionInfo.minorVersion.clear();
        osVersionInfo.osBuild.clear();
        return;
    }

    QSettings settings(filePath, QSettings::IniFormat);
    settings.setIniCodec("UTF8");
    settings.beginGroup("Version");
    osVersionInfo.systemName = settings.value("SystemName").toString();
    osVersionInfo.systemNameCH = settings.value("SystemName[zh_CN]").toString();
    osVersionInfo.productType = settings.value("ProductType").toString();
    osVersionInfo.productTypeCH = settings.value("ProductType[zh_CN]").toString();
    osVersionInfo.editionName = settings.value("EditionName").toString();
    osVersionInfo.editionNameCH = settings.value("EditionName[zh_CN]").toString();
    osVersionInfo.majorVersion = settings.value("MajorVersion").toString();
    osVersionInfo.minorVersion = settings.value("MinorVersion").toString();
    osVersionInfo.osBuild = settings.value("OsBuild").toString();
    settings.endGroup();
}

//读配置文件函数
void SystemFileOperate::readConfigIniFile(ConfigIniInfo &dataInfo)
{
    auto setConfigValue = [] (int &dataName, const int &maxValue, const int &minValue) {
        if (dataName > maxValue)
            dataName = maxValue;
        else if (dataName < minValue)
            dataName = minValue;
    };

    QSettings settings(licenseConfigIniPath, QSettings::IniFormat);
    dataInfo.heartBeatInfo.offlineTime = settings.value("HeartBeatInfo/offlinetime", defaultOfflineTime).toInt();
    dataInfo.heartBeatInfo.networkIntervalTime = settings.value("HeartBeatInfo/networkintervaltime"
                                                                , defaultNetworkIntervalTime).toInt();
    setConfigValue(dataInfo.heartBeatInfo.offlineTime, maxOfflineTime, minOfflineTime);
    setConfigValue(dataInfo.heartBeatInfo.networkIntervalTime, maxNetworkIntervalTime, minNetworkIntervalTime);
}

void SystemFileOperate::writeConfigIniFile()
{
    QString filePath(licenseConfigIniPath);
    if (!QFile::exists(filePath)) {
        qWarning() << "licenseconfig.ini non-existent: " << filePath;
        QSettings settings(filePath, QSettings::IniFormat);
        settings.beginGroup("HeartBeatInfo");
        settings.setValue("offlinetime", defaultOfflineTime);
        settings.setValue("networkintervaltime", defaultNetworkIntervalTime);
        settings.endGroup();
    }
}
