#ifndef SYSTEMINFO_H
#define SYSTEMINFO_H

#include "licensemodel.h"
#include "sysdevice.h"

#include <QtGlobal>
#include <QString>
#include <QNetworkInterface>
#include <QList>

typedef QMap<QString, QString> DiskInfo;

class SystemDevice;
class SystemInfo : public QObject
{
public:
    SystemInfo();
    ~SystemInfo();

    //获取硬盘信息方法
    QString getUntreatedDiskInfoByDBus();
    QString getTreatedDiskInfoByDBus();

    //根据硬盘信息构造硬盘code方法
    QString createDiskCodeByDiskInfo(const QString &diskInfo);
    //构造md5加密串方法
    QString createMd5String(const QString &source);
    //获取硬盘code方法
    // A1
    QString getDiskCodeByIoctl();
    // A2
    QString getDiskCodeByIoctlOrDBusOrLsblk();
    // A3
    QString getDiskCodeByIoctlOrDBus();
    // A4
    QString getDiskInfoByAllDBus();
    // A5
    QString getDiskInfoByLsblk();
    // A6
    QString getDiskCodeByDbusAndLsblk();

    // 5.1.9(1000) machineCode
    QString getPreviousSp1MachineCode();
    // sp1(1010) machineCode
    QString getSp1MachineCode();
    // sp1-update1(1011) machineCode
    QString getSp1Update1MachineCode();

    //获取系统初始化ID方法
    QString getInitSystemID(const QString &mountPoint = "");
    QString getMacAddress(bool deleteColon = true);
    QString getSystemBoardAndCpuCode();
    int getIsRAID();
    int getIsVirtualMachine();
    bool getIsKLU();
    QString getKLUCID();
    QMap<QString, QVariant> getSystemDeviceInfoMap();
    HardWareInfo getHardWareInfo() const;
    QString getRootDiskPartName();

private:
    void initHardwareInfo();
    QString encodeDiskID(unsigned char *id);
    QString stripValue(const QString &value);
    QList<DiskInfo> parseLsblk(const QString &lsblkOut);
    QString getMountDisk();
    void idBufferMd5Update(unsigned char *id, const char *buffer);
    // 5.1.9(1000)根据硬盘信息构造硬盘code方法
    QString createPreviousSp1DiskInfo(const QString &diskInfo);
    // 5.2.6(1010)根据硬盘信息构造硬盘code方法
    QString createSp1DiskInfo(const QString &diskInfo);

private:
    SystemDevice *m_systemDevice;
    struct HardWareInfo m_hardWareInfo;
    QString m_rootDiskPartName;
};

#endif // SYSTEMINFO_H
